package ubiware.core;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import ubiware.util.saplbuilder.InvalidResourceException;
import ubiware.util.saplbuilder.Resource;

/**
 * A bindingsSet is a collection of {@link Binding} which can be iterated.
 * 
 * 23.6.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public class BindingsSet implements Iterable<Binding> {
	private List<HashMap<String, String>> bindings;

	/**
	 * Construct a bindingsSet with a list of hashmaps which contain a mapping between variable names and their values.
	 * 
	 * @param bindings
	 *            the bindings this gets initialized with
	 * @deprecated this constructor should get replaced with a no argument constructor. Bindings could then be added to the set later.
	 */
	@Deprecated
	public BindingsSet(List<HashMap<String, String>> bindings) {
		if (bindings == null) {
			throw new NullPointerException("bindings cannot be null");
		}
		this.bindings = bindings;
	}

	/*
		
		this method does not always do what it is expected to do!
		when not all bindings have the same number of variables bound, there might be differences in the variables for each binding
		this can happen by using sapl:or , sapl:Optional
		use binding.getVariables() instead for each used binding
		public Set<String> getVariables() {
			if (this.bindings.size() == 0) {
				return Collections.emptySet();
			}
			HashMap<String, String> binding = this.bindings.get(0);
			Set<String> returnValue = new HashSet<String>(binding.keySet());
			//the set has a 'mystical'variable with name "", this seems to be inherited from an implementation detail.
			//we just remove it to not expose this internal detail
			returnValue.remove("");

			return returnValue;
		}
	*/
	//public BindingsSet() {
	//	this.bindings = new Vector<HashMap<String, String>>();
	//}

	/**
	 * gives the number of elements in the set
	 */
	public int getSize() {
		return this.bindings.size();
	}

	/**
	 * Internally, bindings are ordered, this method returns the binding at a certain position
	 */
	private Binding getBinding(int counter) {
		return new BindingsSetBinding(this.bindings.get(counter));
	}

	/**
	 * implementation of the binding interface, for the iterator.
	 * 
	 * 23.6.2010
	 * 
	 * @author Michael Cochez for UBIWARE project
	 */
	private static class BindingsSetBinding implements Binding {
		/**
		 * reference to the actual binding this is encapsulating
		 */
		private HashMap<String, String> binding;

		public BindingsSetBinding(HashMap<String, String> binding) {
			this.binding = binding;
		}

		@Override
		public String get(String string) {
			return this.binding.get(string);
		}

		@Override
		public Resource getResource(String variableName) throws InvalidResourceException {
			return new Resource(this.get(variableName));
		}

		@Override
		public Set<String> getVariables() {
			Set<String> returnValue = new HashSet<String>(this.binding.keySet());
			//the set has a 'mystical'variable with name "", this seems to be inherited from an implementation detail.
			//we just remove it to not expose this internal detail
			returnValue.remove("");
			return returnValue;
		}

	}

	@Override
	public Iterator<Binding> iterator() {
		return new Iterator<Binding>() {
			private int counter = 0;

			@Override
			public boolean hasNext() {
				return this.counter < BindingsSet.this.getSize();
			}

			@Override
			public Binding next() {
				Binding binding = BindingsSet.this.getBinding(this.counter);
				this.counter++;
				return binding;
			}

			@Override
			public void remove() {
				throw new UnsupportedOperationException("A bindingsSet is immutable");
			}

		};
	}

}
