package ubiware.core;

import jade.core.behaviours.CyclicBehaviour;

import java.util.concurrent.ConcurrentLinkedQueue;

import ubiware.core.commands.UbiwareAgentCommand;

/**
 * This class implements a cyclic behavior that manages the execution of commands given to the agent.
 * 
 * 15.3.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public class CommandHandlingBehavior extends CyclicBehaviour {
	private static final long serialVersionUID = 1L;
	/**
	 * The queue of commands this behavior is responsible for.
	 */
	private ConcurrentLinkedQueue<UbiwareAgentCommand<?>> commands = new ConcurrentLinkedQueue<UbiwareAgentCommand<?>>();

	/**
	 * The agent the queue belongs to.
	 */
	private final UbiwareAgent myAgent;

	/**
	 * Creates the bahavior
	 * 
	 * @param agent
	 *            the agent this CommandHandler belongs to.
	 */
	public CommandHandlingBehavior(UbiwareAgent agent) {
		this.myAgent = agent;
	}

	/**
	 * Add a command that will be executed later.
	 * 
	 * @param command
	 *            The command which should get executed.
	 */
	public void addCommand(UbiwareAgentCommand<?> command) {
		this.commands.add(command);
		this.restart();
	}

	@Override
	public void action() {
		while (!this.commands.isEmpty()) {
			UbiwareAgentCommand<?> command = this.commands.remove();
			command.performCommand(this.myAgent);
			this.myAgent.live.action();
			this.myAgent.live.restart();
		}
		this.block();
	}
}
