package ubiware.core;

/**
 * Options for producing N3, those options are passed to {@link SaplN3} when Producing an S-APL N3 String from beliefs of the agent.
 * 
 * 23.6.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public class ProduceN3Options {

	/**
	 * Identical to ProduceN3Options(true, true, true, false)
	 */
	private ProduceN3Options() {
		//default constructor
		this(true, true, true, false);
	}

	public static ProduceN3Options defaultOptions = new ProduceN3Options();

	/**
	 * Identical to ProduceN3Options(indents, true, useNameSpaces, false)
	 * 
	 * @param indents
	 *            Should indentation be used?
	 * @param useNameSpaces
	 *            Should prefixes be used?
	 */
	public ProduceN3Options(boolean indents, boolean useNameSpaces) {
		this(indents, true, useNameSpaces, false);
	}

	/**
	 * Create a options object for SaplN3 generation
	 * 
	 * @param indents
	 *            Should indentation be used?
	 * @param includeRules
	 *            Should rules be included in the result?
	 * @param useNameSpaces
	 *            Should prefixes be used?
	 * @param cleanContent
	 *            Should certain content be cleaned?
	 */
	public ProduceN3Options(boolean indents, boolean includeRules, boolean useNameSpaces, boolean cleanContent) {
		super();
		this.indents = indents;
		this.includeRules = includeRules;
		this.useNameSpaces = useNameSpaces;
		this.cleanContent = cleanContent;
	}

	/**
	 * Should the produced S-APL N3 code use indentation (More readable, more verbose)
	 */
	public final boolean indents;
	/**
	 * Should rules be included in the output (The ones in sapl:Rule or sapl:MetaRule context? If false and the S-APL contains rules,
	 * "Contents are skipped" is added to the output instead of the rules.
	 */
	public final boolean includeRules;
	/**
	 * Should prefixes be used (true) or should the string contain the whole namespace name for every resource(false)
	 */
	public final boolean useNameSpaces;
	/**
	 * Should certain content be cleaned from the output? Check {@link SaplN3Producer} implementation for what gets removed exactly. Don't
	 * use this feature without knowing what it does!
	 */
	public final boolean cleanContent;
}
