package ubiware.core;

import java.util.HashMap;
import java.util.Map;

/**
 * Class loader to load {@link ReusableAtomicBehavior} classes.
 * 
 * @author Michael Cochez for Ubiware project
 * @since 30 Jul 2010
 */
public class RABLoader extends ClassLoader {
	private final static Map<String, String> embeddedActions = RABLoader.generateEmbeddedActionsMap();

	private static Map<String, String> generateEmbeddedActionsMap() {
		Map<String, String> theMap = new HashMap<String, String>();
		theMap.put("<#Print>", "ubiware.shared.PrintBehavior");
		theMap.put("<#PrintBeliefs>", "ubiware.shared.PrintBeliefsBehavior");
		//here other embedded actions can be added.

		return theMap;
	}

	//This cast is fine since it is checked trough reflection that the class is of that type.
	@SuppressWarnings("unchecked")
	@Override
	protected Class<? extends ReusableAtomicBehavior> findClass(String actionName) throws ClassNotFoundException {
		String className = RABLoader.embeddedActions.get(actionName);
		if (className == null) {
			if (actionName.startsWith("<#")) {
				throw new ClassNotFoundException("The embedded action '" + actionName + "' does not exist");
			}
			//it is not an embedded action => continue searching
			if (actionName.startsWith("<") && actionName.endsWith(">")) {
				actionName = actionName.substring(1, actionName.length() - 1);
			}
			int ind = actionName.indexOf("#");
			if (ind == -1) {
				ind = actionName.lastIndexOf("/");
			}
			className = actionName.substring(ind + 1);

		}
		//String actionName = name;
		//System.out.println("load" + className);
		//use the system class loader to do the loading

		Class<?> theClass = ClassLoader.getSystemClassLoader().loadClass(className);

		Class<?> superClass = theClass.getSuperclass();
		while (!ReusableAtomicBehavior.class.equals(superClass)) {
			if (superClass == null) {
				//the superclass is object or interface or ... but not an RAB
				throw new ClassNotFoundException("The classes loaded by RABLoader must be derived from ubiware.core.ReusableAtomicBehavior");
			}
			superClass = superClass.getSuperclass();
		}
		return (Class<? extends ReusableAtomicBehavior>) theClass;
	}

	//This cast is fine since only RAB's can be loaded with this
	@SuppressWarnings("unchecked")
	@Override
	public Class<ReusableAtomicBehavior> loadClass(String name) throws ClassNotFoundException {
		return (Class<ReusableAtomicBehavior>) super.loadClass(name);
	}
}
