package ubiware.core;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;

import ubiware.util.StringEscape;

public class RABUtils {

	//escapes a String
	public static String escapeString(String str) {
		return StringEscape.escape(str);
	}

	//un-escapes a String
	public static String unescapeString(String str) {
		return StringEscape.unEscape(str);
	}

	//saves result of a RAB operation.
	//saveTo can be either filename or a context container specifying a pattern with ?result variable
	public static boolean putResult(UbiwareAgent agent, String result, String saveTo) {
		if (saveTo.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
			//saving to beliefs
			HashMap<String, String> vars = new HashMap<String, String>();
			vars.put("result", result);
			agent.addBeliefs(saveTo, vars);
			return true;
		}
		//saving to file
		BufferedWriter ff = null;
		FileWriter fwrite = null;
		try {
			File datafile = new File(saveTo);
			File p = datafile.getParentFile();
			if ((p != null) && !p.exists()) {
				boolean directoryMakingSuccessFul = p.mkdirs();
				if (!directoryMakingSuccessFul) {
					System.err.println("Failed creating nessecary directories : " + p.getCanonicalPath());
					return false;
				}
			}
			fwrite = new FileWriter(saveTo, false);
			ff = new BufferedWriter(fwrite);
			ff.write(result, 0, result.length());
			ff.close();
			return true;
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		} finally {
			if (ff != null) {
				try {
					ff.close();
				} catch (IOException e) {
					// not much we can do here.
				}
			} else {
				//try closing FileWriter at least
				if (fwrite != null) {
					try {
						fwrite.close();
					} catch (IOException e) {
						// not much we can do here.
					}
				}
			}
		}
	}

}
