package ubiware.core;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import ubiware.util.SetOperation;

public class VariablesBindingManager {

	List<HashMap<String, String>> solutions;//this variable will never be null, since it is assigned in constructor or assigned a non null value
	HashSet<String> definedVars;

	HashMap<String, HashSet<LinkInfo>> forRemove; //when used in removing statements

	HashMap<String, Integer> solutionIndices;

	public VariablesBindingManager() {
		this(null);
	}

	public VariablesBindingManager(List<HashMap<String, String>> vec) {
		this.solutions = new ArrayList<HashMap<String, String>>();
		this.definedVars = new HashSet<String>();

		this.forRemove = new HashMap<String, HashSet<LinkInfo>>();
		this.solutionIndices = new HashMap<String, Integer>();

		if (vec != null) {
			Iterator<HashMap<String, String>> it = vec.iterator();
			while (it.hasNext()) {
				HashMap<String, String> b = it.next();
				this.solutions.add(new HashMap<String, String>(b));
				this.definedVars.addAll(b.keySet());
				this.definedVars.remove("");
			}
		}
	}

	//removes solutions with the indices listed
	void removeSolutions(List<Integer> vec) {
		Collections.sort(vec);
		for (int i = vec.size() - 1; i >= 0; i--) {
			int ind = vec.get(i).intValue();
			this.forRemove.remove(this.solutions.get(ind).get(""));
			this.solutions.remove(ind);
		}
	}

	//returns the number of possible solutions
	int getNumberOfSolutions() {
		return this.solutions.size();
	}

	//returns Nth of possible solutions
	HashMap<String, String> getSolution(int index) {
		if ((index > -1) && (index < this.solutions.size())) {
			return this.solutions.get(index);
		} else {
			return new HashMap<String, String>();
		}
	}

	//fixes all solutions so each has all the variables that are in definedVars
	//added ones are marked as unbound
	private void fix(List<HashMap<String, String>> vec) {
		Iterator<HashMap<String, String>> it = vec.iterator();
		while (it.hasNext()) {
			HashMap<String, String> solution = it.next();
			if (solution.size() == (this.definedVars.size() + 1)) {
				continue;
			}

			Iterator<String> var_it = this.definedVars.iterator();
			while (var_it.hasNext()) {
				String var = var_it.next();
				if (solution.get(var) == null) {
					solution.put(var, UbiwareAgent.UNBOUND_VARIABLE);
				}
			}
		}
	}

	//get a set of solutions taking set modifiers into account
	List<HashMap<String, String>> getSolutions(List<String> forAll) {
		return this.getSolutions(forAll, new ArrayList<String>(), new ArrayList<String>(), -1, 0, null);
	}

	List<HashMap<String, String>> getSolutions(List<String> forAll, String groupCountVar) {
		return this.getSolutions(forAll, new ArrayList<String>(), new ArrayList<String>(), -1, 0, groupCountVar);
	}

	List<HashMap<String, String>> getSolutions(List<String> forAll, List<String> forSome, List<String> orderBy) {
		return this.getSolutions(forAll, forSome, orderBy, -1, 0, null);
	}

	List<HashMap<String, String>> getSolutions(List<String> forAll, List<String> forSome, List<String> orderBy, int limit, int offset,
			String groupCountVar) {

		if ((this.getNumberOfSolutions() == 0) || ((forAll.size() == 0) && (forSome.size() == 0))) {
			List<HashMap<String, String>> res = new ArrayList<HashMap<String, String>>();
			if (this.getNumberOfSolutions() == 0) {
				res.add(new HashMap<String, String>());
			} else {
				if ((orderBy.size() > 0) || (offset > 0)) {
					List<HashMap<String, String>> temp = new ArrayList<HashMap<String, String>>(this.solutions);
					temp = this.rearrangeSolutions(temp, orderBy, limit, offset);
					if (temp.size() > 0) {
						res.add(temp.get(0));
					}
				} else {
					res.add(this.getSolution(0));
				}
			}
			this.fix(res);
			return res;
		}

		List<List<String>> vvv = new ArrayList<List<String>>();
		if (forSome.size() > 0) {
			for (int j = 0; j < forSome.size(); j++) {
				List<String> values = new ArrayList<String>();
				HashSet<String> hashvalues = new HashSet<String>();
				String var = forSome.get(j);
				Iterator<HashMap<String, String>> it = this.solutions.iterator();
				while (it.hasNext()) {
					String value = it.next().get(var);
					if (!hashvalues.contains(value)) {
						values.add(value);
						hashvalues.add(value);
					}
				}
				//randomizing
				List<String> randomized = new ArrayList<String>();
				int num = values.size();
				for (int k = num; k > 0; k--) {
					int ind = (int) Math.round(Math.random() * (k - 1));
					randomized.add(values.get(ind));
					values.remove(ind);
				}
				vvv.add(randomized);
			}
		}

		HashMap<HashMap<String, String>, HashMap<String, String>> result = new HashMap<HashMap<String, String>, HashMap<String, String>>();
		HashMap<HashMap<String, String>, Integer> groupCount = null;
		if (groupCountVar != null) {
			groupCount = new HashMap<HashMap<String, String>, Integer>();
		}

		Iterator<HashMap<String, String>> it = this.solutions.iterator();
		while (it.hasNext()) {
			HashMap<String, String> solution = it.next();
			boolean add = true;

			HashMap<String, String> keyMap = new HashMap<String, String>();
			for (int i = 0; i < forAll.size(); i++) {
				String var = forAll.get(i);
				keyMap.put(var, solution.get(var));
			}

			HashMap<String, String> res = result.get(keyMap);
			if (res != null) {
				if (groupCountVar != null) {
					int cnt = groupCount.get(keyMap).intValue() + 1;
					groupCount.put(keyMap, cnt);
				}

				boolean change = true;
				int sumdiff = 0;
				for (int k = 0; k < forSome.size(); k++) {
					String var = forSome.get(k);
					String this_value = solution.get(var);
					String exist_value = res.get(var);
					if (exist_value == null) {
						break;
					}
					if (this_value == null) {
						change = false;
						break;
					}
					sumdiff += vvv.get(k).indexOf(exist_value) - vvv.get(k).indexOf(this_value);
				}
				if (sumdiff <= 0) {
					add = false;
				} else {
					result.remove(keyMap);
				}
			} else if (groupCountVar != null) {
				groupCount.put(keyMap, 1);
			}

			if (add) {
				result.put(keyMap, solution);
			}
		}

		if (groupCountVar != null) {
			Iterator<HashMap<String, String>> itt = this.solutions.iterator();
			while (itt.hasNext()) {
				HashMap<String, String> solution = itt.next();
				HashMap<String, String> keyMap = new HashMap<String, String>();
				for (int i = 0; i < forAll.size(); i++) {
					String var = forAll.get(i);
					keyMap.put(var, solution.get(var));
				}
				String s = String.valueOf(groupCount.get(keyMap));
				solution.put(groupCountVar, s);
			}
			this.definedVars.add(groupCountVar);
		}

		List<HashMap<String, String>> res_vec = new ArrayList<HashMap<String, String>>(result.values());
		res_vec = this.rearrangeSolutions(res_vec, orderBy, limit, offset);
		this.fix(res_vec);
		return res_vec;
	}

	List<HashMap<String, String>> rearrangeSolutions(List<HashMap<String, String>> res_vec, List<String> orderBy, int limit, int offset) {
		if (orderBy.size() > 0) {
			SolutionComparator c = new SolutionComparator(orderBy);
			Collections.sort(res_vec, c);
		}

		if ((offset != 0) || ((limit != -1) && (limit < res_vec.size()))) {
			if (limit == -1) {
				limit = res_vec.size();
			}
			List<HashMap<String, String>> temp_vec = new ArrayList<HashMap<String, String>>();
			for (int i = offset; (i < (offset + limit)) && (i < res_vec.size()); i++) {
				temp_vec.add(res_vec.get(i));
			}
			res_vec = temp_vec;
		}
		return res_vec;
	}

	//evaluates the statement against a context container
	boolean evaluate(SemanticStatement st1, String contextID, UbiwareAgent myAgent, boolean removeMode, List<String> id_vars) {

		final List<HashMap<String, String>> newSolutions = new ArrayList<HashMap<String, String>>();
		boolean any = false; //any solutions remain?

		Iterator<HashMap<String, String>> it;

		if (this.solutions.size() > 0) {
			it = this.solutions.iterator();
		} else {
			List<HashMap<String, String>> tv = new ArrayList<HashMap<String, String>>();
			tv.add(new HashMap<String, String>());
			it = tv.iterator();
		}

		while (it.hasNext()) {
			HashMap<String, String> solution = it.next();
			String solutionID = solution.get("");
			if (solutionID == null) {
				solutionID = "";
			}
			boolean survive = false; //this solution stays?
			boolean extended = false; //this solution is extended with new variables?

			List<HashMap<String, String>> newSubSolutions = null;
			HashMap<String, String> index_newSubSolutions = null;

			SemanticStatement st1_copy = new SemanticStatement(st1);
			boolean changed[] = st1_copy.bindVars(solution);
			boolean subject_changed = changed[0];
			boolean predicate_changed = changed[1];
			boolean object_changed = changed[2];

			//check if the statement refers to a varible bound as optional, and this solution does not have it
			if (st1_copy.hasVariables()) {
				if (!subject_changed) {
					String var = st1_copy.getSubjectVariableName();
					if ((var != null) && this.definedVars.contains(var)) {
						continue;
					}
				}
				if (!predicate_changed) {
					String var = st1_copy.getPredicateVariableName();
					if ((var != null) && this.definedVars.contains(var)) {
						continue;
					}
				}
				if (!object_changed) {
					String var = st1_copy.getObjectVariableName();
					if ((var != null) && this.definedVars.contains(var)) {
						continue;
					}
				}
			}

			//check rdf:subject, rdf:predicate and rdf:object conditions
			if (st1_copy.predicate.equals("<" + UbiwareAgent.RDF_NS + "subject>")) {
				if (st1_copy.subject.startsWith(UbiwareAgent.STATEMENT_PREFIX)) {
					if (!st1_copy.object.startsWith("?")) {
						if (myAgent.beliefs.get(st1_copy.subject).subject.equals(st1_copy.object)) {
							newSolutions.add(solution);
							any = true;
							continue;
						}
					} else {
						solution.put(st1_copy.object.substring(1), myAgent.beliefs.get(st1_copy.subject).subject);
						newSolutions.add(solution);
						any = true;
						continue;
					}
					if (removeMode) {
						this.forRemove.remove(solutionID);
					}
					continue;
				} else if (st1_copy.subject.startsWith("?")) {
					id_vars.add(st1_copy.subject.substring(1));
					st1_copy = new SemanticStatement(st1_copy.object, "*", "*");
				}
			} else if (st1_copy.predicate.equals("<" + UbiwareAgent.RDF_NS + "predicate>")) {
				if (st1_copy.subject.startsWith(UbiwareAgent.STATEMENT_PREFIX)) {
					if (!st1_copy.object.startsWith("?")) {
						if (myAgent.beliefs.get(st1_copy.subject).predicate.equals(st1_copy.object)) {
							newSolutions.add(solution);
							any = true;
							continue;
						}
					} else {
						solution.put(st1_copy.object.substring(1), myAgent.beliefs.get(st1_copy.subject).predicate);
						newSolutions.add(solution);
						any = true;
						continue;
					}
					if (removeMode) {
						this.forRemove.remove(solutionID);
					}
					continue;
				} else if (st1_copy.subject.startsWith("?")) {
					id_vars.add(st1_copy.subject.substring(1));
					st1_copy = new SemanticStatement("*", st1_copy.object, "*");
				}
			} else if (st1_copy.predicate.equals("<" + UbiwareAgent.RDF_NS + "object>")) {
				if (st1_copy.subject.startsWith(UbiwareAgent.STATEMENT_PREFIX)) {
					if (!st1_copy.object.startsWith("?")) {
						if (myAgent.beliefs.get(st1_copy.subject).object.equals(st1_copy.object)) {
							newSolutions.add(solution);
							any = true;
							continue;
						}
					} else {
						solution.put(st1_copy.object.substring(1), myAgent.beliefs.get(st1_copy.subject).object);
						newSolutions.add(solution);
						any = true;
						continue;
					}
					if (removeMode) {
						this.forRemove.remove(solutionID);
					}
					continue;
				} else if (st1_copy.subject.startsWith("?")) {
					id_vars.add(st1_copy.subject.substring(1));
					st1_copy = new SemanticStatement("*", "*", st1_copy.object);
				}
			}

			//unwrap statements from "{} is true"
			else if (st1.predicate.equals(SaplN3Constants.is) && st1.object.equals(SaplN3Constants.TRUE)) {
				if (st1_copy.isSubjectContext() || st1_copy.subject.startsWith(UbiwareAgent.STATEMENT_PREFIX)) {
					List<HashMap<String, String>> vec = new ArrayList<HashMap<String, String>>();
					vec.add(solution);
					VariablesBindingManager temp_manager = new VariablesBindingManager(vec);
					if (myAgent.live.evaluate(st1_copy.subject, contextID, temp_manager, removeMode, id_vars)) { //super-recursion
						any = true;
						newSolutions.addAll(temp_manager.solutions);
						continue;
					}
					if (removeMode) {
						this.forRemove.remove(solutionID);
					}
					continue; //next solution
				} else if (st1_copy.subject.startsWith("?")) { //unbound yet variable -> link to IDs of all statements in container
					id_vars.add(st1_copy.subject.substring(1));
					st1_copy = new SemanticStatement("*", "*", "*");
				}
			}

			String bindContextID = null;
			if ((contextID.length() > 1) && contextID.startsWith("?")) {
				String value = solution.get(contextID.substring(1));
				if (value != null) {
					bindContextID = value;
				}
			} else {
				bindContextID = contextID;
			}

			if (bindContextID == null) {
				myAgent.print("Error: Wrong query: unbound yet context " + contextID);
				this.forRemove.clear();
				this.solutions.clear();
				return false;
			}

			ContextContainer context = myAgent.contexts.get(bindContextID);
			if (context == null) {
				if (removeMode) {
					this.forRemove.remove(solutionID);
				}
				continue; //not a proper solution, not a context - should not normally appear
			}

			List<String> members = null;
			boolean indexFiltered = true;

			if (st1_copy.isSubjectAny() && st1_copy.isPredicateAny() && st1_copy.isObjectAny()) {
				members = context.members;
			} else {
				members = new ArrayList<String>();
				boolean ok = true;
				HashSet<String> sub_set = null;
				HashSet<String> pred_set = null;
				HashSet<String> ob_set = null;
				if (!st1_copy.isSubjectAny()) {
					sub_set = context.index_sub.get(st1_copy.subject);
					if (sub_set == null) {
						ok = false;
					}
				}
				if (ok && !st1_copy.isPredicateAny()) {
					pred_set = context.index_pred.get(st1_copy.predicate);
					if (pred_set == null) {
						ok = false;
					}
				}
				if (ok && !st1_copy.isObjectAny()) {
					ob_set = context.index_ob.get(st1_copy.object);
					if (ob_set == null) {
						ok = false;
					}
				}

				if (ok) {
					if ((sub_set != null) && (pred_set == null) && (ob_set == null)) {
						members = new ArrayList<String>(sub_set);
					} else if ((sub_set == null) && (pred_set != null) && (ob_set == null)) {
						members = new ArrayList<String>(pred_set);
					} else if ((sub_set == null) && (pred_set == null) && (ob_set != null)) {
						members = new ArrayList<String>(ob_set);
					} else if (ob_set != null) {
						Set<String> result = SetOperation.intersect3Sets(ob_set, pred_set, sub_set);
						if (result.size() > 0) {
							members = new ArrayList<String>(result);
						}
					} else {
						Set<String> result = SetOperation.intersect3Sets(sub_set, pred_set, null);
						if (result.size() > 0) {
							members = new ArrayList<String>(result);
						}
					}
				}
			}

			int basicCount = members.size();
			if (context.statementsWithVariables != null) {
				for (int i = 0; i < context.statementsWithVariables.size(); i++) {
					String st_id = context.statementsWithVariables.get(i);
					if (!members.contains(st_id)) {
						members.add(st_id);
					}
				}
			}

			for (int iStat = 0; iStat < members.size(); iStat++) {

				String id = members.get(iStat);
				SemanticStatement st2 = myAgent.beliefs.get(id);

				if (st2 == null) {
					myAgent.print(st1_copy);
				}
				//checking equality taking universalities and variables into account
				if ((indexFiltered && (iStat < basicCount)) || st1_copy.equalsUniversal(st2)) {

					boolean ok = true;
					HashMap<String, String> new_solution = new HashMap<String, String>(solution);

					String svar = "", pvar = "", ovar = "";

					if (st1_copy.hasVariables()) {
						if (!subject_changed) {
							svar = st1_copy.getSubjectVariableName();
							if (svar != null) {
								if (svar.startsWith(UbiwareAgent.CONTEXT_PREFIX) && !st2.subject.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
									ok = false;
								} else {
									String old = new_solution.put(svar, st2.subject);
									if ((old != null) && !old.equals(st2.subject)) {
										ok = false;
									}
								}
							}
						}
						if (!predicate_changed) {
							pvar = st1_copy.getPredicateVariableName();
							if (pvar != null) {
								String old = new_solution.put(pvar, st2.predicate);
								if ((old != null) && !old.equals(st2.predicate)) {
									ok = false;
								}
							}
						}
						if (!object_changed) {
							ovar = st1_copy.getObjectVariableName();
							if (ovar != null) {
								if (ovar.startsWith(UbiwareAgent.CONTEXT_PREFIX) && !st2.isObjectContext()) {
									ok = false;
								} else {
									String old = new_solution.put(ovar, st2.object);
									if ((old != null) && !old.equals(st2.object)) {
										ok = false;
									}
								}
							}
						}
					}
					if (ok) {
						//record ID of matching statement if was asked
						if ((id_vars != null) && (id_vars.size() > 0)) {
							for (int i = 0; i < id_vars.size(); i++) {
								String id_var = id_vars.get(i);
								String old = new_solution.get(id_var);
								if (old != null) {
									if (!old.equals(id)) {
										ok = false;
									}
								} else {
									new_solution.put(id_var, id);
								}
							}
						}
						if (ok) {
							String sol_id = solutionID;
							if (new_solution.size() > 0) {
								if (newSubSolutions == null) {
									newSubSolutions = new ArrayList<HashMap<String, String>>();
									index_newSubSolutions = new HashMap<String, String>();
								}

								new_solution.remove("");

								String tempid = index_newSubSolutions.get(new_solution.toString());

								if (tempid == null) {
									Integer ind = this.solutionIndices.get(solutionID);
									if (ind == null) {
										ind = 1;
									} else {
										ind = ind + 1;
									}
									this.solutionIndices.put(solutionID, ind);
									sol_id = solutionID + "." + ind.toString();

									index_newSubSolutions.put(new_solution.toString(), sol_id);

									new_solution.put("", sol_id);
									newSubSolutions.add(new_solution);
									extended = true;
								} else {
									sol_id = tempid;
								}
							}
							survive = true;
							any = true;

							//remember ID of matching statement for potential use in gb:remove
							if (removeMode) {
								if (!(st1.subject.startsWith(UbiwareAgent.QUERY_CONTEXT_PREFIX) ^ st1.object
										.startsWith(UbiwareAgent.QUERY_CONTEXT_PREFIX))) {
									if (!solutionID.equals(sol_id) && (this.forRemove.get(sol_id) == null)) {
										HashSet<LinkInfo> t = this.forRemove.get(solutionID);
										if (t != null) {
											this.forRemove.put(sol_id, new HashSet<LinkInfo>(t));
										}
									}
									HashSet<LinkInfo> temp = this.forRemove.get(sol_id);
									if (temp != null) {
										temp.add(new LinkInfo(bindContextID, id));
									} else {
										temp = new HashSet<LinkInfo>();
										temp.add(new LinkInfo(bindContextID, id));
										this.forRemove.put(sol_id, temp);
									}
								}
							}
						}
					}

				}//equals

			}//container members cycle

			if (newSubSolutions != null) {
				newSolutions.addAll(newSubSolutions);
			}
			if (removeMode && (!survive || extended)) {
				this.forRemove.remove(solutionID);
			}

		}//solution cycle

		if (st1.hasVariables()) {
			String var = st1.getSubjectVariableName();
			if (var != null) {
				this.definedVars.add(var);
			}
			var = st1.getPredicateVariableName();
			if (var != null) {
				this.definedVars.add(var);
			}
			var = st1.getObjectVariableName();
			if (var != null) {
				this.definedVars.add(var);
			}
		}

		this.solutions = newSolutions;

		return any;
	}

	void addEmptySolution() {
		if (this.getNumberOfSolutions() == 0) {
			HashMap<String, String> solution = new HashMap<String, String>();
			solution.put("", ".0");
			this.solutions.add(solution);
		}
	}

	HashSet<LinkInfo> getForRemove() {
		HashSet<LinkInfo> result = new HashSet<LinkInfo>();
		Iterator<HashSet<LinkInfo>> it = this.forRemove.values().iterator();
		while (it.hasNext()) {
			result.addAll(it.next());
		}
		return result;
	}

	@Override
	public String toString() {
		return this.solutions.toString();
	}

	//used only in getSolutions() for sorting solutions
	private static class SolutionComparator implements Comparator<HashMap<String, String>> {
		List<String> orderBy;
		int[] direction;

		public SolutionComparator(List<String> orderBy) {
			this.orderBy = orderBy;
			this.direction = new int[orderBy.size()];
			for (int i = 0; i < orderBy.size(); i++) {
				String var = orderBy.get(i);
				if (var.startsWith(" ")) {
					this.direction[i] = -1;
					orderBy.set(i, var.substring(1));
				} else {
					this.direction[i] = 1;
				}
			}
		}

		@Override
		public int compare(HashMap<String, String> o1, HashMap<String, String> o2) {
			for (int i = 0; i < this.orderBy.size(); i++) {
				String var = this.orderBy.get(i);
				String v1 = o1.get(var);
				String v2 = o2.get(var);
				try {
					Double d1 = new Double(v1);
					Double d2 = new Double(v2);
					int r = this.direction[i] * d1.compareTo(d2);
					if (r != 0) {
						return r;
					}
				} catch (NumberFormatException e) {
					int r = this.direction[i] * v1.compareTo(v2);
					if (r != 0) {
						return r;
					}
				}
			}
			return 0;
		}
	}

}
