package ubiware.core.behaviors;

import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.IllegalParameterConfigurationException.State;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.ontology.param;

public class DFLookupBehavior extends ReusableAtomicBehavior {

	private String role;
	private String owner;
	private List<String> noneBeliefs;
	private List<String> oneBeliefs;
	private List<String> severalBeliefs;
	private List<String> addSevBeliefs;

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.role = parameters.getObligatedNonEmptyStringParameter(param.search);
		this.owner = parameters.getOptionalStringParameter(param.owner);
		this.noneBeliefs = new ArrayList<String>(parameters.getParameterValues(param.addIfNone));
		this.oneBeliefs = new ArrayList<String>(parameters.getParameterValues(param.addIfOne));
		this.severalBeliefs = new ArrayList<String>(parameters.getParameterValues(param.storeIfSeveral));
		this.addSevBeliefs = new ArrayList<String>(parameters.getParameterValues(param.addIfSeveral));

		if (this.oneBeliefs.size() == 0) {
			throw new IllegalParameterConfigurationException(param.addIfOne, "", State.missing);
		}
		if (this.severalBeliefs.size() == 0) {
			throw new IllegalParameterConfigurationException(param.storeIfSeveral, "", State.missing);
		}
	}

	@Override
	public void doAction() throws FIPAException {

		DFAgentDescription template = new DFAgentDescription();
		ServiceDescription sd = new ServiceDescription();
		sd.setName(this.role);
		if (this.owner != null) {
			sd.setOwnership(this.owner);
		}
		template.addServices(sd);
		DFAgentDescription[] result = DFService.search(this.myAgent, template);
		if (result.length == 0) {
			this.print(this.getClass().getName() + " warning: " + this.role + " was not found with Directory Facilitator!");
			for (int i = 0; i < this.noneBeliefs.size(); i++) {
				String contextID = this.noneBeliefs.get(i);
				this.moveBeliefs(contextID);
			}
		} else if (result.length == 1) {
			String name = result[0].getName().getLocalName();
			for (int i = 0; i < this.oneBeliefs.size(); i++) {
				String contextID = this.oneBeliefs.get(i);
				HashMap<String, String> vars = new HashMap<String, String>();
				vars.put("result", name);
				this.addBeliefs(contextID, vars);
			}
		} else {
			for (int r = 0; r < result.length; r++) {
				String name = result[r].getName().getLocalName();
				for (int i = 0; i < this.severalBeliefs.size(); i++) {
					String contextID = this.severalBeliefs.get(i);
					HashMap<String, String> vars = new HashMap<String, String>();
					vars.put("result", name);
					this.addBeliefs(contextID, vars);
				}
			}
			for (int i = 0; i < this.addSevBeliefs.size(); i++) {
				String contextID = this.addSevBeliefs.get(i);
				this.moveBeliefs(contextID);
			}
		}
		this.wakeAgent();

	}
}
