package ubiware.core.behaviors;

import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import ubiware.core.BehaviorStartParameters;
import ubiware.core.Binding;
import ubiware.core.BindingsSet;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.UbiwareAgent;
import ubiware.ontology.param;

public class DFRegisterBehavior extends ReusableAtomicBehavior {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private String input;
	private String owner;

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.input = parameters.getParameterValue(param.service);
		this.owner = parameters.getParameterValue(param.owner);

	}

	@Override
	public void doAction() {
		if (this.input == null) {
			this.print(this.getClass().getName() + " error: Mandatory parameter \"service\" is not given");
			this.setFailed();
			return;
		}

		DFAgentDescription dfd = new DFAgentDescription();
		dfd.setName(this.myAgent.getAID());

		String ownerValue = this.owner;
		if ((this.owner != null) && this.owner.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
			BindingsSet bindings = this.hasBeliefs(this.owner);
			if (bindings == null) {
				this.print(this.getClass().getName() + " warning: Owner specified, but no found. Using query " + this.owner);
			}
			// using just the first match
			ownerValue = bindings.iterator().next().get("owner");

		}

		if (this.input.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
			BindingsSet bindings = this.hasBeliefs(this.input);
			if (bindings == null) {
				this.print(this.getClass().getName() + " warning: No service to register is found");
				this.setFailed();
				return;
			}
			for (Binding b : bindings) {
				String value = b.get("input");
				if (value != null) {
					ServiceDescription sd = new ServiceDescription();
					if (ownerValue != null) {
						sd.setOwnership(ownerValue);
					}
					sd.setName(value);
					sd.setType("UBIWARE");
					dfd.addServices(sd);
				}
			}
		} else {
			String[] result = this.input.split("\\s");
			for (int x = 0; x < result.length; x++) {
				ServiceDescription sd = new ServiceDescription();
				if (ownerValue != null) {
					sd.setOwnership(ownerValue);
				}
				sd.setName(result[x]);
				sd.setType("UBIWARE");
				dfd.addServices(sd);
			}
		}

		try {
			DFService.deregister(this.myAgent);
		} catch (FIPAException fe) {
			//
		}
		try {
			DFService.register(this.myAgent, dfd);
		} catch (FIPAException fe) {
			throw new Error("This exception can only happen when the JAde platform malfunctions", fe);
		}
	}
}
