package ubiware.core.behaviors.creation;

import jade.core.AID;
import jade.lang.acl.ACLMessage;
import jade.wrapper.AgentContainer;
import jade.wrapper.AgentController;
import jade.wrapper.StaleProxyException;

import java.util.HashSet;
import java.util.Set;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.UbiwareAgent;
import ubiware.core.policies.PolicyChecker;
import ubiware.util.URITools;
import ubiware.util.UniqueInstanceGenerator;
import ubiware.util.saplbuilder.Resource;
import ubiware.util.saplbuilder.SaplDocument;

public abstract class AbstractAgentCreationBehavior extends ReusableAtomicBehavior {

	//private static final long serialVersionUID = 1L;
	private final static UniqueInstanceGenerator conversationIDGenerator = new UniqueInstanceGenerator("agent_creation",
			URITools.createUncheckedURI("http://ubiware.jyu.fi/2010/08/06/conversations/"));
	public static final String ZONE_MEMEBEROF = "<http://ubiware.jyu.fi/2010/08/06/zones#memberOf>";

	//	private String name;
	//	private Object[] params;
	//	private String initBeliefs;

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		//default empty implementation, concrete implementations are not obligated to override
	}

	@Override
	protected final void doAction() throws StaleProxyException {
		//creation of the agent
		Resource name = this.getAgentName();
		Set<String> scripts = this.getAgentScripts();
		Set<String> roles = this.getAgentRoles();
		// sapl string with initial beliefs.
		SaplDocument initBeliefs = this.getAgentInitialBeliefs();

		UbiwareAgent agentToStart = new UbiwareAgent(scripts, roles, initBeliefs);
		//setting policy check
		agentToStart.setPolicyChecker(this.getPolicyChecker(agentToStart));
		//registration
		//The agent might need to be registerred with certain zones, send a message to the zone agent about this.
		this.doOwnPolicyagentRegistration();
		this.registerToPolicyAgent();
		//agent start
		AgentContainer ac = this.myAgent.getContainerController();
		//try {
		AgentController actl = ac.acceptNewAgent(name.toString(), agentToStart);
		actl.start();
	}

	private void doOwnPolicyagentRegistration() {
		Set<String> zones = this.getAgentZones();
		if (zones.isEmpty()) {
			//nothing to be done
			return;
		}
		StringBuilder zoneInfoBuilder = new StringBuilder();
		boolean createDot = false;
		for (String zone : zones) {
			if (createDot) {
				zoneInfoBuilder.append(' ');
				zoneInfoBuilder.append('.');
				zoneInfoBuilder.append(' ');
			}
			zoneInfoBuilder.append(this.getAgentName());
			zoneInfoBuilder.append(' ');
			zoneInfoBuilder.append(AbstractAgentCreationBehavior.ZONE_MEMEBEROF);
			zoneInfoBuilder.append(' ');
			zoneInfoBuilder.append(zone);
			createDot = true;
		}
		this.informPolicyAgent(zoneInfoBuilder.toString(), "http://ubiware.jyu.fi/2010/08/06/zones");

	}

	protected void registerToPolicyAgent() {
		//hook to be overridden by concrete implementations.
	}

	protected abstract Set<String> getAgentZones();

	protected abstract Resource getAgentName();

	protected abstract Set<String> getAgentScripts();

	protected Set<String> getAgentRoles() {
		//by default an empty set.
		return new HashSet<String>();
	}

	protected SaplDocument getAgentInitialBeliefs() {
		return null;
	}

	//TODO, is the newAgent needed for this??
	protected abstract PolicyChecker getPolicyChecker(UbiwareAgent newAgent);

	protected final void informPolicyAgent(String content, String ontology) {
		if ((content == null) || content.equals("")) {
			throw new Error("Informing the policy agent must happen with a propper message!");
		}
		ACLMessage message = new ACLMessage(ACLMessage.INFORM);
		//TODO how to find the policy agent? DF? SHOULD THIS BE ZoneAgent???
		message.addReceiver(new AID("<http://www.ubiware.jyu.fi/infrastructure#PolicyAgent>", AID.ISLOCALNAME));
		message.setContent(content);
		message.setOntology(ontology);
		final String conversationID = AbstractAgentCreationBehavior.conversationIDGenerator.getNewID();
		message.setConversationId(conversationID);
		this.myAgent.send(message);
	}

}
