package ubiware.core.behaviors.creation;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.ontology.param;
import ubiware.util.saplbuilder.Resource;
import ubiware.util.saplbuilder.SaplDocument;

public abstract class AbstractDefaultAgentCreationBehavior extends AbstractAgentCreationBehavior {
	//private static final long serialVersionUID = 1L;

	private Resource name;
	private HashSet<String> initialScripts;
	private HashSet<String> initialRoles;
	private SaplDocument initialBeliefs;

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.name = parameters.getOptionalResourceParameter(param.name);
		if (this.name == null) {
			//use a generated name
			this.name = this.getNewAgentName();
		}
		String obsoleteScripts = parameters.getOptionalStringParameter(param.scripts);
		String obsoleteRoles = parameters.getOptionalStringParameter(param.roles);
		if ((obsoleteScripts != null) || (obsoleteRoles != null)) {
			System.err
					.println("scripts or roles are defined for agent creation but not taken into acount, use p:script and p:role instead.");
		}

		Collection<String> scripts = parameters.getParameterValues(param.script);
		Collection<String> roles = parameters.getParameterValues(param.role);
		// container ID of a container holding initial beliefs of the agent
		String containerInitBeliefs = parameters.getOptionalStringParameter(param.initBeliefs);
		if ((scripts == null) && (roles == null) && (containerInitBeliefs == null)) {
			System.err.println("an agent is started without any initial beliefs, script or roles!");
		}
		if (scripts != null) {
			//there are scripts specified, use them
			this.initialScripts = new HashSet<String>(scripts);
		} else {
			this.initialScripts = new HashSet<String>();
		}

		//add roles from parameters to initialRoles
		if (roles != null) {
			this.initialRoles = new HashSet<String>(roles);
		} else {
			this.initialRoles = new HashSet<String>();
		}

		// if parameter p:initBliefs was not specified, then containerInitBeliefs is null 
		if (containerInitBeliefs != null) {
			this.initialBeliefs = this.produceN3Document(containerInitBeliefs);
		} else {
			this.initialBeliefs = null;
		}
	}

	/**
	 * Implemented by subclasses providing a new name if needed by the behavior.
	 * 
	 * @return
	 */
	protected abstract Resource getNewAgentName();

	@Override
	protected Set<String> getAgentScripts() {
		return this.initialScripts;
	}

	@Override
	protected final Resource getAgentName() {
		return this.name;
	}

	@Override
	protected final Set<String> getAgentRoles() {
		return this.initialRoles;
	}

	@Override
	protected final SaplDocument getAgentInitialBeliefs() {
		return this.initialBeliefs;
	}
}
