package ubiware.core.behaviors.creation;

import java.util.HashSet;
import java.util.Set;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.UbiwareAgent;
import ubiware.core.policies.AIAPolicyChecker;
import ubiware.core.policies.PolicyChecker;
import ubiware.core.policies.SafeActionsPolicyChecker;
import ubiware.ontology.param;
import ubiware.util.StringEscape;
import ubiware.util.URITools;
import ubiware.util.UniqueInstanceGenerator;
import ubiware.util.saplbuilder.Resource;

public class CreateAIABehavior extends AbstractDefaultAgentCreationBehavior {

	private static final long serialVersionUID = 1L;
	private static final UniqueInstanceGenerator instanceGenerator = new UniqueInstanceGenerator("AIA",
			URITools.createUncheckedURI("http://www.ubiware.jyu.fi/agents/AIA/"));
	private String applicationName;
	//private final static UniqueInstanceGenerator conversationIDGenerator = new UniqueInstanceGenerator("aia_registration_conversation",
	//		URITools.createUncheckedURI("http://ubiware.jyu.fi/2010/08/06/aia/conversations/"));
	public static final String hasApplicationContext = "<http://www.ubiware.jyu.fi/applications#hasApplicationContext>";
	public static final String hasPersonalUserAgent = "<http://www.ubiware.jyu.fi/applications#hasPersonalUserAgent>";

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		// Let super do it first
		super.initializeRAB(parameters);
		//now get the application name
		this.applicationName = parameters.getObligatedNonEmptyStringParameter(param.application);
	}

	@Override
	protected Resource getNewAgentName() {
		return CreateAIABehavior.instanceGenerator.getNewResource();
	}

	@Override
	protected Set<String> getAgentZones() {
		Set<String> zones = new HashSet<String>();
		zones.add(AgentTypes.AIA);
		return zones;
	}

	@Override
	protected void registerToPolicyAgent() {
		//some more registration
		StringBuilder AIAInfoBuilder = new StringBuilder();
		AIAInfoBuilder.append('\"');
		AIAInfoBuilder.append(StringEscape.escape(this.getAgentName().toString()));
		AIAInfoBuilder.append('\"');
		AIAInfoBuilder.append(' ');
		AIAInfoBuilder.append(CreateAIABehavior.hasApplicationContext);
		AIAInfoBuilder.append(' ');
		AIAInfoBuilder.append(this.applicationName);

		String content = AIAInfoBuilder.toString();
		this.informPolicyAgent(content, "http://ubiware.jyu.fi/2010/08/06/zones");
	}

	@Override
	protected PolicyChecker getPolicyChecker(UbiwareAgent newAgent) {
		Resource agentName = this.getAgentName();
		return new SafeActionsPolicyChecker(agentName, new AIAPolicyChecker(agentName));
	}

}
