package ubiware.core.behaviors.creation;

import java.util.HashSet;
import java.util.Set;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.UbiwareAgent;
import ubiware.core.policies.AWAPolicyChecker;
import ubiware.core.policies.PolicyChecker;
import ubiware.core.policies.SafeActionsPolicyChecker;
import ubiware.ontology.param;
import ubiware.util.StringEscape;
import ubiware.util.URITools;
import ubiware.util.UniqueInstanceGenerator;
import ubiware.util.saplbuilder.Resource;

public class CreateAWABehavior extends AbstractDefaultAgentCreationBehavior {
	private static final long serialVersionUID = 1L;
	private static final UniqueInstanceGenerator instanceGenerator = new UniqueInstanceGenerator("AWA",
			URITools.createUncheckedURI("http://www.ubiware.jyu.fi/agents/applicationWorkerAgent/"));
	private String applicationName;
	private String pua;

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		// first, super can do its work
		super.initializeRAB(parameters);
		//own initialisation
		this.applicationName = parameters.getObligatedNonEmptyStringParameter(param.application);
		this.pua = parameters.getObligatedNonEmptyStringParameter(param.forAgent);
	}

	@Override
	protected Resource getNewAgentName() {
		return CreateAWABehavior.instanceGenerator.getNewResource();
	}

	@Override
	protected Set<String> getAgentZones() {
		Set<String> zones = new HashSet<String>();
		zones.add(AgentTypes.AWA);
		return zones;
	}

	@Override
	protected PolicyChecker getPolicyChecker(UbiwareAgent newAgent) {
		// TODO Auto-generated method stub
		Resource agentName = this.getAgentName();
		return new SafeActionsPolicyChecker(agentName, new AWAPolicyChecker(agentName, this.pua, this.applicationName));
	}

	@Override
	protected void registerToPolicyAgent() {
		//some more registration
		StringBuilder AIAInfoBuilder = new StringBuilder();
		AIAInfoBuilder.append('\"');
		AIAInfoBuilder.append(StringEscape.escape(this.getAgentName().toString()));
		AIAInfoBuilder.append('\"');
		AIAInfoBuilder.append(' ');
		AIAInfoBuilder.append(CreateAIABehavior.hasApplicationContext);
		AIAInfoBuilder.append(' ');
		AIAInfoBuilder.append(this.applicationName);

		String content = AIAInfoBuilder.toString();
		this.informPolicyAgent(content, "http://ubiware.jyu.fi/2010/08/06/zones");

		StringBuilder PersonalUserAgentInfoBuilder = new StringBuilder();
		PersonalUserAgentInfoBuilder.append('\"');
		PersonalUserAgentInfoBuilder.append(StringEscape.escape(this.getAgentName().toString()));
		PersonalUserAgentInfoBuilder.append('\"');
		PersonalUserAgentInfoBuilder.append(' ');
		PersonalUserAgentInfoBuilder.append(CreateAIABehavior.hasPersonalUserAgent);
		PersonalUserAgentInfoBuilder.append(' ');
		PersonalUserAgentInfoBuilder.append(this.pua);

		String personalUserAgentInfoContent = PersonalUserAgentInfoBuilder.toString();
		this.informPolicyAgent(personalUserAgentInfoContent, "http://ubiware.jyu.fi/2010/08/06/zones");
	}
}
