package ubiware.core.behaviors.creation;

import java.util.HashSet;
import java.util.Set;

import ubiware.core.UbiwareAgent;
import ubiware.core.policies.PolicyChecker;
import ubiware.util.StringEscape;
import ubiware.util.URITools;
import ubiware.util.UniqueInstanceGenerator;
import ubiware.util.saplbuilder.Resource;

public class CreateSlaveBehavior extends AbstractDefaultAgentCreationBehavior {
	private static final UniqueInstanceGenerator instanceGenerator = new UniqueInstanceGenerator("Slave",
			URITools.createUncheckedURI("http://www.ubiware.jyu.fi/agents/slave/"));
	private static final long serialVersionUID = 1L;

	@Override
	protected Resource getNewAgentName() {
		return CreateSlaveBehavior.instanceGenerator.getNewResource();
	}

	@Override
	protected Set<String> getAgentZones() {
		//no zones have to be added since it is inherited.
		return new HashSet<String>();
	}

	@Override
	protected PolicyChecker getPolicyChecker(UbiwareAgent newAgent) {
		//get the policy checker from the creating agent, the slave get the derived one.
		return this.myAgent.getPolicyChecker();
	}

	@Override
	protected void registerToPolicyAgent() {
		//we should tell the policy agent that this agent has this slave to allow it to send messages to it.
		this.informPolicyAgent('"' + StringEscape.escape(this.myAgent.getLocalName()) + '"' + ' ' + AgentTypes.hasSlaveagent + ' ' + '"'
				+ StringEscape.escape(this.getAgentName().toString()) + '"', "http://ubiware.jyu.fi/2010/08/06/zones");
	}
}
