package ubiware.core.commands;

import ubiware.core.UbiwareAgent;
import ubiware.util.saplbuilder.SaplDocument;

/**
 * Command that adds a belief in N3 notation to the agent.
 * 
 * @see UbiwareAgent#addBeliefsN3(String) 15.3.2010
 * @author Michael Cochez for UBIWARE project
 */
public class AddBeliefsN3Command implements UbiwareAgentCommand<Boolean> {

	private final String n3string;
	private Boolean result;

	//	/**
	//	 * Make a command with the String which will be passed to {@link UbiwareAgent#addBeliefsN3(String)} when this command gets executed.
	//	 * 
	//	 * @param n3string The String containing the S-APL which should be added to the agent's beliefs.
	//	 * @deprecated Use the version with a s-apl document instead
	//	 */
	//	@Deprecated
	//	public AddBeliefsN3Command(String n3string) {
	//		super();
	//		this.n3string = n3string;
	//	}

	/**
	 * Make a command with the document which will be passed to {@link UbiwareAgent#addBeliefsN3(String)} when this command gets executed.
	 * 
	 * @param document
	 *            The document containing the S-APL which should be added to the agent's beliefs.
	 */
	public AddBeliefsN3Command(SaplDocument document) {
		super();
		this.n3string = document.generateSapl();
	}

	@Override
	public void performCommand(UbiwareAgent agent) {
		this.result = agent.addBeliefsN3(this.n3string);
	}

	@Override
	public Boolean getResult() {
		return this.result;
	}

}
