package ubiware.core.commands;

import ubiware.core.BindingsSet;
import ubiware.core.UbiwareAgent;
import ubiware.util.saplbuilder.SaplDocument;

/**
 * Command that checks whether an agent has a belief in N3 notation. When delegated, checking the {@link #getResult()} only makes sense when
 * it's for sure that performCommand has been executed. This means : use this Command only in a {@link BlockingCommand} wrapper!
 * 
 * @see UbiwareAgent#addBeliefsN3(String) 15.3.2010
 * @author Michael Cochez for UBIWARE project
 */
public class HasBeliefsN3Command implements UbiwareAgentCommand<BindingsSet> {

	private final String n3string;
	private BindingsSet result;

	///**
	// * Create the command with a String which will be passed to {@link UbiwareAgent#hasBeliefsN3(String)} when the command gets executed.
	// * 
	// * @param n3string The String which will be used to query the agents beliefs.
	// */
	//public HasBeliefsN3Command(String n3string) {
	//	super();
	//	this.n3string = n3string;
	//}

	/**
	 * Create the command with a String which will be passed to {@link UbiwareAgent#hasBeliefsN3(String)} when the command gets executed.
	 * 
	 * @param document
	 *            The {@link SaplDocument} which will be used to query the agents beliefs.
	 */
	public HasBeliefsN3Command(SaplDocument document) {
		super();
		this.n3string = document.generateSapl();
	}

	@Override
	public void performCommand(UbiwareAgent agent) {
		this.result = agent.hasBeliefsN3(this.n3string);

	}

	@Override
	public BindingsSet getResult() {
		return this.result;
	}

}
