package ubiware.core.commands;

import ubiware.core.AgentWrapper;
import ubiware.core.UbiwareAgent;

/**
 * The extra point (compared to Runnable) is that inside the run method, the implementer has access to other methods provided by this class.
 * NEVER run this class from inside the UbiwareAgent thread (This means, not from in the thread which goes trough RAB's) This will cause a
 * deadlock! Use in a newly created thread. example:
 * 
 * <pre>
 * <code>
 * class RabRunnableExampleBehavior extends ReusableAtomicBehavior {
 * 
 * 	
 * 	protected void doAction() {
 * 		//........
 * 		new Thread(new RabRunnable(this.myAgent) {
 * 			
 * 			public void run() {
 * 				//access to the agents beliefs
 * 				this.AddBeliefsN3Blocking(":bill :hasSon :John");
 * 				//in the next line, we're sure that the beliefs have been added, since this was the blocking variety!
 * 
 * 			}
 * 		}).start();
 * 		//..........
 * 
 * 	}
 * 
 * 	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {...}
 * 
 * }
 * </pre>
 * 
 * </code>
 * 
 * 15.3.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public abstract class RabRunnable extends AgentWrapper implements Runnable {

	public RabRunnable(UbiwareAgent myAgent) {
		super(myAgent);
	}

}
