package ubiware.core.commands;

import ubiware.core.UbiwareAgent;
import ubiware.util.saplbuilder.SaplDocument;

/**
 * Command that removes a belief in N3 notation from the agent.
 * 
 * @see UbiwareAgent#removeBeliefsN3(String) 18.3.2010
 * @author Michael Cochez for UBIWARE project
 */
public class RemoveBeliefsN3Command implements UbiwareAgentCommand<Void> {

	private final String n3string;

	///**
	// * Make a command with the String which will be passed to {@link UbiwareAgent#removeBeliefsN3(String)} when this command gets executed.
	// * 
	// * @param n3string the String which contains the S-APL pattern which will be removed from the agents beliefs.
	// */
	//public RemoveBeliefsN3Command(String n3string) {
	//	super();
	//	this.n3string = n3string;
	//}

	/**
	 * Make a command with the {@link SaplDocument} which will be passed to {@link UbiwareAgent#removeBeliefsN3(String)} when this command
	 * gets executed.
	 * 
	 * @param document
	 *            the {@link SaplDocument} which contains the pattern which will be removed from the agents beliefs.
	 */
	public RemoveBeliefsN3Command(SaplDocument document) {
		super();
		this.n3string = document.generateSapl();
	}

	@Override
	public void performCommand(UbiwareAgent agent) {
		agent.removeBeliefsN3(this.n3string);

	}

	@Override
	public Void getResult() {
		//There is no return value (Void, thus null is returned)
		return null;
	}

}
