package ubiware.core.commands;

import ubiware.core.UbiwareAgent;

/**
 * The interface all commands have to implement A command is a piece of code which will be scheduled in the agents own thread. It is not
 * always needed to implement the methods directly. Use the derived classes instead.
 * 
 * 15.3.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public interface UbiwareAgentCommand<E> {
	/**
	 * The code which will get executed by the agent.
	 * 
	 * @param agent
	 *            The agent which is executing the command
	 */
	public void performCommand(UbiwareAgent agent);

	/**
	 * Get the result of the action performed. If the {@link #performCommand(UbiwareAgent)} has not been executed, the behavior of this
	 * method is undefined.
	 * 
	 * @return The result of the performed action
	 */
	public E getResult();
}
