package ubiware.core.policies;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.ParameterGenerator;
import ubiware.core.UbiwareAgent;

/**
 * A PolicyChecker which allows any action to be executed. Its performPolicyCheck method always returns {@link Status#Allowed}
 * 
 * @author Michael Cochez for Ubiware project
 * @since 29 Jul 2010
 */
public final class AllowAllPolicyChecker extends PolicyChecker {
	private static PolicyChecker instance = new AllowAllPolicyChecker();

	private AllowAllPolicyChecker() {
		// singleton	
	}

	/**
	 * get the singleton instance.
	 * 
	 * @return a PolicyChecker which allows any action.
	 */
	public static PolicyChecker getInstance() {
		return AllowAllPolicyChecker.instance;
	}

	@Override
	public Status performPolicyCheck(UbiwareAgent agent, String actionName, BehaviorStartParameters parameters,
			ParameterGenerator paramsgenerator) {
		return Status.Allowed;
	}
}
