package ubiware.core.policies;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.ParameterGenerator;
import ubiware.core.UbiwareAgent;

/**
 * A PolicyChecker which does not allow actions to be executed. Its performPolicyCheck method always returns {@link Status#DisAllowed}
 * 
 * @author Michael Cochez for Ubiware project
 * @since 29 Jul 2010
 */
public final class DenyAllPolicyChecker extends PolicyChecker {
	private static PolicyChecker instance = new DenyAllPolicyChecker();

	private DenyAllPolicyChecker() {
		// singleton	
	}

	/**
	 * get the singleton instance.
	 * 
	 * @return a PolicyChecker which denies any action.
	 */
	public static PolicyChecker getInstance() {
		return DenyAllPolicyChecker.instance;
	}

	@Override
	public Status performPolicyCheck(UbiwareAgent agent, String actionName, BehaviorStartParameters parameters,
			ParameterGenerator paramsgenerator) {
		return Status.DisAllowed;
	}
}
