package ubiware.core.policies;

import jade.core.AID;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import ubiware.core.BehaviorStartParameters;
import ubiware.core.ParameterGenerator;
import ubiware.core.SaplN3Constants;
import ubiware.core.UbiwareAgent;
import ubiware.util.URITools;
import ubiware.util.UniqueInstanceGenerator;
import ubiware.util.saplbuilder.Resource;

public abstract class PolicyAgentQueryingPolicyChecker extends PolicyChecker {

	//private final UbiwareAgent myAgent;
	private final static UniqueInstanceGenerator conversationIDGenerator = new UniqueInstanceGenerator("policy_conversation",
			URITools.createUncheckedURI("http://ubiware.jyu.fi/2010/08/06/policies/conversations/"));
	private static final long MAX_WAITING_TIME = 100000; //TODO in milliseconds - 100 seconds is the maximum waiting time

	public static final String isAllowedToDo = "<http://www.ubiware.jyu.fi/policy#isAllowedToDo>";
	private final String agentName;

	//private final Resource theSuperName;

	public PolicyAgentQueryingPolicyChecker(Resource agentName) {
		this.agentName = agentName.toString();
	}

	@Override
	public Status performPolicyCheck(UbiwareAgent agent, String actionName, BehaviorStartParameters parameters,
			ParameterGenerator paramsgenerator) {

		StringBuilder checkSaplBuilder = new StringBuilder(1000);//estimated length is based on the length of sending simple messages
		checkSaplBuilder.append('{');
		//checkSaplBuilder.append("\"");
		//String agentName = this.agentName;
		//checkSaplBuilder.append(StringEscape.escape(agentName));
		//checkSaplBuilder.append("\"");
		//this has cjanged since agent names are now resources!
		checkSaplBuilder.append(this.agentName);
		checkSaplBuilder.append(' ');
		checkSaplBuilder.append(PolicyAgentQueryingPolicyChecker.isAllowedToDo);
		checkSaplBuilder.append(' ');
		//TODO : should this be the actionName?
		checkSaplBuilder.append(actionName);
		checkSaplBuilder.append('}');
		checkSaplBuilder.append(' ');
		checkSaplBuilder.append(SaplN3Constants.configuredAs);
		checkSaplBuilder.append(' ');
		paramsgenerator.generateParametersN3(checkSaplBuilder);
		//TODO consider option:
		//		this.appendInfo(checkSaplBuilder);

		String executionSapl = checkSaplBuilder.toString();

		//SaplDocument doc = new SaplDocument();
		//Container subject = new Container(1);
		//subject.addStatement(this.theSuperName, PolicyAgentQueryingPolicyChecker.isAllowedToDo, new Literal(actionName));

		//send message
		ACLMessage message = new ACLMessage(ACLMessage.QUERY_IF);
		//TODO how to find the policy agent? DF?
		message.addReceiver(new AID("<http://www.ubiware.jyu.fi/infrastructure#PolicyAgent>", AID.ISLOCALNAME));
		message.setContent(executionSapl);
		message.setOntology("http://ubiware.jyu.fi/2010/08/06/policies");
		final String conversationID = PolicyAgentQueryingPolicyChecker.conversationIDGenerator.getNewID();
		message.setConversationId(conversationID);
		agent.send(message);

		///retrieve reply
		MessageTemplate messageTemplate = MessageTemplate.MatchConversationId(conversationID);
		ACLMessage msg = agent.blockingReceive(messageTemplate, PolicyAgentQueryingPolicyChecker.MAX_WAITING_TIME);
		if (msg == null) {
			//TODO better handling
			throw new Error("didn't get a reply in time for poliy check of " + this.agentName + "for check of " + executionSapl);
		}

		//give answer
		String content = msg.getContent();
		if (content.equals("allowed")) {
			//		System.out.println("ALLOWED " + executionSapl);
			return Status.Allowed;
		} else if (content.equals("disallowed")) {
			System.err.println("DISALLOWED " + executionSapl);
			return Status.DisAllowed;
		} else {
			//TODO better handling
			throw new Error(content);
		}
	}

	//	protected void appendInfo(StringBuilder checkSaplBuilder) {
	//overridden by subclasses willing to append more info to the sapl which gets send to the Policy Agent.
}
