package ubiware.core.policies;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.ParameterGenerator;
import ubiware.core.ProduceN3Options;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.UbiwareAgent;

/**
 * a {@link PolicyChecker} allows its user to check what the policies are saying about executing the specified action.
 * 
 * @author Michael Cochez for Ubiware project
 * @since 29 Jul 2010
 */
public abstract class PolicyChecker {

	/**
	 * Options to produce N3 for the Policy checkers. Usage of this instance is optional but prevents creating objects.
	 */
	public static final ProduceN3Options optionsForPolicyCheckN3Generation = new ProduceN3Options(false, true, false, false);

	/**
	 * Perform a policy check; check whether the given agent is allowed to execute the action specified in actionName with the given
	 * parameters.
	 * 
	 * @param agent
	 *            The agent which wants to perform the action.
	 * @param actionName
	 *            The name of the {@link ReusableAtomicBehavior} class the agent wants to execute.
	 * @param parameters
	 *            The parameters for execution.
	 * @param paramsgenerator
	 *            A generator which is able to generate a S-APL object String representing the parameters.
	 * @return The {@link Status} of the policy check.
	 */
	public abstract Status performPolicyCheck(UbiwareAgent agent, String actionName, BehaviorStartParameters parameters,
			ParameterGenerator paramsgenerator);
}
