package ubiware.core.policies;

import java.util.HashSet;
import java.util.Set;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ParameterGenerator;
import ubiware.core.UbiwareAgent;
import ubiware.ontology.param;
import ubiware.util.saplbuilder.Resource;

/**
 * To be wrapped around an actual policyChecker to speed up when safe actions are executed.
 * 
 * @author Michael Cochez for Ubiware project
 * @since 10 Aug 2010
 */
public class SafeActionsPolicyChecker extends PolicyChecker {

	private static Set<String> safeActions = SafeActionsPolicyChecker.createSafeSet();
	private static final String messageSenderBehaviorName = "ubiware.shared.MessageSenderBehavior";
	private static final String queryMessageSenderBehaviorName = "ubiware.shared.QueryMessageSenderBehavior";

	private static Set<String> createSafeSet() {
		Set<String> theSet = new HashSet<String>();
		theSet.add("ubiware.shared.MessageReceiverBehavior");
		theSet.add("ubiware.shared.PrintBehavior");
		theSet.add("ubiware.shared.PrintBeliefsBehavior");
		theSet.add(ubiware.core.behaviors.DebugBehavior.class.getName());
		return theSet;
	}

	private PolicyChecker checker;
	private String agentName;

	public SafeActionsPolicyChecker(Resource agentName, PolicyChecker actual) {
		this.agentName = agentName.toString();
		this.checker = actual;
	}

	@Override
	public Status performPolicyCheck(UbiwareAgent agent, String actionName, BehaviorStartParameters parameters,
			ParameterGenerator paramsgenerator) {
		// TODO Auto-generated method stub
		if (SafeActionsPolicyChecker.safeActions.contains(actionName)) {
			return Status.Allowed;
		}
		if (actionName.equals(SafeActionsPolicyChecker.messageSenderBehaviorName)
				|| actionName.equals(SafeActionsPolicyChecker.queryMessageSenderBehaviorName)) {

			String receieverParameter;
			try {
				receieverParameter = parameters.getObligatedStringParameter(param.receiver);
				if (receieverParameter.equals(this.agentName)) {
					//sending to self is always allowed!
					return Status.Allowed;
				}
			} catch (IllegalParameterConfigurationException e) {
				throw new Error("the receiver parameter for message sending must not be empty nor multi-valued.", e);
			}
		}
		return this.checker.performPolicyCheck(agent, actionName, parameters, paramsgenerator);
	}

}
