package ubiware.core.visualizer;

import javax.swing.tree.TreePath;

/**
 * This is a simple wrapper for the actual {@link javax.swing.JTree} class which renames some methods for clarity. Specifically, the methods
 * which are renames are those for which the meaning of "visible" is explained in JTree's description. "visible" is replaced with the
 * corresponding meaning ("displayed" or "viewable"), as explained on JTree.
 * 
 * @author Nikos Mouchtaris
 */
public class JTree extends javax.swing.JTree {
	private final static long serialVersionUID = 6;

	/**
	 * <p>
	 * Equivalent to {@link JTree#isRootVisible}.
	 * </p>
	 * 
	 * @return whatever {@link JTree#isRootVisible} returns
	 */
	public boolean isRootDisplayed() {
		return super.isRootVisible();
	}

	/**
	 * Same as {@link #isRootDisplayed}.
	 * 
	 * @return whatever {@link #isRootDisplayed} returns
	 */
	@Override
	public boolean isRootVisible() {
		return this.isRootDisplayed();
	}

	/**
	 * @param displayed
	 */
	public void setRootDisplayed(final boolean displayed) {
		super.setRootVisible(displayed);
	}

	/**
	 * Same as {@link #setRootDisplayed}.
	 * 
	 * @param displayed
	 */
	@Override
	public void setRootVisible(final boolean displayed) {
		this.setRootDisplayed(displayed);
	}

	/**
	 * @param treePath
	 */
	public void scrollPathToDisplayed(final TreePath treePath) {
		super.scrollPathToVisible(treePath);
	}

	/**
	 * Same as {@link #scrollPathToDisplayed}.
	 * 
	 * @param treePath
	 */
	@Override
	public void scrollPathToVisible(final TreePath treePath) {
		this.scrollPathToDisplayed(treePath);
	}

	/**
	 * @param row
	 */
	public void scrollRowToDisplayed(final int row) {
		super.scrollRowToVisible(row);
	}

	/**
	 * Same as {@link #scrollRowToDisplayed}.
	 * 
	 * @param row
	 */
	@Override
	public void scrollRowToVisible(final int row) {
		this.scrollRowToDisplayed(row);
	}

	/**
	 * <p>
	 * Equivalent to {@link JTree#getVisibleRowCount}.
	 * </p>
	 * 
	 * @return whatever {@link JTree#getVisibleRowCount} returns
	 */
	public int getDisplayedRowCount() {
		return super.getVisibleRowCount();
	}

	/**
	 * Same as {@link #getDisplayedRowCount}.
	 * 
	 * @return whatever {@link #getDisplayedRowCount} returns
	 */
	@Override
	public int getVisibleRowCount() {
		return this.getDisplayedRowCount();
	}

	/**
	 * @param count
	 */
	public void setDisplayedRowCount(final int count) {
		super.setVisibleRowCount(count);
	}

	/**
	 * Same as {@link #setDisplayedRowCount}.
	 * 
	 * @param count
	 */
	@Override
	public void setVisibleRowCount(final int count) {
		this.setDisplayedRowCount(count);
	}

	/**
	 * <p>
	 * Equivalent to {@link JTree#isVisible}.
	 * </p>
	 * 
	 * @param treePath
	 *            whatever it means in {@link JTree#isVisible}
	 * @return whatever {@link JTree#isVisible} returns
	 */
	public boolean isViewable(final TreePath treePath) {
		return super.isVisible(treePath);
	}

	/**
	 * Same as {@link #isViewable}.
	 * 
	 * @param tp
	 *            whatever it means in {@link #isViewable}
	 * @return whatever it returns in {@link #isViewable}
	 */
	@Override
	public boolean isVisible(final TreePath tp) {
		return this.isViewable(tp);
	}

	/**
	 * <p>
	 * Equivalent to {@link JTree#makeVisible}.
	 * </p>
	 * 
	 * @param tp
	 *            whatever it means in {@link JTree#makeVisible}
	 */
	public void makeViewable(final TreePath tp) {
		super.makeVisible(tp);
	}

	/**
	 * Same as {@link #makeViewable}.
	 * 
	 * @param tp
	 */
	@Override
	public void makeVisible(final TreePath tp) {
		this.makeViewable(tp);
	}
}
