package ubiware.core.visualizer;

import java.awt.Image;
import java.awt.Toolkit;
import java.net.URL;
import java.util.logging.Logger;

/**
 * Provides access to resources used by the visualiser.
 * 
 * @author Nikos Mouchtaris
 */
public final class ResourceManager {
	private static Image Ferundio = null, Asundio = null, Container = null, Belief = null, Subject = null, Predicate = null, Object = null,
			SelectedContainer = null, SelectedBelief = null, SelectedSubject = null, SelectedPredicate = null, SelectedObject = null;
	private static final String[] SemanticStatementNames = { "subject.png", "predicate.png", "object.png", "subject_selected.png",
			"predicate_selected.png", "object_selected.png" };

	private static Image createAsundio(final Toolkit toolkit) {
		return ResourceManager.createAskiryf(toolkit, "asundio.jpg", ResourceManager.Asundio_data);
	}

	private static Image createFerundio(final Toolkit toolkit) {
		return ResourceManager.createAskiryf(toolkit, "ferundio.png", ResourceManager.Ferundio_data);
	}

	private static Image createContainer(final Toolkit toolkit) {
		return ResourceManager.createImage(toolkit, VisualizerPrototype.class, "container.png", ResourceManager.Container_data);
	}

	private static Image createBelief(final Toolkit toolkit) {
		return ResourceManager.createImage(toolkit, VisualizerPrototype.class, "belief.png", ResourceManager.Belief_data);
	}

	private static Image createSelectedContainer(final Toolkit toolkit) {
		return ResourceManager.createImage(toolkit, VisualizerPrototype.class, "container_selected.png",
				ResourceManager.SelectedContainer_data);
	}

	private static Image createSelectedBelief(final Toolkit toolkit) {
		return ResourceManager.createImage(toolkit, VisualizerPrototype.class, "belief_selected.png", ResourceManager.SelectedBelief_data);
	}

	private static Image createSubject(final Toolkit toolkit) {
		return ResourceManager.createSemanticStatement(toolkit, 0);
	}

	private static Image createPredicate(final Toolkit toolkit) {
		return ResourceManager.createSemanticStatement(toolkit, 1);
	}

	private static Image createObject(final Toolkit toolkit) {
		return ResourceManager.createSemanticStatement(toolkit, 2);
	}

	private static Image createSelectedSubject(final Toolkit toolkit) {
		return ResourceManager.createSemanticStatement(toolkit, 3);
	}

	private static Image createSelectedPredicate(final Toolkit toolkit) {
		return ResourceManager.createSemanticStatement(toolkit, 4);
	}

	private static Image createSelectedObject(final Toolkit toolkit) {
		return ResourceManager.createSemanticStatement(toolkit, 5);
	}

	private static Image createSemanticStatement(final Toolkit toolkit, final int index) {
		return ResourceManager.createImage(toolkit, SAPLTreeCellRenderer.class, ResourceManager.SemanticStatementNames[index],
				ResourceManager.SemanticStatementData[index]);
	}

	private static Image createImage(final Toolkit toolkit, final Class<?> klass, final String resource_basename, final byte[] image_bytes) {
		final URL resource_url = klass.getResource(resource_basename);
		if (resource_url != null) {
			return toolkit.createImage(resource_url);
		} else {
			final Logger logger = Logger.getLogger(ResourceManager.class.getCanonicalName());
			logger.warning(resource_basename + " created from static data");
			return toolkit.createImage(image_bytes);
		}
	}

	private static Image createAskiryf(final Toolkit toolkit, final String resource_name, final byte[] askyrif_data) {
		//disabled the following statement because the logger was never used anywhere.
		//final Logger logger = Logger.getLogger(ResourceManager.class.getCanonicalName());

		// try to get it from some system resource:
		final URL askiryf_url = SAPLTreeCellRenderer.class.getClassLoader().getResource(resource_name);
		// try to load image data, either by resource or if not found, use
		// hardcoded image:
		if (askiryf_url != null) {
			//logger.info("Askiryf[" + resource_name + "] created from resource");
			return toolkit.createImage(askiryf_url);
		} else {
			//logger.info("Askiryf[" + resource_name +
			//		"] created from static data: length="
			//		+ askyrif_data.length);
			return toolkit.createImage(askyrif_data);
		}
	}

	/**
	 * Get a shared instance of the ferundio image (would be wise not to modify it).
	 * 
	 * @param toolkit
	 *            the runtime toolkit being used
	 * @return a shared and sole instance of the ferundio image
	 */
	public static Image getFerundio(final Toolkit toolkit) {
		if (ResourceManager.Ferundio == null) {
			ResourceManager.Ferundio = ResourceManager.createFerundio(toolkit);
		}
		return ResourceManager.Ferundio;
	}

	/**
	 * Get a shared instance of the asundio image (would be wise not to modify it).
	 * 
	 * @param toolkit
	 *            the runtime toolkit being used
	 * @return a shared and sole instance of the asundio image
	 */
	public static Image getAsundio(final Toolkit toolkit) {
		if (ResourceManager.Asundio == null) {
			ResourceManager.Asundio = ResourceManager.createAsundio(toolkit);
		}
		return ResourceManager.Asundio;
	}

	/**
	 * Get a shared instance of the belief container image (would be wise not to modify it).
	 * 
	 * @param toolkit
	 *            the runtime toolkit being used
	 * @return a shared and sole instance of the belief image
	 */
	public static Image getContainer(final Toolkit toolkit) {
		if (ResourceManager.Container == null) {
			ResourceManager.Container = ResourceManager.createContainer(toolkit);
		}
		return ResourceManager.Container;
	}

	/**
	 * Get a shared instance of the belief statement image (would be wise not to modify it).
	 * 
	 * @param toolkit
	 *            the runtime toolkit being used
	 * @return a shared and sole instance of the belief image
	 */
	public static Image getBelief(final Toolkit toolkit) {
		if (ResourceManager.Belief == null) {
			ResourceManager.Belief = ResourceManager.createBelief(toolkit);
		}
		return ResourceManager.Belief;
	}

	/**
	 * Get a shared instance of the belief statement subject image (would be wise not to modify it).
	 * 
	 * @param toolkit
	 *            the runtime toolkit being used
	 * @return a shared and sole instance of the belief image
	 */
	public static Image getSubject(final Toolkit toolkit) {
		if (ResourceManager.Subject == null) {
			ResourceManager.Subject = ResourceManager.createSubject(toolkit);
		}
		return ResourceManager.Subject;
	}

	/**
	 * Get a shared instance of the belief statement predicate image (would be wise not to modify it).
	 * 
	 * @param toolkit
	 *            the runtime toolkit being used
	 * @return a shared and sole instance of the belief image
	 */
	public static Image getPredicate(final Toolkit toolkit) {
		if (ResourceManager.Predicate == null) {
			ResourceManager.Predicate = ResourceManager.createPredicate(toolkit);
		}
		return ResourceManager.Predicate;
	}

	/**
	 * Get a shared instance of the belief statement object image (would be wise not to modify it).
	 * 
	 * @param toolkit
	 *            the runtime toolkit being used
	 * @return a shared and sole instance of the belief image
	 */
	public static Image getObject(final Toolkit toolkit) {
		if (ResourceManager.Object == null) {
			ResourceManager.Object = ResourceManager.createObject(toolkit);
		}
		return ResourceManager.Object;
	}

	/**
	 * Get a shared instance of the selected belief container image (would be wise not to modify it).
	 * 
	 * @param toolkit
	 *            the runtime toolkit being used
	 * @return a shared and sole instance of the belief image
	 */
	public static Image getSelectedCotainer(final Toolkit toolkit) {
		if (ResourceManager.SelectedContainer == null) {
			ResourceManager.SelectedContainer = ResourceManager.createSelectedContainer(toolkit);
		}
		return ResourceManager.SelectedContainer;
	}

	/**
	 * Get a shared instance of the selected belief statement image (would be wise not to modify it).
	 * 
	 * @param toolkit
	 *            the runtime toolkit being used
	 * @return a shared and sole instance of the belief image
	 */
	public static Image getSelectedBelief(final Toolkit toolkit) {
		if (ResourceManager.SelectedBelief == null) {
			ResourceManager.SelectedBelief = ResourceManager.createSelectedBelief(toolkit);
		}
		return ResourceManager.SelectedBelief;
	}

	/**
	 * Get a shared instance of the selected belief statement subject image (would be wise not to modify it).
	 * 
	 * @param toolkit
	 *            the runtime toolkit being used
	 * @return a shared and sole instance of the belief image
	 */
	public static Image getSelectedSubject(final Toolkit toolkit) {
		if (ResourceManager.SelectedSubject == null) {
			ResourceManager.SelectedSubject = ResourceManager.createSelectedSubject(toolkit);
		}
		return ResourceManager.SelectedSubject;
	}

	/**
	 * Get a shared instance of the selected belief statement predicate image (would be wise not to modify it).
	 * 
	 * @param toolkit
	 *            the runtime toolkit being used
	 * @return a shared and sole instance of the belief image
	 */
	public static Image getSelectedPredicate(final Toolkit toolkit) {
		if (ResourceManager.SelectedPredicate == null) {
			ResourceManager.SelectedPredicate = ResourceManager.createSelectedPredicate(toolkit);
		}
		return ResourceManager.SelectedPredicate;
	}

	/**
	 * Get a shared instance of the selected belief statement object image (would be wise not to modify it).
	 * 
	 * @param toolkit
	 *            the runtime toolkit being used
	 * @return a shared and sole instance of the belief image
	 */
	public static Image getSelectedObject(final Toolkit toolkit) {
		if (ResourceManager.SelectedObject == null) {
			ResourceManager.SelectedObject = ResourceManager.createSelectedObject(toolkit);
		}
		return ResourceManager.SelectedObject;
	}

	// no instances
	private ResourceManager() {
	}

	// close your eyes
	private static byte[] Ferundio_data = new byte[] { -119, 80, 78, 71, 13, 10, 26, 10, 0, 0, 0, 13, 73, 72, 68, 82, 0, 0, 0, 19, 0, 0, 0,
			19, 8, 2, 0, 0, 0, -3, 50, -95, -101, 0, 0, 0, 4, 103, 65, 77, 65, 0, 0, -79, -113, 11, -4, 97, 5, 0, 0, 2, 71, 73, 68, 65, 84,
			56, 79, 85, -109, 33, -108, -92, 48, 12, -122, 123, 14, -55, 57, 86, 86, 34, 89, -121, 69, 34, 43, -79, -56, 74, 36, 22, -119,
			68, 98, 43, -111, -107, 88, 36, -110, 115, 115, 18, -55, 58, 112, -77, 95, -55, -20, -68, -69, -68, 62, 38, -109, -2, 127, -14,
			39, 109, 127, -99, -25, 25, 69, -111, -70, -82, 75, 93, 81, 116, 41, -75, 127, -3, 89, -44, -75, 28, -5, 95, -91, -66, -108,
			-118, 127, 127, -24, 56, -2, 84, -70, 80, -22, -29, 2, 18, -59, 96, 3, -27, -7, 124, 66, 126, -98, -9, -17, -79, -19, -37, -76,
			-81, -3, -79, 53, -25, 86, -53, -62, 63, -74, -127, 56, -69, 96, 110, 120, 64, 43, -7, 121, 60, 30, 125, -41, -102, 50, -49,
			-46, -72, 44, 116, 107, -45, 117, -74, -25, -34, -15, -19, -102, -52, -108, 73, -98, -59, -20, -114, 67, -1, -86, 4, 19, 111,
			-102, 38, 99, 76, -45, 88, 54, -68, 119, -109, -21, -5, -82, -82, 76, 98, 74, -48, 9, -66, 27, -5, -39, 59, 118, -63, 116, 93,
			43, 100, -75, 44, 11, -76, 113, 28, -113, 99, 23, 49, -84, -29, 120, -40, -38, 88, 107, -74, 117, -90, -121, 119, 124, -33,
			119, -25, 92, -37, 6, -78, -54, -78, -116, -102, -94, 57, -40, -3, 75, -60, 90, 75, 11, -81, -96, -52, 66, -10, -49, -45, 123,
			63, -49, -77, 74, -110, -124, 52, 77, -45, -112, -126, -54, -95, -3, -13, 20, 21, -56, 41, -53, -78, 40, 10, -96, 111, -102,
			12, 5, -68, -94, 52, -37, 97, -54, 42, 100, 1, 13, -109, 44, -92, -125, 68, 16, -53, -13, -4, 56, -48, -4, -86, -68, 109, 91,
			96, 14, -61, -112, -90, -87, 32, 48, -2, 2, 34, 23, 106, -29, 56, -106, 32, 14, -14, 68, 14, -4, 23, -109, -60, 113, 20, 81,
			77, -54, 34, 1, 49, -16, 17, 73, -124, -72, 48, -111, -64, 120, 48, 6, 57, -49, -66, 50, -91, 50, 101, -106, -60, -54, -71, 17,
			73, 64, -31, -84, -21, 74, 5, 90, 77, 83, -51, -88, 8, 106, -83, 105, -107, 56, -67, 120, 63, -75, -115, -79, 85, -90, 26, 107,
			-14, 76, 83, -113, -123, 108, 106, 118, 93, -121, 84, -48, 112, 68, 8, 73, -5, -66, 103, -85, -86, 42, -83, -109, -38, -28,
			110, -80, -96, 53, 11, 3, -63, 23, 50, 38, 117, 48, -23, -30, -58, 4, -48, 29, 79, 88, 33, 33, -13, -64, 100, -98, 72, -62,
			-95, 79, 112, 104, -61, -25, -117, 79, -28, 62, -58, 9, -116, -100, 118, -104, 27, 71, 15, -109, -42, -111, 36, 87, 66, -104,
			114, 60, 124, -1, 97, 122, 124, 70, -128, -14, 112, 28, 50, 107, -104, -88, -127, 73, 22, -87, 3, -126, -32, -78, -52, -17, -6,
			40, -94, 18, 101, 57, -52, 112, -5, -28, -70, 65, 102, 42, -124, 32, 48, 116, -38, -13, -45, 68, 10, -18, 119, -16, -3, -60,
			-107, 98, -73, -82, -21, -9, 101, -6, 121, -97, 119, -64, -42, 53, -57, -61, -77, 40, -14, 116, 28, 90, 55, -76, -87, 86, 60,
			58, -34, 10, -67, -4, -48, -2, 127, -97, 84, -66, -33, 4, 118, -84, -53, -28, 93, -29, -70, 98, 108, -13, 121, -84, 86, -33,
			-14, -84, -119, -65, 111, -97, -36, -92, 111, -19, -14, 111, 19, 81, -124, -14, 18, 0, 0, 0, 0, 73, 69, 78, 68, -82, 66, 96,
			-126 };
	private static byte[] Asundio_data = new byte[] { -1, -40, -1, -32, 0, 16, 74, 70, 73, 70, 0, 1, 1, 1, 0, 96, 0, 96, 0, 0, -1, -37, 0,
			67, 0, 8, 6, 6, 7, 6, 5, 8, 7, 7, 7, 9, 9, 8, 10, 12, 20, 13, 12, 11, 11, 12, 25, 18, 19, 15, 20, 29, 26, 31, 30, 29, 26, 28,
			28, 32, 36, 46, 39, 32, 34, 44, 35, 28, 28, 40, 55, 41, 44, 48, 49, 52, 52, 52, 31, 39, 57, 61, 56, 50, 60, 46, 51, 52, 50, -1,
			-37, 0, 67, 1, 9, 9, 9, 12, 11, 12, 24, 13, 13, 24, 50, 33, 28, 33, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50,
			50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50,
			50, 50, 50, -1, -64, 0, 17, 8, 0, 19, 0, 19, 3, 1, 34, 0, 2, 17, 1, 3, 17, 1, -1, -60, 0, 31, 0, 0, 1, 5, 1, 1, 1, 1, 1, 1, 0,
			0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, -1, -60, 0, -75, 16, 0, 2, 1, 3, 3, 2, 4, 3, 5, 5, 4, 4, 0, 0, 1, 125,
			1, 2, 3, 0, 4, 17, 5, 18, 33, 49, 65, 6, 19, 81, 97, 7, 34, 113, 20, 50, -127, -111, -95, 8, 35, 66, -79, -63, 21, 82, -47,
			-16, 36, 51, 98, 114, -126, 9, 10, 22, 23, 24, 25, 26, 37, 38, 39, 40, 41, 42, 52, 53, 54, 55, 56, 57, 58, 67, 68, 69, 70, 71,
			72, 73, 74, 83, 84, 85, 86, 87, 88, 89, 90, 99, 100, 101, 102, 103, 104, 105, 106, 115, 116, 117, 118, 119, 120, 121, 122,
			-125, -124, -123, -122, -121, -120, -119, -118, -110, -109, -108, -107, -106, -105, -104, -103, -102, -94, -93, -92, -91, -90,
			-89, -88, -87, -86, -78, -77, -76, -75, -74, -73, -72, -71, -70, -62, -61, -60, -59, -58, -57, -56, -55, -54, -46, -45, -44,
			-43, -42, -41, -40, -39, -38, -31, -30, -29, -28, -27, -26, -25, -24, -23, -22, -15, -14, -13, -12, -11, -10, -9, -8, -7, -6,
			-1, -60, 0, 31, 1, 0, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, -1, -60, 0, -75, 17,
			0, 2, 1, 2, 4, 4, 3, 4, 7, 5, 4, 4, 0, 1, 2, 119, 0, 1, 2, 3, 17, 4, 5, 33, 49, 6, 18, 65, 81, 7, 97, 113, 19, 34, 50, -127, 8,
			20, 66, -111, -95, -79, -63, 9, 35, 51, 82, -16, 21, 98, 114, -47, 10, 22, 36, 52, -31, 37, -15, 23, 24, 25, 26, 38, 39, 40,
			41, 42, 53, 54, 55, 56, 57, 58, 67, 68, 69, 70, 71, 72, 73, 74, 83, 84, 85, 86, 87, 88, 89, 90, 99, 100, 101, 102, 103, 104,
			105, 106, 115, 116, 117, 118, 119, 120, 121, 122, -126, -125, -124, -123, -122, -121, -120, -119, -118, -110, -109, -108, -107,
			-106, -105, -104, -103, -102, -94, -93, -92, -91, -90, -89, -88, -87, -86, -78, -77, -76, -75, -74, -73, -72, -71, -70, -62,
			-61, -60, -59, -58, -57, -56, -55, -54, -46, -45, -44, -43, -42, -41, -40, -39, -38, -30, -29, -28, -27, -26, -25, -24, -23,
			-22, -14, -13, -12, -11, -10, -9, -8, -7, -6, -1, -38, 0, 12, 3, 1, 0, 2, 17, 3, 17, 0, 63, 0, -12, -51, 94, -22, -30, -29, 72,
			-109, 92, -71, -15, 10, -24, -6, 66, 39, -104, -95, 96, 89, 55, 33, 60, 51, -106, 4, -28, -1, 0, 117, 122, 100, 14, 77, 115,
			-102, 126, -80, -83, 99, 125, -85, 73, -30, -21, 91, 11, 75, 53, -114, 65, 53, -91, -94, -104, 102, -119, -14, 17, -120, 108,
			-74, 73, 86, 5, 70, 8, 34, -72, -19, 70, 127, 18, 120, -117, -64, -6, -82, -125, 114, 116, -12, -122, -62, -63, 46, -28, -119,
			-26, -112, 79, 108, 3, 49, 17, 56, -39, -126, -29, 97, -56, -29, -88, -86, 87, -98, 16, -42, 108, 60, 33, -82, -24, 82, 38,
			-114, 33, -126, 40, 117, 25, 26, 41, -27, -5, -123, -100, -84, 96, 50, -110, 64, 42, -35, 79, -15, 117, -84, 93, 10, 114, -110,
			-88, -42, -91, 41, -55, 43, 39, -95, -24, 99, -30, -3, -99, -78, -120, 102, -45, 47, 47, -28, 80, 63, -46, -76, -8, 119, 65,
			40, 60, -122, 82, 121, -23, -116, -125, -48, -28, 118, -94, -71, -69, 31, -123, 62, 43, -76, -77, 72, 33, -67, -80, 72, -44,
			-110, -87, 21, -4, -54, -118, 9, 39, 10, 12, 100, -127, -49, 114, 104, -83, 46, -5, 10, -57, -87, -8, -97, 66, -45, 117, 75,
			25, -89, -68, -75, 18, 74, -112, -76, 65, -125, -78, -110, -115, -112, 84, -32, -115, -61, -109, -63, -50, 51, -59, 114, -98,
			11, -46, 108, 111, -18, 111, 5, -28, 38, 113, 26, 36, 99, -52, 118, 108, -84, 109, -108, 7, 39, -26, 3, 39, -82, 115, -98, 115,
			69, 21, -53, 82, 77, 87, -118, -71, -84, 82, -10, 109, -98, -114, -52, 65, -64, -2, 84, 81, 69, 118, 24, -97, -1, -39 };
	private static byte[] Container_data = new byte[] { -119, 80, 78, 71, 13, 10, 26, 10, 0, 0, 0, 13, 73, 72, 68, 82, 0, 0, 0, 16, 0, 0,
			0, 16, 8, 6, 0, 0, 0, 31, -13, -1, 97, 0, 0, 0, 1, 115, 82, 71, 66, 0, -82, -50, 28, -23, 0, 0, 0, 6, 98, 75, 71, 68, 0, -1, 0,
			-1, 0, -1, -96, -67, -89, -109, 0, 0, 0, 9, 112, 72, 89, 115, 0, 0, 11, 19, 0, 0, 11, 19, 1, 0, -102, -100, 24, 0, 0, 0, 7,
			116, 73, 77, 69, 7, -40, 3, 10, 11, 54, 52, -66, 89, 36, 119, 0, 0, 0, 25, 116, 69, 88, 116, 67, 111, 109, 109, 101, 110, 116,
			0, 67, 114, 101, 97, 116, 101, 100, 32, 119, 105, 116, 104, 32, 71, 73, 77, 80, 87, -127, 14, 23, 0, 0, 0, -60, 73, 68, 65, 84,
			56, -53, -19, -47, 61, 78, 2, 97, 24, 4, -32, 103, 95, 66, 11, 26, 126, 98, -94, 33, 24, 27, -83, -72, -127, -89, -32, 42, -44,
			38, 28, -119, 35, -20, 9, -24, -80, -39, 88, -48, 24, 10, 55, 91, 80, 17, -65, -75, -40, 53, -95, -48, 0, 61, 83, 77, 38, 51,
			-17, 47, 87, 100, 112, -5, 104, -45, -65, -13, -36, -23, 82, -28, -115, -10, 31, -98, 94, -43, -33, 7, -86, 79, -17, -27, -121,
			-105, -128, -101, -5, 38, -116, -63, 25, 77, 7, -99, 110, -109, -127, -24, 77, -52, 34, -64, -66, -56, 125, -99, 74, -73, -98,
			125, 4, -67, -119, 89, -116, -90, -42, 41, -87, -79, -68, 96, -11, 101, 74, -22, -47, -44, 58, -86, -99, 121, -124, 12, -117,
			11, 10, 44, 34, 100, -43, -50, 60, -6, 99, -85, 86, 28, 30, 25, -22, 19, 124, 8, -3, -79, 85, 20, -71, 67, 74, 82, 123, -31,
			-121, -29, -17, -4, -59, 127, 61, 41, 73, 69, -18, 16, 16, -31, 13, 37, -74, 103, -116, -65, 69, -39, 102, -82, -64, 15, -126,
			47, 55, -90, 10, 119, -43, 17, 0, 0, 0, 0, 73, 69, 78, 68, -82, 66, 96, -126 };
	private static byte[] Belief_data = new byte[] { -119, 80, 78, 71, 13, 10, 26, 10, 0, 0, 0, 13, 73, 72, 68, 82, 0, 0, 0, 16, 0, 0, 0,
			16, 8, 6, 0, 0, 0, 31, -13, -1, 97, 0, 0, 0, 1, 115, 82, 71, 66, 0, -82, -50, 28, -23, 0, 0, 0, 6, 98, 75, 71, 68, 0, -1, 0,
			-1, 0, -1, -96, -67, -89, -109, 0, 0, 0, 9, 112, 72, 89, 115, 0, 0, 11, 19, 0, 0, 11, 19, 1, 0, -102, -100, 24, 0, 0, 0, 7,
			116, 73, 77, 69, 7, -40, 3, 10, 12, 8, 46, 7, -79, -48, -11, 0, 0, 0, 25, 116, 69, 88, 116, 67, 111, 109, 109, 101, 110, 116,
			0, 67, 114, 101, 97, 116, 101, 100, 32, 119, 105, 116, 104, 32, 71, 73, 77, 80, 87, -127, 14, 23, 0, 0, 1, 67, 73, 68, 65, 84,
			56, -53, -107, -45, 61, 76, -44, 65, 16, -58, -31, -25, -114, 51, -112, -59, 24, 19, -62, -121, -83, 61, -95, -78, 83, 11, 43,
			75, 90, 26, 10, 99, 73, -74, 33, -124, -58, 66, 43, 104, 40, -40, -62, -50, -106, -54, -62, 18, 105, 49, -46, -112, 35, -58,
			-48, 66, -119, -91, -124, 100, 73, -8, 56, 41, 56, -30, -78, -36, 63, -24, 118, -17, -20, -50, 111, -34, -99, -99, -27, 127,
			87, -120, 67, -91, 108, 53, 28, 122, -118, -57, -40, -61, 115, -36, 36, 13, 97, 29, -117, 114, -6, 10, -99, -122, 58, -117, 56,
			-107, 83, 87, -120, -121, 24, 41, -10, 102, -15, -69, -39, 65, -120, -113, -80, -115, 103, 114, 58, 107, 112, -8, 26, 61, 57,
			109, 13, 2, 124, -64, 46, 126, -32, 85, -125, -61, -119, -66, -5, -43, 58, 121, 70, -120, 63, -17, 105, -30, 84, 41, 59, -43,
			-58, 2, -38, 69, -84, -123, 57, -124, -94, -119, -17, -124, -72, 44, -89, -115, -65, 61, 8, 113, 18, 47, -80, -125, 77, 57, 77,
			-1, -21, -85, 118, -124, -40, 65, 91, 78, -97, -123, -8, 4, -67, -54, -14, -61, 66, -75, -15, 17, 43, 114, -38, -65, 6, -28,
			116, -127, -93, -2, -127, 63, 119, 0, -41, -50, 70, 11, -3, -91, -72, -46, -99, 57, -88, -85, -113, -32, -101, -100, 78, -86,
			-8, 120, 19, -96, 93, -51, -58, 56, -66, 11, -15, 61, 46, -5, -79, 97, -84, 9, -15, -91, -100, -70, 53, -96, 85, 1, -50, 113,
			44, -89, 79, 69, -11, 49, 44, -55, -87, -21, -42, -109, 13, 6, -76, -6, 125, 41, -41, 101, -103, 55, -24, 47, -12, 42, -64,
			-104, 16, -33, 22, -79, 81, 60, 104, 2, 28, 97, -66, -48, -65, -16, 70, 78, -101, -43, 112, 29, -36, -56, 43, -120, 116, 79,
			109, -46, -74, 101, -122, 0, 0, 0, 0, 73, 69, 78, 68, -82, 66, 96, -126 };
	private static byte[] Subject_data = new byte[] { -119, 80, 78, 71, 13, 10, 26, 10, 0, 0, 0, 13, 73, 72, 68, 82, 0, 0, 0, 16, 0, 0, 0,
			16, 8, 4, 0, 0, 0, -75, -6, 55, -22, 0, 0, 0, 1, 115, 82, 71, 66, 0, -82, -50, 28, -23, 0, 0, 0, 2, 98, 75, 71, 68, 0, -1,
			-121, -113, -52, -65, 0, 0, 0, 9, 112, 72, 89, 115, 0, 0, 11, 19, 0, 0, 11, 19, 1, 0, -102, -100, 24, 0, 0, 0, 7, 116, 73, 77,
			69, 7, -40, 3, 10, 13, 26, 21, -49, -116, 35, 53, 0, 0, 0, 25, 116, 69, 88, 116, 67, 111, 109, 109, 101, 110, 116, 0, 67, 114,
			101, 97, 116, 101, 100, 32, 119, 105, 116, 104, 32, 71, 73, 77, 80, 87, -127, 14, 23, 0, 0, 0, 75, 73, 68, 65, 84, 40, -49, 99,
			96, -96, 35, -56, 99, -8, -50, -16, 31, 9, 98, -128, 15, 40, -46, 88, 20, 96, 21, 100, 96, 96, 34, 100, 51, 66, -63, 55, -88,
			41, -1, 25, -2, 51, -4, 98, -104, -118, -87, -76, -119, -31, 7, -110, 11, 126, -29, 55, 23, -55, 61, 44, 72, -126, -56, -32,
			15, -90, 2, 100, -16, -99, -95, -115, 97, 16, 1, 0, 119, -13, 36, -70, 118, -63, 116, -105, 0, 0, 0, 0, 73, 69, 78, 68, -82,
			66, 96, -126 };
	private static byte[] Predicate_data = new byte[] { -119, 80, 78, 71, 13, 10, 26, 10, 0, 0, 0, 13, 73, 72, 68, 82, 0, 0, 0, 16, 0, 0,
			0, 16, 8, 4, 0, 0, 0, -75, -6, 55, -22, 0, 0, 0, 1, 115, 82, 71, 66, 0, -82, -50, 28, -23, 0, 0, 0, 2, 98, 75, 71, 68, 0, -1,
			-121, -113, -52, -65, 0, 0, 0, 9, 112, 72, 89, 115, 0, 0, 11, 19, 0, 0, 11, 19, 1, 0, -102, -100, 24, 0, 0, 0, 7, 116, 73, 77,
			69, 7, -40, 3, 10, 13, 26, 2, 76, 95, -90, -14, 0, 0, 0, 25, 116, 69, 88, 116, 67, 111, 109, 109, 101, 110, 116, 0, 67, 114,
			101, 97, 116, 101, 100, 32, 119, 105, 116, 104, 32, 71, 73, 77, 80, 87, -127, 14, 23, 0, 0, 0, 53, 73, 68, 65, 84, 40, -49, 99,
			96, -96, 35, -8, -113, 4, -65, 49, -44, -30, 87, -16, -97, -31, 39, 118, 5, 88, -40, 44, 104, -118, 96, -32, 23, 118, 5, 48,
			-16, -109, -95, 27, -69, 2, 70, 76, -75, 76, -124, 60, 71, -71, -126, -63, 0, 0, 58, 6, 22, -20, 16, -2, -9, 106, 0, 0, 0, 0,
			73, 69, 78, 68, -82, 66, 96, -126 };
	private static byte[] Object_data = new byte[] { -119, 80, 78, 71, 13, 10, 26, 10, 0, 0, 0, 13, 73, 72, 68, 82, 0, 0, 0, 16, 0, 0, 0,
			16, 8, 4, 0, 0, 0, -75, -6, 55, -22, 0, 0, 0, 1, 115, 82, 71, 66, 0, -82, -50, 28, -23, 0, 0, 0, 2, 98, 75, 71, 68, 0, -1,
			-121, -113, -52, -65, 0, 0, 0, 9, 112, 72, 89, 115, 0, 0, 11, 19, 0, 0, 11, 19, 1, 0, -102, -100, 24, 0, 0, 0, 7, 116, 73, 77,
			69, 7, -40, 3, 10, 13, 23, 27, -99, -102, 112, 127, 0, 0, 0, 25, 116, 69, 88, 116, 67, 111, 109, 109, 101, 110, 116, 0, 67,
			114, 101, 97, 116, 101, 100, 32, 119, 105, 116, 104, 32, 71, 73, 77, 80, 87, -127, 14, 23, 0, 0, 0, 72, 73, 68, 65, 84, 40,
			-49, 99, 96, -96, 27, -88, 102, -8, -50, -16, 31, 14, -65, 51, 84, -94, 43, 64, -106, -2, -49, -16, -97, -31, 27, -70, -126,
			-1, 12, -1, -79, -13, -104, 8, -39, 77, 71, 5, -33, -95, 78, -5, 15, 117, -32, 119, -104, 2, 102, 40, -3, -105, -63, -122,
			-127, 5, -82, -19, 7, 67, 51, -61, 17, -122, 65, 3, 0, 68, -56, 31, 50, -18, -24, -9, -35, 0, 0, 0, 0, 73, 69, 78, 68, -82, 66,
			96, -126 };
	private static byte[] SelectedSubject_data = new byte[] { -119, 80, 78, 71, 13, 10, 26, 10, 0, 0, 0, 13, 73, 72, 68, 82, 0, 0, 0, 16,
			0, 0, 0, 16, 8, 6, 0, 0, 0, 31, -13, -1, 97, 0, 0, 0, 1, 115, 82, 71, 66, 0, -82, -50, 28, -23, 0, 0, 0, 6, 98, 75, 71, 68, 0,
			-1, 0, -1, 0, -1, -96, -67, -89, -109, 0, 0, 0, 9, 112, 72, 89, 115, 0, 0, 11, 19, 0, 0, 11, 19, 1, 0, -102, -100, 24, 0, 0, 0,
			7, 116, 73, 77, 69, 7, -40, 4, 11, 15, 2, 16, 52, -27, 32, -8, 0, 0, 0, 25, 116, 69, 88, 116, 67, 111, 109, 109, 101, 110, 116,
			0, 67, 114, 101, 97, 116, 101, 100, 32, 119, 105, 116, 104, 32, 71, 73, 77, 80, 87, -127, 14, 23, 0, 0, 2, 35, 73, 68, 65, 84,
			56, -53, 117, -111, -51, 106, 20, 81, 16, -123, -65, -70, -9, 118, 103, -70, 39, 25, -14, 39, 9, 3, -63, -83, 16, -73, 110,
			-59, 60, -127, 59, 5, -35, 103, 41, -28, 9, -110, -8, 4, 18, -48, -73, 16, -15, 9, 20, 87, -82, -36, -71, -113, 32, -125, 16,
			99, 38, 78, 102, -70, -89, -5, -34, 42, 23, 29, 53, 65, 115, -106, 5, -89, -22, 59, -89, 4, 0, 14, 28, -32, -32, 56, 112, -85,
			12, -76, 121, 64, -59, 97, -115, 0, 32, -75, -31, 125, 34, -124, -60, -119, 111, 97, 45, -63, 126, 2, -79, 0, 38, 112, -24, 96,
			-108, -77, -44, 47, -72, 8, 125, -62, -68, -64, 92, -64, -60, 33, -50, -80, 82, 33, 53, -44, -66, 102, 48, -101, -15, 51, 84,
			-80, 3, -112, 60, -120, -125, -29, -116, -91, 94, 73, -29, 87, 17, -73, 9, 110, 19, 11, 27, 24, 107, -32, 87, 48, 27, 16, 67,
			-127, 51, 79, -20, 43, -117, 26, -87, -17, 68, 120, 96, -95, 67, 47, 3, 49, -12, 113, -78, -50, -4, -19, 46, 122, -4, 20, 44,
			-29, 127, 90, -36, -69, 75, 12, 21, -100, -49, -31, 107, 114, 48, 18, -106, -13, 64, -96, 64, 88, 70, -65, 60, -66, -47, 12,
			-96, 49, 71, -93, -121, -87, 0, -124, 110, 40, 14, -109, -128, -91, 30, 104, 1, 64, -7, -20, 37, 34, 53, 102, 51, -60, -90, 24,
			99, -112, 111, -32, 4, -94, -4, -34, -25, -2, -82, 78, -46, -11, -15, 71, 25, 102, 57, -58, 2, 102, 61, 76, 122, -104, -28,
			-104, 101, -104, 56, 24, 92, 33, -112, -38, 32, 40, -30, 18, -72, 22, 52, 99, 118, -76, 123, -99, 93, 20, -73, -15, -127, -14,
			-47, 115, 98, -56, 89, 119, -114, -17, -37, -105, 23, 101, -63, 58, -77, 54, -8, -37, -17, 65, -46, -65, -31, -51, -95, -33,
			-18, -93, -46, 39, -45, -116, -40, 120, 24, 73, -128, -95, -31, 78, -107, -118, 72, -96, 97, -31, -31, 27, -52, 62, 33, -84,
			98, 86, 92, 82, 42, -43, -47, -109, 46, 114, 10, -104, 120, 44, 94, -119, -16, 35, 41, 61, -119, 104, -88, -87, 94, -68, -30,
			102, 41, -72, 6, 9, 17, 87, 42, -100, -31, -32, -77, 65, 84, 92, -42, 32, 58, -67, -39, 43, 45, 110, -21, 53, -58, -104, -104,
			106, -50, 38, 9, -122, 22, 96, -37, -32, 52, -111, -51, 107, -26, 126, 76, -71, 119, 15, 101, 25, -105, -6, -104, 101, -35,
			-37, 80, -60, 42, -116, 49, -54, 9, 121, -100, -62, 44, 2, 42, -128, -64, -127, -121, -13, -116, 1, 5, 109, 44, -55, -92, 64,
			45, -61, -126, 7, 21, -60, 20, -111, -106, 72, 69, -120, 83, 38, 109, 5, -61, 6, -10, -109, -17, -16, -34, 25, 124, 52, -26,
			-45, -56, 74, 106, -95, -87, -96, -103, 33, 114, -127, -24, 4, -105, 77, -16, 92, 80, -76, 83, -50, 102, 115, -40, 106, -69,
			62, 118, 76, -82, -68, 73, -32, 80, -70, -90, 71, 2, 67, -71, -34, -63, -56, 96, 104, -99, 113, -33, 64, 12, -32, 23, 79, -72,
			-3, 2, 52, -39, -114, 13, 0, 0, 0, 0, 73, 69, 78, 68, -82, 66, 96, -126 };
	private static byte[] SelectedPredicate_data = new byte[] { -119, 80, 78, 71, 13, 10, 26, 10, 0, 0, 0, 13, 73, 72, 68, 82, 0, 0, 0, 16,
			0, 0, 0, 16, 8, 6, 0, 0, 0, 31, -13, -1, 97, 0, 0, 0, 1, 115, 82, 71, 66, 0, -82, -50, 28, -23, 0, 0, 0, 6, 98, 75, 71, 68, 0,
			-1, 0, -1, 0, -1, -96, -67, -89, -109, 0, 0, 0, 9, 112, 72, 89, 115, 0, 0, 11, 19, 0, 0, 11, 19, 1, 0, -102, -100, 24, 0, 0, 0,
			7, 116, 73, 77, 69, 7, -40, 4, 11, 15, 2, 53, 127, -31, -12, -65, 0, 0, 0, 25, 116, 69, 88, 116, 67, 111, 109, 109, 101, 110,
			116, 0, 67, 114, 101, 97, 116, 101, 100, 32, 119, 105, 116, 104, 32, 71, 73, 77, 80, 87, -127, 14, 23, 0, 0, 1, -4, 73, 68, 65,
			84, 56, -53, -123, -109, -63, 78, 83, 81, 16, -122, -65, -103, 115, -18, -91, -67, 80, 90, -120, 16, -63, -115, 107, 124, 27,
			34, 27, -33, 65, 95, -95, 24, 95, -64, -32, 99, -24, -62, -8, 2, -58, -123, 15, -32, 66, -41, 36, -58, 70, 66, 106, 41, -91,
			-19, 109, -17, -71, 51, 46, 46, -95, 96, 32, 78, 114, 86, 103, -26, -97, -1, -97, -7, 71, 0, -32, 88, 1, -123, -45, -56, 78,
			17, -87, -14, -120, -119, -30, 75, 1, 64, 114, 71, -35, -56, -106, -119, -13, 89, -126, -89, 9, 48, 56, -74, 0, 46, -16, 37,
			-64, 32, -89, -45, 42, 40, -77, 46, 66, 15, -73, 30, 30, -70, -72, 110, -126, 109, -32, -76, 41, 67, 70, 59, -62, 114, -20,
			-48, 49, -8, -20, 1, 68, -31, 52, -93, -45, 42, 88, -122, 109, -124, 61, -100, 125, 76, 30, -29, -66, -117, -13, 8, -105, 109,
			-36, 59, 64, 78, 82, -89, -112, -60, -14, -68, -122, 111, 22, 27, -22, 69, 36, -59, 117, -44, 119, -104, -97, 124, -27, -34,
			-112, 10, 125, -14, -111, -75, -61, -73, -92, -72, -128, 98, 1, 36, -123, -127, -48, -53, 35, -111, 54, -30, 91, 60, 24, -98,
			97, -65, 14, 17, 122, -60, -48, 98, -85, 19, 96, 32, 17, 0, 19, -59, 82, -114, -121, -30, 38, -65, 120, -7, 1, 4, 28, 3, 18,
			-13, -109, 23, -32, 1, -56, 112, 11, -40, -91, 2, 68, 24, 9, -98, 9, -88, -94, 22, 111, 0, 102, -17, -98, -33, 35, -93, 6, 42,
			68, 107, 116, -45, 96, -124, -62, -77, -43, -65, -119, 63, 44, 65, 18, -70, -9, 9, -25, -126, 84, -105, -116, 38, 53, -20, 123,
			-124, -17, -64, 46, -96, 14, -106, 86, 18, 94, -67, -63, -11, 15, 106, 99, 44, 76, 17, -97, -30, 54, -62, 56, 39, 79, 83, -104,
			37, -64, 34, 108, 57, -38, 49, 124, -102, 16, 22, -73, 58, -2, 68, -4, 12, -45, 33, -22, 83, 52, -107, -92, 48, 35, 79, 83, 38,
			-43, -68, 49, 83, -33, 27, -51, -22, 70, 25, -105, 100, -11, -43, 45, 61, 3, -112, -33, -72, 12, -111, 114, -122, 74, -59,
			-122, 85, 119, -99, 40, 30, 97, -33, 25, 93, 37, -38, -52, 49, -65, 88, 1, -60, 51, -52, -121, -108, -83, 49, 72, 9, -35, -70,
			41, -62, -96, -17, -48, -52, 75, -102, 59, 24, 102, 116, -41, 11, 22, -109, 77, 98, -42, -84, 50, -123, 25, -71, 77, -72, -20,
			-50, -32, -96, -126, 35, -69, -106, 118, 103, -48, -46, -36, -62, -21, 0, -29, -116, -98, -81, -111, -22, -84, 33, 16, 42, 46,
			100, 1, -35, 10, -6, -11, -65, -123, -73, 0, -96, 1, 121, -81, -16, 35, 52, -42, 110, -122, 0, 7, 117, -45, -7, -31, -11, -22,
			-118, -42, 81, -29, 56, 88, 94, -65, -12, -65, 98, -128, -65, -4, -43, -20, -115, -18, 49, -27, 58, 0, 0, 0, 0, 73, 69, 78, 68,
			-82, 66, 96, -126 };
	private static byte[] SelectedObject_data = new byte[] { -119, 80, 78, 71, 13, 10, 26, 10, 0, 0, 0, 13, 73, 72, 68, 82, 0, 0, 0, 16, 0,
			0, 0, 16, 8, 6, 0, 0, 0, 31, -13, -1, 97, 0, 0, 0, 1, 115, 82, 71, 66, 0, -82, -50, 28, -23, 0, 0, 0, 6, 98, 75, 71, 68, 0, -1,
			0, -1, 0, -1, -96, -67, -89, -109, 0, 0, 0, 9, 112, 72, 89, 115, 0, 0, 11, 19, 0, 0, 11, 19, 1, 0, -102, -100, 24, 0, 0, 0, 7,
			116, 73, 77, 69, 7, -40, 4, 11, 13, 53, 0, -70, -47, 68, -58, 0, 0, 0, 25, 116, 69, 88, 116, 67, 111, 109, 109, 101, 110, 116,
			0, 67, 114, 101, 97, 116, 101, 100, 32, 119, 105, 116, 104, 32, 71, 73, 77, 80, 87, -127, 14, 23, 0, 0, 1, -78, 73, 68, 65, 84,
			56, -53, -83, -109, 77, 110, 19, 65, 16, -123, -65, -22, -23, -103, -79, 61, 114, 18, 80, -78, -79, 100, -55, 23, 98, 5, 7,
			-32, 4, 57, 1, 82, -126, -60, 9, -32, 24, 68, -30, 74, -111, 64, -34, 4, -103, 36, -58, -79, -5, 103, -86, 88, -76, 1, -101,
			-40, 59, 106, -41, -46, 123, 79, -81, 94, -11, 19, -72, 118, -128, -125, 91, -49, -59, -56, -109, 26, -113, -118, -61, -94,
			-16, 123, -92, 49, -100, 41, 117, -52, -36, 61, 101, -104, 101, 64, -31, 90, 125, 33, -49, 27, -58, -35, -112, -97, -66, -61,
			-89, 1, 70, -115, -119, -5, 43, 16, 21, 35, 17, -22, 13, -29, 110, -59, 114, -66, -122, 73, 4, 51, -127, -73, 3, -58, 93, 71,
			-84, 94, -30, -36, 57, 78, 79, 49, 90, -116, 29, 1, 20, 33, -96, -18, 1, -43, -17, 52, -3, -126, -27, 106, 5, -77, -24, 97,
			-28, -55, -66, -61, -27, 115, -62, -105, 75, -12, -37, 107, -80, -102, 103, 35, 9, 55, -67, -95, 125, -11, -119, -20, 3, -116,
			2, -112, 29, 103, -115, 71, -13, 0, 87, -99, 30, 39, 3, 88, -115, 126, 125, 83, 112, 121, -64, 89, -29, 97, 46, 37, -80, -102,
			26, -91, -3, 67, 30, 93, -66, -61, 120, 68, 8, 24, 45, -62, 9, 79, 31, 63, -128, -43, 24, 109, -63, -105, -116, 60, 22, 5, 19,
			-73, -73, -77, -15, 72, -59, 29, -54, -102, -118, 33, -3, -82, 17, 92, -63, 71, -127, 31, -30, 15, -70, 21, 2, -54, 26, -83,
			86, -48, -105, -9, -111, 57, 44, 96, -76, 84, 12, -95, 7, -57, -112, -98, -10, -72, -128, 52, -122, 68, 69, -48, 29, 7, 39,
			-12, 91, 39, -3, 54, -125, -67, -109, -102, 34, -115, -63, 11, -13, 56, 83, 2, 9, 79, 0, 73, 96, 117, 9, -20, -32, 110, 9, 33,
			-112, 72, 12, 77, 1, 28, -9, 49, -29, -4, 6, -19, 31, 112, -45, -49, 69, -28, 8, -39, 77, 111, 10, -50, 111, -72, -113, 25, 38,
			-1, -27, 39, -50, 50, -53, -7, -102, 49, 11, -78, 4, 92, 90, 96, -50, -17, 119, -63, 20, -47, -116, 54, 107, -102, 126, -59,
			50, -83, 75, -95, -82, -52, -105, 86, 77, 34, -53, 91, -27, 98, 20, 72, -19, -15, 54, 14, -2, 109, -93, -40, 22, 100, 2, -17,
			101, -37, 76, -127, -119, 60, -49, 96, 110, 48, -79, 66, -68, 50, 16, 3, -8, 5, -21, 11, -23, 125, -45, -128, -41, 21, 0, 0, 0,
			0, 73, 69, 78, 68, -82, 66, 96, -126 };
	private static byte[] SelectedContainer_data = new byte[] { -119, 80, 78, 71, 13, 10, 26, 10, 0, 0, 0, 13, 73, 72, 68, 82, 0, 0, 0, 16,
			0, 0, 0, 16, 8, 6, 0, 0, 0, 31, -13, -1, 97, 0, 0, 0, 1, 115, 82, 71, 66, 0, -82, -50, 28, -23, 0, 0, 0, 6, 98, 75, 71, 68, 0,
			-1, 0, -1, 0, -1, -96, -67, -89, -109, 0, 0, 0, 9, 112, 72, 89, 115, 0, 0, 11, 19, 0, 0, 11, 19, 1, 0, -102, -100, 24, 0, 0, 0,
			7, 116, 73, 77, 69, 7, -40, 4, 11, 15, 18, 27, -23, -11, -21, 33, 0, 0, 0, 25, 116, 69, 88, 116, 67, 111, 109, 109, 101, 110,
			116, 0, 67, 114, 101, 97, 116, 101, 100, 32, 119, 105, 116, 104, 32, 71, 73, 77, 80, 87, -127, 14, 23, 0, 0, 2, -112, 73, 68,
			65, 84, 56, -53, -91, -109, -69, 110, -100, 85, 20, -123, -65, -77, -49, 57, 51, -29, -71, 120, 60, 99, -29, -111, -119, -94,
			68, -118, 100, -84, 56, -62, 18, -94, -128, 58, 2, -119, 87, -128, -122, 18, 9, 2, 18, 60, 0, 78, -125, 68, 23, 36, 40, -110,
			119, 72, -108, -118, -102, 7, -32, 82, 68, 30, -69, -95, -55, -123, -56, -56, -111, 61, 51, -1, -4, -13, 95, -50, 57, -101,
			-30, 79, 20, 65, -53, -106, 86, -73, -42, -46, -34, -38, 107, 25, 56, 20, -72, 110, 96, 106, -32, -70, -62, 84, -31, 91, 5,
			-93, -4, 107, -44, -64, 109, -13, 95, -82, -127, 79, 59, 76, -74, 44, 41, 55, 72, 87, 57, 45, 3, 108, 70, 32, -63, -31, 75,
			-109, 67, 3, 8, -68, -80, 76, -38, -18, 53, -9, 44, 10, -3, -83, 117, -26, 50, 34, -13, 99, 22, -59, -112, 97, -81, 11, 51,
			-33, 8, -108, 6, 8, -52, 60, -61, 94, -105, 69, 49, 36, -13, 99, -26, 50, -94, -65, -75, -18, -48, -76, 3, 85, 15, -116, -128,
			-53, 41, -77, 115, -122, -15, -126, 89, 72, 112, 59, 53, 27, -68, -80, 12, 67, -121, -78, -34, 64, -36, 8, -76, 11, 69, 66,
			-35, -46, -111, -30, -18, 112, -5, -18, 15, -93, 55, -53, 29, -21, -31, -49, 95, 111, -67, 67, -35, 46, 32, 47, -31, -105, -40,
			24, -20, 90, -22, 110, 7, -119, 27, -41, -34, -67, -13, 91, -84, -31, -4, -81, -10, -13, -39, -33, -97, 125, 37, 24, 110, -116,
			47, 55, -30, -57, 71, -97, 124, -114, -38, 46, -86, 30, -100, -64, -82, 105, -32, 4, 85, -113, 74, -17, -15, -47, -57, 95, 88,
			15, -29, -53, -27, 14, -122, 27, 98, -3, -109, -9, 68, 64, -109, -44, -11, 106, -84, 104, -16, -88, 17, 70, -63, 66, -43, 96,
			20, 44, 26, 45, 86, 125, -67, -38, 66, -109, -44, 34, 96, -3, -109, -9, -27, -22, -63, -125, 15, 99, -124, -7, -39, -2, -17,
			-88, 113, 56, 4, -115, -106, -32, 29, 108, 123, -40, -10, 4, 60, -22, 45, -86, 22, 85, 63, 63, -37, -1, 35, 70, -72, 122, -16,
			-32, 3, 121, 122, 114, -13, -40, 90, 24, 108, -98, 28, -128, 90, -44, 56, 90, -22, -119, -66, -59, 96, -39, 32, 118, 61, 62,
			-76, 80, -21, 17, -29, 6, -101, 39, 111, 91, 11, 79, -113, 111, -98, 72, -107, -65, 53, 5, 16, 91, 119, -64, 40, -86, -106,
			-59, -113, -57, 104, -76, -24, -102, 101, -15, -45, 25, 42, 14, 21, 71, 126, -25, 103, -46, 43, 46, 84, -85, -67, 35, -47, -28,
			-90, 49, 54, -49, -18, 12, -98, -75, 49, 54, -48, -65, -75, -113, -47, -124, -103, 39, 6, -33, -68, 1, -102, 48, -102, -24,
			125, -7, 81, 103, -16, -52, 3, -60, -120, 106, -78, 83, 65, -51, -93, 34, -69, -14, 80, -109, 44, 47, -19, -35, -1, -98, -60,
			-100, -96, 43, -84, -106, 44, -38, 21, -117, -86, -62, 45, 75, -126, -51, 73, 50, -69, -76, 119, -1, 59, 77, 54, 47, -78, 43,
			15, 81, -13, -56, -48, -5, -6, -128, 20, 122, -88, 90, 68, 87, -72, 116, 65, -78, 51, 122, -19, -68, -119, 53, 48, 105, 59,
			-106, 101, 23, 49, 27, 4, -77, -127, -80, 70, 52, 17, 97, -23, 48, -14, 28, 92, 27, 74, 65, 91, 1, -83, 10, -78, -117, -126,
			108, -83, -126, -13, 38, -119, -89, -93, 4, -21, 48, 88, 37, -76, -107, 19, -60, 53, 39, 74, -23, -56, -6, 115, 38, -103, 37,
			-87, 65, -84, 114, 42, 1, -82, -67, 44, -45, -67, 87, 101, 82, -96, 100, 81, 6, 38, -74, 120, 93, -90, 126, 52, -1, -73, -50,
			-1, 0, 101, 7, 74, -75, -115, 20, 63, 0, 0, 0, 0, 0, 73, 69, 78, 68, -82, 66, 96, -126 };
	private static byte[] SelectedBelief_data = new byte[] { -119, 80, 78, 71, 13, 10, 26, 10, 0, 0, 0, 13, 73, 72, 68, 82, 0, 0, 0, 16, 0,
			0, 0, 16, 8, 6, 0, 0, 0, 31, -13, -1, 97, 0, 0, 0, 1, 115, 82, 71, 66, 0, -82, -50, 28, -23, 0, 0, 0, 6, 98, 75, 71, 68, 0, -1,
			0, -1, 0, -1, -96, -67, -89, -109, 0, 0, 0, 9, 112, 72, 89, 115, 0, 0, 11, 19, 0, 0, 11, 19, 1, 0, -102, -100, 24, 0, 0, 0, 7,
			116, 73, 77, 69, 7, -40, 4, 11, 15, 56, 52, 45, 79, 26, 80, 0, 0, 0, 25, 116, 69, 88, 116, 67, 111, 109, 109, 101, 110, 116, 0,
			67, 114, 101, 97, 116, 101, 100, 32, 119, 105, 116, 104, 32, 71, 73, 77, 80, 87, -127, 14, 23, 0, 0, 2, 83, 73, 68, 65, 84, 56,
			-53, 53, -46, -79, -117, 93, 85, 20, -59, -31, 111, -97, 115, -18, -101, -52, -101, 73, 50, 35, 26, 66, 58, -47, -96, -99, 32,
			41, 20, 59, 59, 11, 75, 11, 3, 118, 41, -13, 71, 88, 89, 89, -40, 10, 6, 45, -59, -58, 74, 3, 86, 54, -74, 66, 16, 69, 68, 49,
			-115, 49, -112, 32, 10, -103, -52, -52, 125, -9, -100, 109, 113, 117, -61, -86, 54, 11, -42, -34, -65, 21, -42, 9, -34, 41, 92,
			109, -12, -119, 77, 115, -7, 66, -84, -85, 127, 24, -91, -56, -1, -107, 85, -19, -25, -10, 14, 102, -113, 79, -25, -70, -102,
			-33, 15, -82, 52, -20, 57, 60, -37, -86, -15, -110, -34, 95, -44, -25, -65, -11, 120, -61, 40, 47, -56, 120, 94, -26, 117, 124,
			102, -60, 31, -78, -36, 119, -36, 70, 91, -51, 15, 42, -105, 38, -121, 79, -10, 45, -19, 64, -72, 45, -57, -103, -116, 95, 69,
			121, -120, 61, -78, -54, 44, -72, 77, -4, -91, -97, 111, -100, -73, -91, -15, 99, 112, 80, 93, -22, 123, -26, -70, -81, -26,
			21, 25, 55, -124, -101, 74, 78, -94, 61, -90, -57, 26, 127, 52, 38, 44, -81, 25, -27, 101, 125, 119, -73, 113, 92, 56, -102,
			44, 79, 47, 24, -45, 86, -23, -73, -92, -113, 69, 121, -42, 24, -81, -78, 75, 34, 24, -123, 82, -24, 112, 25, 67, -18, 125,
			-45, -72, 86, 93, 124, -76, -79, -37, 92, -48, -58, 43, -46, -101, -108, 79, -124, 83, -91, -4, -50, -110, 50, -85, -100, 38,
			-71, 52, -54, 51, 74, -5, -45, 50, 63, 81, -22, -46, 120, 80, -116, -3, 38, 119, -41, 100, -68, -117, -62, 72, 97, -79, -85,
			-117, -88, 111, 11, -121, -116, 74, -103, -120, 91, 114, -7, 80, -108, -49, -107, 28, -115, 86, -28, 114, 69, 45, 55, 100, -71,
			-61, -8, 72, -76, 97, 55, -70, -40, 45, -46, -105, 70, -35, -104, -38, 102, 77, 16, 95, 40, -3, 76, 78, 93, -37, -12, -94,
			-100, 86, 70, -88, -27, 46, 99, -63, 32, 7, 75, -82, 61, 104, 7, -40, -54, 121, 43, 29, -56, -4, -64, 46, -82, -125, -57, 52,
			-29, 104, 86, 79, 30, -104, -57, 101, -75, -20, -28, 24, 32, -93, 16, 77, -28, -21, 34, 47, -54, 90, -55, -126, 111, -107, -78,
			-107, -67, 58, -102, 107, 99, 25, -22, 118, 17, 58, 125, 65, 50, -62, -90, 52, 115, -33, -86, -11, 39, -100, -83, -65, 41, 101,
			77, 53, 14, 100, -37, -24, 39, -83, -15, 48, -59, 113, -118, -3, -79, -30, -54, 32, -118, 49, 38, -83, 94, -107, 62, 37, -17,
			88, -7, 5, -71, -63, 109, -91, -65, 103, -76, -17, 26, -57, 41, -10, -110, 49, 68, -4, -57, -100, -75, 121, 1, 39, -44, -81,
			-23, 73, 37, -57, -79, 112, 83, -116, 95, -116, 81, 11, -41, 82, -19, 67, -44, -114, -1, 79, 88, -120, 78, 34, 115, 53, -25,
			-64, 16, 22, 50, 68, 73, -47, -78, -127, 71, -53, 112, 88, 23, 75, -103, -107, -47, -107, 122, -58, -82, 24, 109, 71, 94, 34,
			-34, 90, -23, -116, -64, 22, 85, -12, 89, -76, 94, 121, 46, 56, 10, -13, 121, -40, 120, 42, -37, -9, -58, -14, -48, -120, -59,
			24, 39, 74, -4, -90, -8, 74, 41, -9, -108, -15, 3, -18, 9, -9, 45, -7, -77, -87, -99, -2, 11, 103, -103, 30, -37, -35, 127,
			-96, 45, 0, 0, 0, 0, 73, 69, 78, 68, -82, 66, 96, -126 };

	private static final byte[][] SemanticStatementData = new byte[][] { ResourceManager.Subject_data, ResourceManager.Predicate_data,
			ResourceManager.Object_data, ResourceManager.SelectedSubject_data, ResourceManager.SelectedPredicate_data,
			ResourceManager.SelectedObject_data };
}
