package ubiware.core.visualizer;

import javax.swing.tree.TreeModel;

/**
 * <p>
 * A specification of {@link StringLiteralTreeModel} in order to have indication about what exactly this string literal represents. This can
 * be achieved by checking the {@link #getType() type} of an instance.
 * </p>
 * 
 * @author Nikos Mouchtaris
 * @see Type
 */
public class SemanticStatementElementTreeModel extends StringLiteralTreeModel {
	private final Type type;

	/**
	 * Creates a new instance of this tree model. The actual root of the whole tree (including other tree models that co-operate with this
	 * one) will be {@code root}. The literal string value of this tree with be {@code value}. Its {@code type} indicates whether it's a
	 * string literal for the subject, predicate or object of a {@link ubiware.core.SemanticStatement}.
	 * 
	 * @param root
	 *            the actual tree of the whole tree structure
	 * @param value
	 *            the literal string value of this tree model
	 * @param type
	 *            the type of this tree model
	 */
	public SemanticStatementElementTreeModel(final TreeModel root, final String value, final Type type) {
		super(root, value);
		this.type = type;
	}

	/**
	 * Returns the {@link Type} of this tree model.
	 * 
	 * @return the {@link Type} of this tree model.
	 */
	public Type getType() {
		return this.type;
	}

	/**
	 * The enumeration which describes the possible types that this tree model can have.
	 */
	public enum Type {
		/**
		 * Indicates that the tree model is a string literal representing the {@link ubiware.core.SemanticStatement#subject} of a
		 * {@link ubiware.core.SemanticStatement}.
		 */
		Subject,
		/**
		 * Indicates that the tree model is a string literal representing the {@link ubiware.core.SemanticStatement#predicate} of a
		 * {@link ubiware.core.SemanticStatement}.
		 */
		Predicate,
		/**
		 * Indicates that the tree model is a string literal representing the {@link ubiware.core.SemanticStatement#object} of a
		 * {@link ubiware.core.SemanticStatement}.
		 */
		Object
	}
}
