package ubiware.core.visualizer;

import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

/**
 * <p>
 * This is a trivial implementation of a tree model, for the sake of representing literal string values which have no children as trees.
 * </p>
 * 
 * @author Nikos Mouchtaris
 */
public class StringLiteralTreeModel implements SubtreesTreeModel {
	private final String value;
	private final TreeModel root;

	/**
	 * Creates a new and immutable instance of this tree model.
	 * 
	 * @param root
	 *            the actual tree root or {@code this} if {@code root == null}
	 * @param value
	 *            the literal string value of this node/tree
	 */
	public StringLiteralTreeModel(final TreeModel root, final String value) {
		this.root = root == null ? this : root;
		this.value = value;
	}

	@Override
	public Object getRoot() {
		return this.root;
	}

	@Override
	public Object getChild(Object parent, int index) {
		assert false; // illegal program state
		return null;
	}

	@Override
	public int getChildCount(Object parent) {
		assert parent == this;
		return 0;
	}

	@Override
	public boolean isLeaf(Object node) {
		assert node == this;
		return true;
	}

	@Override
	public void valueForPathChanged(TreePath path, Object newValue) {
		// ignore 
	}

	@Override
	public int getIndexOfChild(Object parent, Object child) {
		assert false; // illegal program state
		return -1;
	}

	@Override
	public void addTreeModelListener(TreeModelListener l) {
		// ignore
	}

	@Override
	public void removeTreeModelListener(TreeModelListener l) {
		// ignore
	}

	@Override
	public String toString() {
		return this.value;
	}
}