package ubiware.core.visualizer;

import javax.swing.tree.TreeModel;

/**
 * <p>
 * A {@link SubtreesTreeModel} is a tree model which does not solely handle the whole tree structure, but instead, it handles only its very
 * own immediate children. Children of its children and further down are responsibility of their own immediate parents and this tree will
 * pass queries further down, to its own children, if it is not the {@code parent} of a query.
 * </p>
 * 
 * <p>
 * Classes which implement this interface will usually have as children other {@link SubtreesTreeModel}s but this is not necessary. The
 * children should at least, though, be able to handle queries themselves, since this tree will pass down the queries if it is not the
 * parent of the {@code child}.
 * </p>
 * 
 * <p>
 * Generally, for each method {@code mXXX} of this interface, the operation will be something like:
 * </p>
 * 
 * <pre>
 * public Object mXXX(Object parent, Object child) {
 * 	if (parent == this)
 * 		return myOwnResult(parent, child);
 * 	return TreeModel.class.cast(parent).mXXX(parent, child);
 * }
 * </pre>
 * 
 * @author Nikos Mouchtaris
 */
public interface SubtreesTreeModel extends TreeModel {

}
