package ubiware.infrastructure.packageManager;

import jade.core.behaviours.TickerBehaviour;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import ubiware.core.UbiwareAgent;
import ubiware.core.commands.AddBeliefsN3Command;
import ubiware.ontology.pma;
import ubiware.ontology.rdf;
import ubiware.util.IDGenerator;
import ubiware.util.saplbuilder.Literal;
import ubiware.util.saplbuilder.Resource;
import ubiware.util.saplbuilder.SaplDocument;

import com.google.common.collect.Lists;

public class FileCheckerTickBehavior extends TickerBehaviour {
	private static final long serialVersionUID = 1L;

	private File folder;
	private UbiwareAgent myUbiwareAgent;
	private IDGenerator idGen = new IDGenerator("pmaUBIPackage");

	public FileCheckerTickBehavior(UbiwareAgent myAgent, int timeInterval, File folder) {
		super(myAgent, timeInterval);
		this.folder = folder;
		this.myUbiwareAgent = myAgent;
	}

	@Override
	protected void onTick() {
		System.out.println("Checking folder: " + this.folder.getAbsolutePath());
		File[] files = this.folder.listFiles();
		if (files != null) {
			if (files.length == 0) {
				System.out.println("empty");
				return;
			} else {
				System.out.println("something is here: " + files.length);
				ArrayList<String> fileNames = Lists.newArrayList(this.folder.list());
				for (String fn : fileNames) {
					if (fn.endsWith(".ubi")) {
						if (fileNames.contains(fn + ".deployed") || fileNames.contains(fn + ".inprogress")) {
							System.out.println("File " + fn + " has been deployed already or is being deployed. No need to deploy.");
						} else {
							System.out.println("File " + fn + " has not been deployed yet. Let's deploy it now!.");
							File fileToDeploy = new File(this.folder.getAbsolutePath() + File.separator + fn);
							File inProgressFile = new File(this.folder.getAbsolutePath() + File.separator + fn + ".inprogress");
							try {
								boolean res = inProgressFile.createNewFile();
								if (res == false) {
									System.err.println("File " + inProgressFile.getAbsolutePath() + " cannot be created");
									this.stop();
									return;
								}
							} catch (IOException e) {
								e.printStackTrace();
								System.err.println("File " + inProgressFile.getAbsolutePath() + " cannot be created");
								this.stop();
								return;
							}
							//		?id rdf:type pma:UBIDeploymentAction
							//			; pma:pathToUBIFile ?dep
							//			; pma:deploymentStatus pma:UBIIdentified .
							Resource actionID = this.idGen.getNewResource();
							SaplDocument sd = new SaplDocument();
							sd.addStatement(actionID, rdf.type, pma.UBIDeploymentAction);
							sd.addStatement(actionID, pma.pathToUBIFile, new Literal(this.folder.getAbsolutePath() + File.separator + fn));
							sd.addStatement(actionID, pma.deploymentStatus, pma.UBIIdentified);
							this.myUbiwareAgent.addCommand(new AddBeliefsN3Command(sd));
							System.out.println("to deploy: " + fileToDeploy.getAbsolutePath());
						}
					}
				}

			}
		} else {
			System.err.println("Something went wrong, because if I list the content of folder " + this.folder.getAbsolutePath()
					+ " I get null as the return value");
			this.stop();
			return;
		}
	}
}
