package ubiware.infrastructure.ubiwareDF;

import java.util.HashSet;
import java.util.Set;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.Binding;
import ubiware.core.BindingsSet;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.IllegalParameterConfigurationException.State;
import ubiware.ontology.udf;
import ubiware.util.saplbuilder.InvalidResourceException;
import ubiware.util.saplbuilder.Resource;

public class DefineServiceBehavior extends ReusableAtomicBehavior {

	private static final long serialVersionUID = 1L;
	private Service service;

	@Override
	protected void doAction() {
		ServiceRepository.getServiceRepository().defineService(this.service);
	}

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		Resource serviceResource = parameters.getObligatedResourceParameter(udf.serviceName);
		String handlerContainerID = parameters.getObligatedNonEmptyStringParameter(udf.handlers);
		BindingsSet set = this.hasBeliefsN3(
				"?x <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.ubiware.jyu.fi/communication#Handler>",
				handlerContainerID);
		Set<Handler> handlers = new HashSet<Handler>();
		for (Binding b : set) {
			try {
				Resource handlerResource = b.getResource("x");
				handlers.add(new Handler(handlerResource));
			} catch (InvalidResourceException e) {
				throw new IllegalParameterConfigurationException(udf.handlers, e.getViolatingResource(), State.notAResource, e);
			}
		}
		this.service = new Service(serviceResource, handlers);
	}
}
