package ubiware.infrastructure.ubiwareDF;

import jade.core.AID;
import jade.lang.acl.ACLMessage;

import java.util.HashSet;
import java.util.Set;

import ubiware.core.UbiwareAgent;
import ubiware.ontology.udf;
import ubiware.util.saplbuilder.Literal;
import ubiware.util.saplbuilder.Resource;
import ubiware.util.saplbuilder.SaplDocument;

import com.google.common.collect.Sets;

public class InformingServiceProviderListener implements ServiceProviderListener {

	private String receiver;
	private UbiwareAgent myAgent;
	private String conversationID;
	private Set<Agent> previousUpdate;

	//private final static URI messageNamespace = URITools
	//		.createUncheckedURI("http://www.ubiware.jyu.fi/directoryFacilitator/messages/send/");

	//private final static UniqueInstanceGenerator messageInstanceGenerator = new UniqueInstanceGenerator("",
	//		InformingServiceProviderListener.messageNamespace);

	//	public InformingServiceProviderListener(Agent receiver, UbiwareAgent myAgent) {
	//		this(receiver, myAgent, InformingServiceProviderListener.messageInstanceGenerator.getNewResource());
	//	}

	public InformingServiceProviderListener(Agent receiver, UbiwareAgent myAgent, Resource conversationID) {
		super();
		this.receiver = receiver.getAgentAsResource().toString();
		this.myAgent = myAgent;
		this.conversationID = conversationID.toString();
	}

	@Override
	public void serviceProviderListChanged(Service service, Set<Agent> apts) {
		HashSet<Agent> agentsProvidingThisService = Sets.newHashSet(apts);
		// This is ran every time service providers for some service have changed.
		//check whether it is a real change:
		if ((this.previousUpdate != null) && this.previousUpdate.equals(agentsProvidingThisService)) {
			return;
		}
		ACLMessage message = new ACLMessage(ACLMessage.INFORM);
		message.addReceiver(new AID(this.receiver, AID.ISLOCALNAME));
		//StringBuilder info = new StringBuilder();
		//for (Agent agentProviding : agentsProvidingThisService) {
		//	// This is what is sent to all continuous subscribers.
		//	info.append(agentProviding.getName() + " :providerOfService " + service.getName() + " . ");
		//}
		SaplDocument document = new SaplDocument();
		if (agentsProvidingThisService.size() > 0) {
			for (Agent agent : agentsProvidingThisService) {
				document.addStatement(agent.getAgentAsResource(), udf.providesService, service.getName());
			}
			document.addStatement(service.getName(), udf.isProvided, new Literal(true));
		} else {
			document.addStatement(service.getName(), udf.isProvided, new Literal(false));
		}

		String sapl = document.generateSapl();
		message.setContent(sapl);
		message.setConversationId(this.conversationID);
		message.setOntology("<http://www.ubiware.jyu.fi/directoryFacilitator#service>");
		//System.out.println("informing " + this.receiver + " about " + service.getName().toString() + "\n message " + message);
		this.myAgent.send(message);
		this.previousUpdate = agentsProvidingThisService;
	}
}