package ubiware.infrastructure.web;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import com.google.common.io.Files;
import com.google.common.io.InputSupplier;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.Binding;
import ubiware.core.BindingsSet;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.IllegalParameterConfigurationException.State;
import ubiware.ontology.param;
import ubiware.util.saplbuilder.InvalidResourceException;
import ubiware.util.saplbuilder.Resource;
import ubiware.util.saplbuilder.SaplDocument;

public class DeployWarBehavior extends ReusableAtomicBehavior {
	private static final long serialVersionUID = 1L;
	private WebServer webServer;
	private File warFile;
	private String displayName;
	private String contextPath;
	private Set<Resource> infraAgents;
	private Resource awaService;

	@Override
	protected void doAction() {
		try {
			this.webServer.deployWar(this.contextPath, this.displayName, this.warFile, this.infraAgents, this.awaService);
		} catch (Exception e) {
			e.printStackTrace();
			this.setFailed();
		}
	}

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		Resource objectID = parameters.getObligatedResourceParameter(param.server);
		this.webServer = (WebServer) this.getFromBlackboard(objectID);
		
		
		if (this.webServer == null) {
			throw new IllegalParameterConfigurationException(param.socket, objectID.toString(), State.notOnBlackboard);
		}
		this.contextPath = parameters.getObligatedNonEmptyStringParameter(param.context);
		this.displayName = parameters.getObligatedNonEmptyStringParameter(param.name);
		this.warFile = parameters.getOptionalFileParameter(param.warPath);
		if (this.warFile == null){
			searchWarFileFromClassPath(parameters);
		}
		// FIXME add infra agents
		String infraContainer = parameters.getOptionalContainerID(param.infra, null);
		if (infraContainer == null) {
			this.infraAgents = Collections.<Resource> emptySet();
		} else {
			this.infraAgents = new HashSet<Resource>();
			//@prefix rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#> .
			//	@prefix type: <http://www.ubiware.jyu.fi/agenttypes#> .
			//	<http://www.ubiware.jyu.fi/infrastructure#UserManagerAgent> rdf:type type:AIA
			BindingsSet b = this
					.hasBeliefsN3(
							"@prefix rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#> .  @prefix type: <http://www.ubiware.jyu.fi/agenttypes#> . ?agent rdf:type type:AIA",
							infraContainer);
			if (b != null) {
				for (Binding binding : b) {
					try {
						Resource agent = binding.getResource("agent");
						this.infraAgents.add(agent);
					} catch (InvalidResourceException e) {
						throw new IllegalParameterConfigurationException(param.infra, e.getViolatingResource(), State.notAResource);
					}
				}
			}
		}
		this.awaService = parameters.getOptionalResourceParameter(param.awaService, null);
	}

	private void searchWarFileFromClassPath(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		String warFileArgument = parameters.getObligatedNonEmptyStringParameter(param.warResourcePath);
		//try to get it from the classpath
		final InputStream warFileStream = DeployWarBehavior.class.getResourceAsStream(warFileArgument);
		if (warFileStream == null){
			throw new IllegalParameterConfigurationException(param.warResourcePath, warFileArgument, State.resourceNotFound);
		}
		File tempDir = Files.createTempDir();
		File tempFile = new File (tempDir, "warFile.war");
		try {
			Files.copy(new InputSupplier<InputStream>() {

				@Override
				public InputStream getInput() throws IOException {
					// TODO Auto-generated method stub
					return warFileStream;
				}
			}, tempFile );
		} catch (IOException e) {
			throw new Error("an IOException occured while copying a stream which has been checked to exist to a temp location");
		}
		//now, the file is at the temp location.
		this.warFile = tempFile;
	}
}
