package ubiware.infrastructure.web;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;

import ubiware.util.saplbuilder.Resource;

class PlatformConnectionFilter implements Filter {

	private final TicketService ticketService;
	private final AgentWrapperProvider awp;
	private final Resource workersService;
	private final InfraAgentWrapperProvider infraAgents;

	public PlatformConnectionFilter(TicketService ticketService, AgentWrapperProvider awp, Resource service,
			InfraAgentWrapperProvider aIAProvider) {
		super();
		this.ticketService = ticketService;
		this.awp = awp;
		this.workersService = service;
		this.infraAgents = aIAProvider;
	}

	@Override
	public void destroy() {
		//no resources to clean up
	}

	@Override
	public void doFilter(ServletRequest req, ServletResponse resp, FilterChain chain) throws IOException, ServletException {
		if (!(req instanceof HttpServletRequest)) {
			return;
		}
		//System.out.println("doing platform connection filter");
		HttpServletRequest httpReq = (HttpServletRequest) req;
		PlatformInterfaceImpl pif = new PlatformInterfaceImpl();
		pif.setLoggedIn(false);
		try {
			Ticket ticket = Util.getUbiwareTicket(httpReq);
			if ((ticket != null) && this.ticketService.isLoggedIn(ticket)) {
				pif.setLoggedIn(true);
				if (this.workersService != null) {
					AgentWrapper worker = this.awp.getWorkerAgent(this.ticketService.getPersonalUseragent(ticket), this.workersService);
					pif.setWorkerAgent(worker);
				}
			}
		} catch (NoTicketException e) {
			//ok, in case there is no ticket, no worker
		} catch (NoSuchWorkerException e) {
			//ok, no worker is set.
		}
		pif.setInfraAgentsWrapperProvider(this.infraAgents);

		//ADD the PlatformInterface
		req.setAttribute(PlatformInterface.PLATFORM_INTERFACE_KEY, pif);

		if (httpReq.getSession() != null) {
			//System.err.println(httpReq.getSession().getMaxInactiveInterval());
			httpReq.getSession().setMaxInactiveInterval(60 * 60);//in seconds
		}
		chain.doFilter(req, resp);
	}

	@Override
	public void init(FilterConfig arg0) throws ServletException {
		//no resources to set up
	}

}
