package ubiware.infrastructure.web;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import ubiware.ontology.infra;
import ubiware.ontology.user;
import ubiware.util.saplbuilder.ActionArgumentContainer;
import ubiware.util.saplbuilder.InvalidResourceException;
import ubiware.util.saplbuilder.Literal;
import ubiware.util.saplbuilder.Resource;

class TicketServiceImpl implements TicketService {

	private final AgentWrapperProvider awp;

	public TicketServiceImpl(AgentWrapperProvider awp) {
		this.awp = awp;
	}

	private final Map<Ticket, Resource> loggedInUsers = new ConcurrentHashMap<Ticket, Resource>();

	@Override
	public void doLogOut(Ticket ticket) {
		this.loggedInUsers.remove(ticket);
	}

	@Override
	public Ticket doLogin(String username, String password) throws AuthenticationFailedException {
		AgentWrapper userManager = this.awp.getAgentWrapper(infra.UserManagerAgent);

		//ask the userManager for the anme of the PUA
		ActionArgumentContainer arguments = new ActionArgumentContainer();
		arguments.addArgument(user.username, new Literal(username));
		String puaName = userManager.syncAction(user.puaquery, arguments);
		if (puaName.equals("")) {
			throw new AuthenticationFailedException();
		}
		Resource puaResource;
		try {
			puaResource = new Resource(puaName);
		} catch (InvalidResourceException e) {
			throw new Error("An agent name must always be a valid resource", e);
		}

		//ask the pua for authentication:
		AgentWrapper PUA = this.awp.getAgentWrapper(puaResource);

		ActionArgumentContainer loginArguments = new ActionArgumentContainer();
		loginArguments.addArgument(user.password, new Literal(password));
		String puaAnswer = PUA.syncAction(user.login, loginArguments);
		if (!":login :is :LoginOK".equals(puaAnswer)) {
			throw new AuthenticationFailedException();
		}

		Ticket t = new Ticket();
		this.loggedInUsers.put(t, puaResource);
		return t;
	}

	@Override
	public boolean isLoggedIn(Ticket ticket) {
		boolean loggedIn = this.loggedInUsers.containsKey(ticket);
		return loggedIn;
	}

	@Override
	public Resource getPersonalUseragent(Ticket ticket) {
		Resource user = this.loggedInUsers.get(ticket);
		return user;
	}
}
