package ubiware.infrastructure.web;

import java.io.File;
import java.util.Set;

import org.eclipse.jetty.server.Handler;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.handler.ContextHandler;
import org.eclipse.jetty.server.handler.ContextHandlerCollection;
import org.eclipse.jetty.server.handler.HandlerCollection;
import org.eclipse.jetty.servlet.FilterHolder;
import org.eclipse.jetty.servlet.FilterMapping;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.servlet.ServletHolder;
import org.eclipse.jetty.webapp.WebAppContext;

import ubiware.core.UbiwareAgent;
import ubiware.util.saplbuilder.Resource;

public class WebServer extends Server {

	private final UbiwareAgent theWebInterfaceAgent;

	private final TicketService ticketService;

	public WebServer(int port, UbiwareAgent agent) throws Exception {
		super(port);
		this.theWebInterfaceAgent = agent;
		this.ticketService = new TicketServiceImpl(new AgentWrapperProvider(this.theWebInterfaceAgent));
		ContextHandlerCollection contextHandlerCollection = new ContextHandlerCollection();

		ServletContextHandler platformHandler = new ServletContextHandler();
		platformHandler.setContextPath("/platform");
		ServletHolder loginHolder = new ServletHolder(new DoLoginServlet(this.ticketService));
		platformHandler.addServlet(loginHolder, "/doLogin");

		ServletHolder logoutHolder = new ServletHolder(new DoLogoutServlet(this.ticketService));
		platformHandler.addServlet(logoutHolder, "/doLogout");

		contextHandlerCollection.addHandler(platformHandler);

		this.setHandler(contextHandlerCollection);
		this.start();

	}

	/**
	 * 
	 * @param contextPath
	 *            the context on which this app must be deployed.
	 * @param displayName
	 *            The displayName of the application
	 * @param warFile
	 *            the war file to be deployed
	 * @param infraAgents
	 *            set of all agents which are infrastructure agents for this application (empty set if none)
	 * @param service
	 *            can be null if this application does not have workers
	 * @throws Exception
	 */
	void deployWar(String contextPath, String displayName, File warFile, Set<Resource> infraAgents, Resource service) throws Exception {
		HandlerCollection hc = (HandlerCollection) this.getHandler();
		WebAppContext webapp = new WebAppContext();
		webapp.setContextPath(contextPath);
		webapp.setDisplayName(displayName);
		webapp.setWar(warFile.getAbsolutePath());
		AgentWrapperProvider awp = new AgentWrapperProvider(this.theWebInterfaceAgent);
		InfraAgentWrapperProvider AIAProvider = new InfraAgentWrapperProvider(infraAgents, awp);
		PlatformConnectionFilter filter = new PlatformConnectionFilter(this.ticketService, awp, service, AIAProvider);
		FilterHolder fh = new FilterHolder(filter);
		webapp.addFilter(fh, "/*", FilterMapping.REQUEST);
		Handler[] chc = hc.getHandlers();
		if (chc != null) {
			for (Handler handler : chc) {
				ContextHandler webhandler = (ContextHandler) handler;
				if (displayName.equals(webhandler.getDisplayName())) {
					// found webapp with same name -> undeploying old one
					webhandler.stop();
					System.out.println("Stopped web application: " + webhandler.getDisplayName());

				}
			}
		}

		hc.addHandler(webapp);
		webapp.start();
		//System.out.println("webapp started");
	}

}
