package ubiware.rdf2bean.core;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Type;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import ubiware.util.saplbuilder.InvalidResourceException;
import ubiware.util.saplbuilder.Resource;

public abstract class RDFResource {
	private String uri;
	private Storage storage;
	// TODO the whole properties map should be Map<Resource, Set<String>>
	protected Map<String, Set<String>> properties = new HashMap<String, Set<String>>();

	public static <T> T giveMeResourceInstance(String resourceURI, Class<T> x, Map<String, Set<String>> map, Storage s) throws InstantiationException, IllegalAccessException,
			InvocationTargetException {
		Constructor<?>[] constructors = x.getConstructors();
		for (Constructor<?> c : constructors) {
			Type[] types = c.getGenericParameterTypes();
			if (types.length == 3) {
				//TODO: finish this, otherwise it's very dangerous
				/*
				Type type1 = types[0];
				Type type2 = types[1];
				Map<String, String> testMap = new HashMap<String, String>();
				if (type1.toString().equals(String.class.toString()) && type2.toString().equals(testMap.getClass().toString())) {
					System.out.println("great!");
				} else {
					System.out.println("not great :(");
					String str1 = type2.toString();
					String str2 = testMap.getClass().toString();
					System.out.println(str1);
					System.out.println(str2);
					System.out.println(str1.equals(str2));
				}
				*/
				T obj = (T) c.newInstance(resourceURI, map, s);
				//System.out.println("!!!!! " + obj);
				return obj;
			}
		}
		//TODO: take it away
		throw new IllegalStateException("Contact Michal Nagy. This is a very weird exception that should never be thrown.");
	}

	public RDFResource(String resourceURI, Map<String, Set<String>> props, Storage s) {
		if ((resourceURI == null) || (props == null) || (s == null)) {
			throw new RuntimeException("All the parameters of the constructor must be non-null");
		}
		this.uri = resourceURI;
		this.properties = props;
		this.storage = s;
	}

	protected Storage getStorage() {
		return this.storage;
	}

	public String getUri() {
		return this.uri;
	}

	public Resource getResourceURI() {
		try {
			return new Resource('<' + this.uri + '>');
		} catch (InvalidResourceException e) {
			throw new Error("the resource of a RDFresource must never be invalid", e);
		}
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder("[" + this.uri + ",");
		Set<String> keys = this.properties.keySet();
		for (String k : keys) {
			sb.append("(");
			sb.append(k);
			sb.append("=");
			sb.append(this.properties.get(k));
			sb.append("),");
		}
		sb.append("]");
		return sb.toString();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = (prime * result) + ((this.uri == null) ? 0 : this.uri.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (this.getClass() != obj.getClass()) {
			return false;
		}
		RDFResource other = (RDFResource) obj;
		if (this.uri == null) {
			if (other.uri != null) {
				return false;
			}
		} else if (!this.uri.equals(other.uri)) {
			return false;
		}
		return true;
	}

	protected String getSingleStringValue(Resource r) throws StorageException {
		String truncatedResourceString = RDFUtils.getRidOfBrackets(r);
		Set<String> set = this.properties.get(truncatedResourceString);
		if (set == null) {
			return null;
			//throw new Exception("The value cannot be found");
		}
		int setSize = set.size();
		if (setSize == 0) {
			return null;
		} else if (setSize == 1) {
			return set.iterator().next();
		} else {
			//return set.iterator().next();
			// TODO: for the future it might be more reasonable to do this:
			throw new StorageException("Property " + r + " can have at most one value");
		}
	}

	// This is messed up because of backwards compatibility
	// It seems that we started to use this class in a read-only way and therefore there are some inconsistencies
	// On inconsistency: In this method we should check if the property is an object property, because only then you can return a resource - but we don't do it 
	protected Resource getSingleResourceValue(Resource property) throws StorageException {
		String truncatedResourceString = RDFUtils.getRidOfBrackets(property);
		Set<String> set = this.properties.get(truncatedResourceString);
		if (set == null) {
			return null;
			//throw new Exception("The value cannot be found");
		}
		int setSize = set.size();
		if (setSize == 0) {
			return null;
		} else if (setSize == 1) {
			String stringValue = set.iterator().next();
			try {
				return new Resource("<" + stringValue + ">");
			} catch (InvalidResourceException e) {
				e.printStackTrace();
				throw new StorageException("Property " + property + " is expected to be an object property, however it's value is '" + stringValue + "'", e);
			}
		} else {
			//return set.iterator().next();
			// TODO: for the future it might be more reasonable to do this:
			throw new StorageException("Property " + property + " can have at most one value");
		}
	}
}
