package ubiware.rdf2bean.social;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Map;
import java.util.Set;

import org.openrdf.query.MalformedQueryException;
import org.openrdf.query.QueryEvaluationException;
import org.openrdf.repository.RepositoryException;
import org.openrdf.rio.RDFParseException;

import ubiware.rdf2bean.core.Storage;
import ubiware.rdf2bean.social.helper.SocialHelper;

public class AdapterOAuth1 extends Adapter {
	//; oauth1:consumerKey ?consumerKey
	//; oauth1:consumerSecret consumerSecret
	//; oauth1:requestTokenURL ?tokenRequestURL
	//; oauth1:accessTokenURL ?accessTokenURL
	//; oauth1:authorizeURL ?authorizeURL

	//	private String consumerKey;
	//	private String consumerSecret;
	//	private String requestTokenURL;
	//	private String accessTokenURL;
	//	private String authorizeURL;

	public static String handledResource = "http://www.ubiware.jyu.fi/apps/socialNetworkApp#AdapterOAuth1";

	public AdapterOAuth1(String resourceURI, Map<String, Set<String>> props, Storage s) {
		super(resourceURI, props, s);
	}

	public String getConsumerKey() {
		String propertyName = "consumerKey";
		return SocialHelper.getSingleStringValue(this.properties, SocialHelper.NS_OAUTH1, propertyName);
	}

	public void setConsumerKey(String value) {
		SocialHelper.setSingleStringValue(this.properties, SocialHelper.NS_OAUTH1, "consumerKey", value);
	}

	public String getConsumerSecret() {
		String propertyName = "consumerSecret";
		return SocialHelper.getSingleStringValue(this.properties, SocialHelper.NS_OAUTH1, propertyName);
	}

	public void setConsumerSecret(String value) {
		SocialHelper.setSingleStringValue(this.properties, SocialHelper.NS_OAUTH1, "consumerSecret", value);
	}

	public String getRequestTokenURL() {
		String propertyName = "requestTokenURL";
		return SocialHelper.getSingleStringValue(this.properties, SocialHelper.NS_OAUTH1, propertyName);
	}

	public void setRequestTokenURL(String value) {
		SocialHelper.setSingleStringValue(this.properties, SocialHelper.NS_OAUTH1, "requestTokenURL", value);
	}

	public String getAccessTokenURL() {
		String propertyName = "accessTokenURL";
		return SocialHelper.getSingleStringValue(this.properties, SocialHelper.NS_OAUTH1, propertyName);
	}

	public void setAccessTokenURL(String value) {
		SocialHelper.setSingleStringValue(this.properties, SocialHelper.NS_OAUTH1, "accessTokenURL", value);
	}

	public String getAuthorizeURL() {
		String propertyName = "authorizeURL";
		return SocialHelper.getSingleStringValue(this.properties, SocialHelper.NS_OAUTH1, propertyName);
	}

	public void setAuthorizeURL(String value) {
		SocialHelper.setSingleStringValue(this.properties, SocialHelper.NS_OAUTH1, "authorizeURL", value);
	}

	//to test adapter
	public static void main(String[] args) throws SecurityException, NoSuchFieldException, RepositoryException, RDFParseException,
			MalformedQueryException, QueryEvaluationException, IllegalArgumentException, IOException, InstantiationException,
			IllegalAccessException, InvocationTargetException {
		String rdfCodeAsString = "@prefix soc: <http://www.ubiware.jyu.fi/apps/socialNetworkApp#> . @prefix rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#> . @prefix oauth: <http://oauth.net/> . @prefix oauth1: <http://tools.ietf.org/rfc/rfc5849#> . 	@prefix oauth2: <http://oauth.net/version2#> ."
				+ "<isbn:adapter1> rdf:type soc:AdapterOAuth1 "
				+ " ; oauth:version \"1.0a\""
				+ " ; oauth:name \"twitter adapter\""
				+ " ; oauth:authenticated \"false\""
				+ " ; soc:handles soc:TwitterProfile "
				+ " ; oauth1:consumerKey \"61vfRSzrtseHIXteC6WX0g\""
				+ " ; oauth1:consumerSecret \"QbLzItQrXB6ubtJRaEOCYYoyfUteuh80R1wsSeU1zDY\""
				+ " ; oauth1:requestTokenURL \"https://api.twitter.com/oauth/request_token\""
				+ " ; oauth1:accessTokenURL \"https://api.twitter.com/oauth/access_token\""
				+ " ; oauth1:authorizeURL \"https://api.twitter.com/oauth/authorize\" . \n";
		Storage s = SocialHelper.getDefaultStorage();
		s.register(AdapterOAuth1.class, AdapterOAuth1.handledResource);
		s.read(rdfCodeAsString);

		AdapterOAuth1 ad = s.getAllResourcesOfThisType(AdapterOAuth1.class).iterator().next();
		System.out.println(ad.getUri());
		System.out.println(ad.getAuthenticated());
		//	System.out.println(ad.getHandles());
		System.out.println(ad.getName());
		//	System.out.println(ad.getVersion());

		System.out.println(ad.getConsumerKey());
		System.out.println(ad.getConsumerSecret());
		System.out.println(ad.getRequestTokenURL());
		System.out.println(ad.getAccessTokenURL());
		System.out.println(ad.getAuthorizeURL());

	}
}
