package ubiware.rdf2bean.social;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.openrdf.query.MalformedQueryException;
import org.openrdf.query.QueryEvaluationException;
import org.openrdf.repository.RepositoryException;

import ubiware.rdf2bean.core.RDFResource;
import ubiware.rdf2bean.core.Storage;
import ubiware.rdf2bean.social.helper.SocialHelper;

public class PersonalUserNetwork extends RDFResource {
	public static String handledResource = "http://www.ubiware.jyu.fi/apps/socialNetworkApp#PersonalUserNetwork";

	public PersonalUserNetwork(String resourceURI, Map<String, Set<String>> props, Storage s) {
		super(resourceURI, props, s);
	}

	private static final long serialVersionUID = 5627674280630639539L;

	//private List<HumanConnection> hasConnection;

	public List<HumanConnection> getHasConnection() {
		String propertyName = "hasConnection";
		return SocialHelper.getMultipleNonStringValues(this.getStorage(), this.properties, propertyName, HumanConnection.class);
	}

	public void setHasConnection(List<HumanConnection> listHumanConnections) {
		String propertyName = "hasConnection";
		SocialHelper.setMultipleNonStringValues(this.getStorage(), this.properties, propertyName, listHumanConnections);
	}

	/** Start - Convenience methods for Mashupper webapp **/

	public List<SocialNetworkProfile> getProfilesOfType(String profileType) {

		List<SocialNetworkProfile> list = new ArrayList<SocialNetworkProfile>();
		if (this.getHasConnection() == null) {
			return list;
		}
		int i = 0;
		for (HumanConnection hm : this.getHasConnection()) {
			boolean isEmpty = true;
			boolean isHMEmpty = true;
			for (SocialNetworkProfile profile : hm.getHasProfile()) {
				//System.out.println("profile name: " + profile.getHasDisplayName());				
				if ("Facebook".equals(profileType) && (profile instanceof FacebookProfile)) {
					list.add(profile);
					isEmpty = false;
				} else if ("LinkedIn".equals(profileType) && (profile instanceof LinkedInProfile)) {
					list.add(profile);
					isEmpty = false;
				} else if ("Twitter".equals(profileType) && (profile instanceof TwitterProfile)) {
					list.add(profile);
					isEmpty = false;
				} else if ("PhoneBook".equals(profileType) && (profile instanceof PhoneBookProfile)) {
					list.add(profile);
					isEmpty = false;
				}

				if (!profile.getUri().equals("empty")) {
					isHMEmpty = false;
				}

			}
			if (isEmpty && !isHMEmpty) {
				//if(isEmpty) {
				// check if both are empty
				list.add(new SocialNetworkProfile(hm.getUri() + "_empty_" + i, new HashMap<String, Set<String>>(), this.getStorage()));
			}
			i++;
		}
		return list;

	}

	public void addProfiles(List<SocialNetworkProfile> profiles) {
		for (SocialNetworkProfile p : profiles) {

			HumanConnection hm = this.findMatch(p);

			if (hm == null) {
				hm = this.newHumanConnection();
				List<SocialNetworkProfile> newProfiles = new ArrayList<SocialNetworkProfile>();
				newProfiles.add(p);
				List<HumanConnection> hms = this.getHasConnection();
				if (hms == null) {
					hms = new ArrayList<HumanConnection>();
					this.setHasConnection(hms);
				}
				hms.add(hm);
				hm.setHasProfile(newProfiles);
				this.setHasConnection(hms);

				try {
					Thread.sleep(20);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

			} else {
				if (!hm.getHasProfile().contains(p)) {
					List<SocialNetworkProfile> oldProfiles = hm.getHasProfile();
					oldProfiles.add(p);
					hm.setHasProfile(oldProfiles);
				}
			}

		}
	}

	private HumanConnection newHumanConnection() {
		HumanConnection hm = new HumanConnection("http://www.ubiware.jyu.fi/apps/socialNetworkApp#hm_" + System.currentTimeMillis(),
				SocialHelper.getDefaultProperties(HumanConnection.handledResource), this.getStorage());
		return hm;
	}

	public void print() {
		System.out.println("Personal User Network:");
		if (this.getHasConnection() != null) {
			for (HumanConnection hm : this.getHasConnection()) {
				System.out.println("Human connection: " + hm.getUri());
				for (SocialNetworkProfile profile : hm.getHasProfile()) {
					System.out.println("\t - " + profile.getUri() + " : " + profile.getHasDisplayName() + " URI:" + profile.getUri());
				}
			}
		}
	}

	private HumanConnection findMatch(SocialNetworkProfile p) {
		//return (HumanConnection) CollectionUtils.find(connections, new HumanConnectionMatcher(p));
		if (this.getHasConnection() == null) {
			return null;
		}
		for (HumanConnection hm : this.getHasConnection()) {
			if (hm.getHasProfile().contains(p)) {
				return hm;
			}

			for (SocialNetworkProfile _p : hm.getHasProfile()) {
				if ((p.getHasDisplayName() != null) && (_p.getHasDisplayName() != null)
						&& p.getHasDisplayName().equals(_p.getHasDisplayName())) {
					return hm;
				}
			}
		}
		return null;
	}

	/**
	 * 
	 * @param sourceIndex
	 *            the index of the HumanConnection where drag initiated
	 * @param targetIndex
	 *            the index of the HumanConnection where item was dragged
	 * @param sourceProfile
	 *            the profile being dragged
	 * @param targetProfile
	 *            the profile on top of which the source is dropped
	 */
	public void switchPlaces(int sourceIndex, int targetIndex, SocialNetworkProfile sourceProfile, SocialNetworkProfile targetProfile) {

		HumanConnection targetHm = (this.getHasConnection()).get(targetIndex);
		HumanConnection sourceHm = (this.getHasConnection()).get(sourceIndex);

		List<SocialNetworkProfile> sourceProfiles = null;
		List<SocialNetworkProfile> targetProfiles = null;

		if ((targetProfile.getHasDisplayName() == null) || targetProfile.getHasDisplayName().equals("empty")) {
			// drag actual item on top of empty
			// remove sourceProfile from sourceHm and 
			// add sourceProfile to the targetHm
			sourceProfiles = sourceHm.getHasProfile();
			targetProfiles = targetHm.getHasProfile();
			sourceProfiles.remove(sourceProfile);
			targetProfiles.add(sourceProfile);
			sourceHm.setHasProfile(sourceProfiles);
			targetHm.setHasProfile(targetProfiles);
			//sourceHm.getHasProfile().remove(sourceProfile);
			//targetHm.getHasProfile().add(sourceProfile);
		} else if ((sourceProfile.getHasDisplayName() == null) || sourceProfile.getHasDisplayName().equals("empty")) {
			// dragging empty on an actual item
			// add targetProfile to the sourceHm
			// remove targetProfile from the targetHm
			sourceProfiles = sourceHm.getHasProfile();
			targetProfiles = targetHm.getHasProfile();
			sourceProfiles.add(targetProfile);
			targetProfiles.remove(targetProfile);
			sourceHm.setHasProfile(sourceProfiles);
			targetHm.setHasProfile(targetProfiles);

			//targetHm.getHasProfile().remove(targetProfile);
			//sourceHm.getHasProfile().add(targetProfile);
		} else {
			// switching actual items
			sourceProfiles = sourceHm.getHasProfile();
			targetProfiles = targetHm.getHasProfile();
			sourceProfiles.add(targetProfile);
			sourceProfiles.remove(sourceProfile);
			targetProfiles.add(sourceProfile);
			targetProfiles.remove(targetProfile);
			sourceHm.setHasProfile(sourceProfiles);
			targetHm.setHasProfile(targetProfiles);

			//targetHm.getHasProfile().add(sourceProfile);
			//sourceHm.getHasProfile().remove(sourceProfile);			
			//sourceHm.getHasProfile().add(targetProfile);
			//targetHm.getHasProfile().remove(targetProfile);

		}

	}

	public void setupHumanConnectionNames() {
		if (this.getHasConnection() == null) {
			return;
		}
		for (HumanConnection hm : this.getHasConnection()) {
			// priorities: fb, li, tw
			for (SocialNetworkProfile p : hm.getHasProfile()) {
				if (p instanceof FacebookProfile) {
					hm.setHasDisplayName(p.getHasDisplayName());
					break;
				}

			}
			if (hm.getHasDisplayName() == null) {
				for (SocialNetworkProfile p : hm.getHasProfile()) {
					if (p instanceof LinkedInProfile) {
						hm.setHasDisplayName(p.getHasDisplayName());
						break;
					}
				}
			}
			if (hm.getHasDisplayName() == null) {
				for (SocialNetworkProfile p : hm.getHasProfile()) {
					if (p instanceof TwitterProfile) {
						hm.setHasDisplayName(p.getHasDisplayName());
						break;
					}
				}
			}
			if (hm.getHasDisplayName() == null) {
				for (SocialNetworkProfile p : hm.getHasProfile()) {
					if (p instanceof PhoneBookProfile) {
						hm.setHasDisplayName(p.getHasDisplayName());
						break;
					}
				}
			}

			if (hm.getHasDisplayName() == null) {
				hm.setHasDisplayName("empty");
			}

		}
	}

	public void removeEmptyRows() {
		List<HumanConnection> list = this.getHasConnection();
		List<HumanConnection> newList = new ArrayList<HumanConnection>();
		for (HumanConnection hm : list) {

			System.out.println(hm.getHasDisplayName());

			//if(hm.getProfileByType(FacebookProfile.class) == null && hm.getProfileByType(LinkedInProfile.class) == null && hm.getProfileByType(TwitterProfile.class) == null)
			if ((hm.getHasProfile() != null) && (hm.getHasProfile().size() > 0)) {
				newList.add(hm);
			}
		}
		this.setHasConnection(newList);

	}

	public String toRdf() throws Exception {
		return this.getStorage().getStringRepresentation();
	}

	/**
	 * Method for splitting a HumanConnection. Creates a new HumanConnection object with the targeted SocialNetworkProfile alone in it.
	 * 
	 * @param profile
	 *            The SocialNetworkProfile targeted for splitting
	 * @param connectionIndex
	 *            The index identifying the to-be-split HumanConnection
	 * @return true if split was done
	 */
	public boolean split(SocialNetworkProfile profile, int connectionIndex) {
		// remove given profile from the connection as ConnectionInder
		// add new connection after the connectionIndex and 
		// add profile to the new connection

		System.out.println("Splitting profile " + profile.getUri() + " with connection index " + connectionIndex);
		System.out.println("Before split: ");
		this.print();
		List<HumanConnection> hmList = this.getHasConnection();
		HumanConnection oldHm = hmList.get(connectionIndex);

		if (oldHm.getHasProfile().size() <= 1) {
			return false;
		}

		List<SocialNetworkProfile> oldProfiles = oldHm.getHasProfile();
		oldProfiles.remove(profile);
		oldHm.setHasProfile(oldProfiles);

		if (oldHm.getHasDisplayName().equals(profile.getHasDisplayName())) {
			String name = oldProfiles.get(0).getHasDisplayName();
			oldHm.setHasDisplayName(name);
		}

		HumanConnection newHm = this.newHumanConnection();
		List<SocialNetworkProfile> newProfiles = new ArrayList<SocialNetworkProfile>();
		newProfiles.add(profile);
		newHm.setHasProfile(newProfiles);

		hmList.add(newHm);
		this.setHasConnection(hmList);

		System.out.println("After split: ");
		this.print();
		return true;
	}

	/** End - Convenience methods for Mashupper webapp **/

	public static void main(String[] args) throws QueryEvaluationException, RepositoryException, MalformedQueryException {
		Storage s = SocialHelper.getDefaultStorage();

		PersonalUserNetwork mypun = new PersonalUserNetwork("http://test.com/test", SocialHelper
				.getDefaultProperties(PersonalUserNetwork.handledResource), s);

		FacebookProfile fbProfile = new FacebookProfile("http://profile.com/profile", SocialHelper
				.getDefaultProperties(FacebookProfile.handledResource), s);

		HumanConnection hc1 = new HumanConnection("http://humancon.com/hc1", SocialHelper
				.getDefaultProperties(HumanConnection.handledResource), s);

		List<SocialNetworkProfile> profiles = new ArrayList<SocialNetworkProfile>();
		profiles.add(fbProfile);
		hc1.setHasProfile(profiles);

		List<HumanConnection> listHumanConnections = new ArrayList<HumanConnection>();
		listHumanConnections.add(hc1);
		mypun.setHasConnection(listHumanConnections);

		s.addResource(mypun);
		System.out.println(s.getStringRepresentation());
	}

}
