package ubiware.rdf2bean.social;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;
import java.util.Set;

import ubiware.rdf2bean.core.RDFResource;
import ubiware.rdf2bean.core.Storage;
import ubiware.rdf2bean.social.helper.SocialHelper;

public class Status extends RDFResource implements Comparable<Status> {
	DateFormat df = new SimpleDateFormat(SocialHelper.DATE_FORMAT);

	public Status(String resourceURI, Map<String, Set<String>> props, Storage s) {
		super(resourceURI, props, s);
	}

	private static final long serialVersionUID = 1L;
	public static String handledResource = "http://www.ubiware.jyu.fi/apps/socialNetworkApp#Status";

	//private String hasText;

	//private GeoLocation hasGeoLocation;
	//private Date createdAt;

	private String hasContributors;
	private String hasID;
	private String inReplyToScreenName;
	private String inReplyToStatusId;
	private String inReplyToUserId;
	private String hasPlace;
	private String hasRetweetedStatus;

	//private String hasLongitude;
	//private String hasLatitude;
	//private String hasSource;

	public GeoLocation getHasGeoLocation() {
		String propertyName = "hasGeoLocation";
		return SocialHelper.getSingleNonStringValue(this.getStorage(), this.properties, propertyName, GeoLocation.class);
	}

	public void setHasGeoLocation(GeoLocation hasGeoLocation) {
		String propertyName = "hasGeoLocation";
		SocialHelper.setSingleNonStringValue(this.getStorage(), this.properties, propertyName, hasGeoLocation);
	}

	public String getHasSource() {
		String propertyName = "hasSource";
		return SocialHelper.getSingleStringValue(this.properties, SocialHelper.NS_SOCIAL, propertyName);
	}

	public void setHasSource(String value) {
		SocialHelper.setSingleStringValue(this.properties, SocialHelper.NS_SOCIAL, "hasSource", value);
	}

	public String getHasID() {
		String propertyName = "hasID";
		return SocialHelper.getSingleStringValue(this.properties, SocialHelper.NS_SOCIAL, propertyName);
	}

	public void setHasID(String value) {
		SocialHelper.setSingleStringValue(this.properties, SocialHelper.NS_SOCIAL, "hasID", value);
	}

	private String hasUser;
	private String isFavorited;
	private String isRetweet;
	private String isTruncated;

	public String getHasText() {
		String propertyName = "hasText";
		return SocialHelper.getSingleStringValue(this.properties, SocialHelper.NS_SOCIAL, propertyName);
	}

	public void setHasText(String value) {
		SocialHelper.setSingleStringValue(this.properties, SocialHelper.NS_SOCIAL, "hasText", value);
	}

	public Date getCreatedAt() {
		String propertyName = "createdAt";
		String stringDate = SocialHelper.getSingleStringValue(this.properties, SocialHelper.NS_SOCIAL, propertyName);
		Date dateDate;
		try {
			dateDate = this.df.parse(stringDate);
		} catch (ParseException e) {
			dateDate = null;
		}
		return dateDate;
	}

	public void setCreatedAt(Date value) {
		SocialHelper.setSingleStringValue(this.properties, SocialHelper.NS_SOCIAL, "createdAt", this.df.format(value));
	}

	@Override
	public int compareTo(Status otherStatus) {
		Date myDate = this.getCreatedAt();
		Date otherDate = otherStatus.getCreatedAt();
		if (myDate == null) {
			return 0;
		}
		return myDate.compareTo(otherDate);
	}

}
