package ubiware.rdf2bean.social;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;
import java.util.Set;

import org.openrdf.query.MalformedQueryException;
import org.openrdf.query.QueryEvaluationException;
import org.openrdf.repository.RepositoryException;
import org.openrdf.rio.RDFParseException;

import ubiware.rdf2bean.core.RDFResource;
import ubiware.rdf2bean.core.Storage;
import ubiware.rdf2bean.social.helper.SocialHelper;

public class WorkExperience extends RDFResource {
	DateFormat df = new SimpleDateFormat(SocialHelper.DATE_FORMAT);
	public static String handledResource = "http://www.ubiware.jyu.fi/apps/socialNetworkApp#WorkExperience";

	public WorkExperience(String resourceURI, Map<String, Set<String>> props, Storage s) {
		super(resourceURI, props, s);
	}

	private String hasID;
	private String hasSummary;
	private String isCurrent;

	//private String hasEmployer; // company name
	//private Date hasEndDate;
	//private Date hasStartDate;
	//private String hasPosition; // position title

	public String getHasEmployer() {
		String propertyName = "hasEmployer";
		return SocialHelper.getSingleStringValue(this.properties, SocialHelper.NS_SOCIAL, propertyName);
	}

	public void setHasEmployer(String value) {
		SocialHelper.setSingleStringValue(this.properties, SocialHelper.NS_SOCIAL, "hasEmployer", value);
	}

	public Date getHasEndDate() {
		String propertyName = "hasEndDate";
		String stringDate = SocialHelper.getSingleStringValue(this.properties, SocialHelper.NS_SOCIAL, propertyName);
		if ((stringDate == null) || "".equals(stringDate)) {
			return null;
		}
		Date dateDate;
		try {
			dateDate = this.df.parse(stringDate);
		} catch (ParseException e) {
			dateDate = null;
		}
		return dateDate;
	}

	public void setHasEndDate(Date value) {
		SocialHelper.setSingleStringValue(this.properties, SocialHelper.NS_SOCIAL, "hasEndDate", this.df.format(value));
	}

	public Date getHasStartDate() {
		String propertyName = "hasStartDate";
		String stringDate = SocialHelper.getSingleStringValue(this.properties, SocialHelper.NS_SOCIAL, propertyName);
		Date dateDate;
		try {
			dateDate = this.df.parse(stringDate);
		} catch (ParseException e) {
			dateDate = null;
		}
		return dateDate;
	}

	public void setHasStartDate(Date value) {
		SocialHelper.setSingleStringValue(this.properties, SocialHelper.NS_SOCIAL, "hasStartDate", this.df.format(value));
	}

	public String getHasPosition() {
		String propertyName = "hasPosition";
		return SocialHelper.getSingleStringValue(this.properties, SocialHelper.NS_SOCIAL, propertyName);
	}

	public void setHasPosition(String value) {
		SocialHelper.setSingleStringValue(this.properties, SocialHelper.NS_SOCIAL, "hasPosition", value);
	}

	public static void main(String[] args) throws SecurityException, NoSuchFieldException, RepositoryException, RDFParseException,
			MalformedQueryException, QueryEvaluationException, IllegalArgumentException, IOException, InstantiationException,
			IllegalAccessException, InvocationTargetException {
		String rdfCodeAsString = "<http://social.ubiware.jyu.fi/facebook/workExperiences#564435942_410977924> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.ubiware.jyu.fi/apps/socialNetworkApp#WorkExperience> . "
				+ "<http://social.ubiware.jyu.fi/facebook/workExperiences#564435942_410977924> <http://www.ubiware.jyu.fi/apps/socialNetworkApp#hasStartDate> \"01-12-0002 00-00-00 EET\" . "
				+ "<http://social.ubiware.jyu.fi/facebook/workExperiences#564435942_410977924> <http://www.ubiware.jyu.fi/apps/socialNetworkApp#hasEndDate> \"22-11-1232 00-00-00 EET\" . "
				+ "<http://social.ubiware.jyu.fi/facebook/workExperiences#564435942_410977924> <http://www.ubiware.jyu.fi/apps/socialNetworkApp#hasPosition> \"researcher\" . "
				+ "<http://social.ubiware.jyu.fi/facebook/workExperiences#564435942_410977924> <http://www.ubiware.jyu.fi/apps/socialNetworkApp#hasEmployer> \"Faculty of Physical Education and Sports, Comenius University, Bratislava, Slovakia\" .";

		Storage s = SocialHelper.getDefaultStorage();
		s.register(WorkExperience.class, WorkExperience.handledResource);
		s.read(rdfCodeAsString);

		WorkExperience we = s.getAllResourcesOfThisType(WorkExperience.class).iterator().next();
		System.out.println(we.getUri());
		System.out.println(we.getHasEmployer());
		System.out.println(we.getHasPosition());
		System.out.println(we.getHasEndDate());
		System.out.println(we.getHasStartDate());
	}
}
