package ubiware.rdf2bean.social.helper;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import ubiware.rdf2bean.core.RDFResource;
import ubiware.rdf2bean.core.Storage;
import ubiware.util.saplbuilder.Resource;

public class SocialHelper {

	public static final String DATE_FORMAT = "dd-MM-yyyy HH-mm-ss z";

	public static final String NS_SOCIAL = "http://www.ubiware.jyu.fi/apps/socialNetworkApp#";
	public static final String NS_OAUTH = "http://oauth.net/";
	public static final String NS_OAUTH1 = "http://tools.ietf.org/rfc/rfc5849#";
	public static final String NS_OAUTH2 = "http://oauth.net/version2#";

	public static String getURI(String ns, String rest) {
		return ns + rest;
	}

	public static String getSingleStringValue(Map<String, Set<String>> properties, String namespace, String propertyName) {
		String uri = SocialHelper.getURI(namespace, propertyName);
		Set<String> set = properties.get(uri);
		if (set == null) {
			return null;
		}
		int setSize = set.size();
		if (setSize == 0) {
			return null;
		} else if (setSize == 1) {
			return set.iterator().next();
		} else {
			return set.iterator().next();
			// TODO: for the future it might be more reasonable to do this:
			// throw new StorageException("Property " + propertyName + " can have at most one value");
		}
	}

	public static void setSingleStringValue(Map<String, Set<String>> properties, String namespace, String propertyName, String value) {
		String uri = SocialHelper.getURI(namespace, propertyName);
		Set<String> set = new HashSet<String>();
		set.add(value);
		properties.put(uri, set);
	}

	public static <T extends RDFResource> T getSingleNonStringValue(Storage s, Map<String, Set<String>> properties, String propertyName,
			Class<T> cl) {
		String propertyNameURI = SocialHelper.getURI(SocialHelper.NS_SOCIAL, propertyName);
		Set<String> valueSet = properties.get(propertyNameURI);
		if (valueSet != null) {
			if (!valueSet.isEmpty()) {
				String uri = valueSet.iterator().next();
				T resource = s.getResourceOfThisTypeWithThisURI(uri, cl);
				// if I have URI, but I cannot find the resource, then I should not return null, but an instance of class T that has just one field filled in - URI
				if (resource == null) {
					try {
						return RDFResource.giveMeResourceInstance(uri, cl, new HashMap<String, Set<String>>(), s);
					} catch (Exception e) {
						// TODO reconsider?
						e.printStackTrace();
						return null;
					}
				} else {
					return resource;
				}

			}
		}
		return null;
	}

	public static <T extends RDFResource> void setSingleNonStringValue(Storage storage, Map<String, Set<String>> properties,
			String propertyName, T element) {
		String propertyNameURI = SocialHelper.getURI(SocialHelper.NS_SOCIAL, propertyName);
		Set<String> set = new HashSet<String>();
		String uri = element.getUri();
		set.add(uri);
		if (!storage.existsResourceWithThisURI(uri)) {
			storage.addResource(element);
		}
		properties.put(propertyNameURI, set);
	}

	public static <T extends RDFResource> List<T> getMultipleNonStringValues(Storage storage, Map<String, Set<String>> properties,
			String propertyName, Class<T> cl) {
		String propertyNameURI = SocialHelper.getURI(SocialHelper.NS_SOCIAL, propertyName);
		List<T> results = new ArrayList<T>();
		Set<String> uris = properties.get(propertyNameURI);
		//System.out.println(propertyName);
		if (uris == null) {
			return null;
		} else {
			for (String uri : uris) {
				T element = storage.getResourceOfThisTypeWithThisURI(uri, cl);
				results.add(element);
			}
			return results;
		}
	}

	public static <T extends RDFResource> void setMultipleNonStringValues(Storage storage, Map<String, Set<String>> properties,
			String propertyName, List<T> list) {
		String propertyNameURI = SocialHelper.getURI(SocialHelper.NS_SOCIAL, propertyName);
		Set<String> set = new HashSet<String>();
		for (T element : list) {
			String uri = element.getUri();
			set.add(uri);
			if (!storage.existsResourceWithThisURI(uri)) {
				storage.addResource(element);
			}
		}
		properties.put(propertyNameURI, set);
	}

	public static Map<String, Set<String>> getDefaultProperties(String className) {
		Map<String, Set<String>> props = new HashMap<String, Set<String>>();
		Set<String> set = new HashSet<String>();
		set.add(className);
		props.put("http://www.w3.org/1999/02/22-rdf-syntax-ns#type", set);
		return props;
	}

	public static String getSingleStringValue(Map<String, Set<String>> properties, Resource uri) {
		String uriString = uri.toString().substring(1, uri.toString().length() - 1);
		Set<String> set = properties.get(uriString);
		if (set == null) {
			return null;
		}
		int setSize = set.size();
		if (setSize == 0) {
			return null;
		} else if (setSize == 1) {
			return set.iterator().next();
		} else {
			return set.iterator().next();
			// TODO: for the future it might be more reasonable to do this:
			// throw new StorageException("Property " + propertyName +
			// " can have at most one value");
		}
	}

	public static String getObligatedSingleStringValue(Map<String, Set<String>> properties, Resource uri) {
		String result = SocialHelper.getSingleStringValue(properties, uri);
		if (result == null) {
			throw new Error("no such string property " + uri);
		}
		return result;
	}

	/**
	 * 
	 * @param properties
	 * @param selected
	 * @return
	 * @throws NullPointerException
	 *             if the value does not exists
	 */
	public static boolean getSingleBooleanValue(Map<String, Set<String>> properties, Resource selected) {
		String string = SocialHelper.getSingleStringValue(properties, selected);
		return Boolean.parseBoolean(string);
	}

	public static boolean getSingleOptionalBooleanValue(Map<String, Set<String>> properties, Resource selected, boolean defaultValue) {
		String string = SocialHelper.getSingleStringValue(properties, selected);
		if (string == null) {
			return defaultValue;
		}
		return Boolean.parseBoolean(string);
	}

	public static Storage getDefaultStorage() {
		Set<String> props = new HashSet<String>();
		props.add("http://www.ubiware.jyu.fi/apps/socialNetworkApp#hasConnection");
		props.add("http://www.ubiware.jyu.fi/apps/socialNetworkApp#hasProfile");
		props.add("http://www.ubiware.jyu.fi/apps/socialNetworkApp#hasGeoLocation");
		props.add("http://www.ubiware.jyu.fi/apps/socialNetworkApp#hasSender");
		props.add("http://www.ubiware.jyu.fi/apps/socialNetworkApp#hasReceiver");
		props.add("http://www.ubiware.jyu.fi/apps/socialNetworkApp#sentFrom");
		props.add("http://www.w3.org/1999/02/22-rdf-syntax-ns#type");

		//props.add("http://www.ubiware.jyu.fi/apps/socialNetworkApp#");
		//props.add("http://www.ubiware.jyu.fi/apps/socialNetworkApp#");
		//props.add("http://www.ubiware.jyu.fi/apps/socialNetworkApp#");
		Storage s = new Storage(props);
		return s;
	}

}
