package ubiware.shared;

import java.io.File;
import java.io.IOException;
import java.util.List;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.SaplN3;
import ubiware.core.UbiwareAgent;
import ubiware.core.IllegalParameterConfigurationException.State;
import ubiware.ontology.param;
import ubiware.util.StreamingUtils;

public class BeliefsLoadBehavior extends ReusableAtomicBehavior {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private String input;
	private File file;
	private String output;
	private boolean doChecks;

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.input = parameters.getOptionalStringParameter(param.input);
		this.file = parameters.getOptionalFileParameter(param.inputFromFile);
		if ((this.input == null) && (this.file == null)) {
			throw new IllegalParameterConfigurationException(param.input, this.input, State.missing);
		}
		if ((this.input != null) && (this.file != null)) {
			throw new IllegalParameterConfigurationException(param.messageFromFile, this.file.getName(), State.conflicting);
		}

		this.output = parameters.getObligatedNonEmptyStringParameter(param.saveTo);
		if (this.output.equals(UbiwareAgent.GENERAL_CONTEXT)) {
			this.output = UbiwareAgent.GENERAL_CONTEXT;
		}
		this.doChecks = parameters.getOptionalBooleanParameter(param.doChecks, true);

	}

	@Override
	protected void doAction() {
		String beliefs;
		if (this.input == null) {
			try {
				beliefs = StreamingUtils.getInputFromFile(this.file);
			} catch (IOException e) {
				e.printStackTrace();
				this.setFailed();
				return;
			}
		} else {
			//this.input is guaranteed to not be null, see parameter initialization
			beliefs = this.input;
		}
		String compilationUnitName = (this.file == null) ? this.getClass().getName() : this.file.getName();
		List<String> code = SaplN3.compile(beliefs, compilationUnitName);
		if (code == null) {
			this.setFailed();
			return;
		}

		SaplN3.load(code, this.myAgent, this.output, compilationUnitName, this.doChecks);
		this.wakeAgent();
	}

}
