package ubiware.shared;

import jade.wrapper.AgentContainer;
import jade.wrapper.AgentController;
import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.UbiwareAgent;
import ubiware.core.commands.AddBeliefsN3Command;
import ubiware.ontology.param;
import ubiware.util.saplbuilder.SaplDocument;

public class CreateAgentBehavior extends ReusableAtomicBehavior {

	private String name;
	private Object[] params;
	private SaplDocument initBeliefs;

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.name = parameters.getObligatedResourceParameter(param.name).toString();
		String scripts = parameters.getOptionalStringParameter(param.scripts, "");
		String roles = parameters.getOptionalStringParameter(param.roles);
		// container ID of a container holding initial beliefs of the agent
		String containerInitBeliefs = parameters.getOptionalStringParameter(param.initBeliefs);

		// if parameter p:initBliefs was not specified, then containerInitBeliefs is null 
		if (containerInitBeliefs != null) {
			this.initBeliefs = this.produceN3Document(containerInitBeliefs);
		} else {
			this.initBeliefs = null;
		}

		if (roles == null) {
			this.params = new Object[1];
			this.params[0] = scripts;
		} else {
			this.params = new Object[2];
			this.params[0] = scripts;
			this.params[1] = roles;
		}
	}

	@Override
	protected void doAction() {
		try {
			AgentContainer ac = this.myAgent.getContainerController();
			UbiwareAgent agentToStart = new UbiwareAgent();
			// if the user specified some initial beliefs, use a Command object to feed them into the agent 
			if (this.initBeliefs != null) {
				agentToStart.addCommand(new AddBeliefsN3Command(this.initBeliefs));
			}
			agentToStart.setArguments(this.params);
			AgentController actl = ac.acceptNewAgent(this.name, agentToStart);
			actl.start();

		} catch (Throwable t) {
			//TODO check why Throwable is caught.
			t.printStackTrace();
			this.setFailed();
		}
	}
}
