package ubiware.shared;

import java.io.FileInputStream;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.RABUtils;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.ontology.param;

public class ExcelReaderBehavior extends ReusableAtomicBehavior {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private String filename;

	private String getValue(HSSFCell cell) {
		if (cell == null) {
			return null;
		}
		int type = cell.getCellType();
		String value;
		if ((type == Cell.CELL_TYPE_NUMERIC) || (type == Cell.CELL_TYPE_FORMULA)) {
			Double v = cell.getNumericCellValue();
			double d = Math.floor(v);
			if (d == v) {
				value = String.valueOf((long) d);
			} else {
				value = v.toString();
			}
		} else {
			value = cell.getStringCellValue();
		}
		return value;
	}

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.filename = parameters.getParameterValue(param.input);
		if (this.filename == null) {
			this.print(this.getClass().getName() + " error: Mandatory parameter \"input\" is not given");
			this.setFailed();
			return;
		}
	}

	@Override
	public void doAction() {

		try {
			HSSFWorkbook workbook = new HSSFWorkbook(new FileInputStream(this.filename));
			HSSFSheet sheet = workbook.getSheetAt(0);

			int RN = sheet.getLastRowNum();

			HSSFRow row = sheet.getRow(0);

			short RC = row.getLastCellNum();
			String labels[] = new String[RC];

			for (short iCol = 0; iCol < RC; iCol++) {
				labels[iCol] = RABUtils.escapeString(this.getValue(row.getCell(iCol)));
			}

			StringBuilder beliefs = new StringBuilder();
			beliefs.append(this.filename + " :table {");

			boolean foundRow = false;
			for (int iRow = 1; iRow <= RN; iRow++) {
				row = sheet.getRow(iRow);
				if (row != null) {
					String rid = String.valueOf(iRow);
					beliefs.append(((foundRow) ? ". " : "") + rid + " :row {");
					foundRow = true;

					boolean foundColumn = false;
					for (short iCol = 0; iCol < RC; iCol++) {
						if (labels[iCol] != null) {
							HSSFCell cell = row.getCell(iCol);
							String value = this.getValue(cell);
							if ((value == null) || value.equals("")) {
								value = ("N/A");
							}
							beliefs.append(((foundColumn) ? ". " : "") + "\"" + labels[iCol] + "\" :column \""
									+ RABUtils.escapeString(value) + "\"");
							foundColumn = true;
						}
					}
					beliefs.append("}");
				}
			}
			beliefs.append("}");

			this.addBeliefsN3(beliefs.toString());
			this.wakeAgent();
		} catch (Exception ex) {
			ex.printStackTrace();
			this.setFailed();
		}

	}
}
