package ubiware.shared;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.ontology.param;

/**
 * from Saplguide.pdf<br/>
 * <h3>Full name: ubiware.shared.ExternalAppStarterBehavior</h3>
 * 
 * <p>
 * Action: Starts an external software application. If the command is successfully executed (it does not necessarily implies that the
 * application is started), ends in success. If an exception occurs, ends in failure.
 * </p>
 * <p>
 * Inputs:
 * <table border="1">
 * <tr>
 * <th>Name</th>
 * <th>Meaning</th>
 * <th>Mandatory</th>
 * <th>Default</th>
 * </tr>
 * <tr>
 * <td>p:execute</td>
 * <td>Command to execute.</td>
 * <td>Yes</td>
 * <td></td>
 * </tr>
 * </table>
 * </p>
 * <p>
 * Outputs: none.
 * </p>
 * <p>
 * Example of usage: Execute the text editor Notepad and open in it file Dog.txt.
 * 
 * <pre>
 * sapl:I :have Dog.
 * {sapl:I :have ?who} =>
 * {{sapl:I sapl:do java:ubiware.shared.ExternalAppStarterBehavior}
 * sapl:configuredAs
 * {p:execute sapl:is notepad.exe ?who.txt}}
 * </pre>
 * 
 * </p>
 * <p>
 * Libraries needed: none.
 * </p>
 * 
 * 24.3.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public class ExternalAppStarterBehavior extends ReusableAtomicBehavior {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private String command;

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.command = parameters.getObligatedStringParameter(param.execute);

	}

	@Override
	protected void doAction() throws Exception {
		try {
			Runtime.getRuntime().exec(this.command);
		} catch (Throwable t) {
			throw new Exception(t);
		}
	}
}
