package ubiware.shared;

import java.io.File;
import java.net.URI;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.ontology.param;

public class FilesSensorBehavior extends ReusableAtomicBehavior {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	String foldername;

	private String senseFolder(File folder) {
		StringBuilder result = new StringBuilder();
		File[] files = folder.listFiles();
		for (int i = 0; i < files.length; i++) {
			File file = files[i];
			URI uri = file.toURI();
			String str_uri = uri.toString();
			String filename = str_uri.substring(str_uri.indexOf(this.foldername));

			if (i > 0) {
				result.append(". ");
			}
			if (file.isDirectory()) {
				result.append(filename + " :folder {" + this.senseFolder(file) + "}");
			} else {
				int ind = filename.lastIndexOf(".");
				String ext;
				if (ind == -1) {
					ext = "empty";
				} else {
					ext = filename.substring(ind);
				}
				result.append(filename + " :file " + ext);
			}
		}
		return result.toString();
	}

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.foldername = parameters.getObligatedStringParameter(param.folder);
	}

	@Override
	public void doAction() {
		StringBuilder beliefs = new StringBuilder();

		File file = new File(this.foldername);
		if (!file.exists()) {
			this.print(this.getClass().getName() + " error: The folder " + this.foldername + " does not exist");
			this.setFailed();
			return;
		}
		if (!file.isDirectory()) {
			this.print(this.getClass().getName() + " error: " + this.foldername + " is not a directory");
			this.setFailed();
			return;
		}

		//URI uri=file.toURI();

		beliefs.append(this.foldername + " :folder {" + this.senseFolder(file) + "}");

		this.addBeliefsN3(beliefs.toString());
		this.wakeAgent();
	}
}
