package ubiware.shared;

import jade.core.AID;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;

import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.RABUtils;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.core.SaplN3Constants;
import ubiware.core.UbiwareAgent;
import ubiware.ontology.param;
import ubiware.util.URITools;
import ubiware.util.UniqueInstanceGenerator;

public class MessageReceiverBehavior extends ReusableAtomicBehavior {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	List<String> matchConversationID = null;
	List<String> matchPerformative = null;
	List<String> matchOntology = null;
	List<String> matchSender = null;
	List<String> matchContent = null;

	String contentIsSAPL = null;

	String print = null;
	int print_length = -1;

	String addBeliefs = null;
	String contentToFile = null;

	String waitOnlyFirst = null;
	String maxWait = null;

	MessageTemplate mt;

	long waitTill = 0;

	private final static URI messageNamespace = URITools.createUncheckedURI("http://ubiware.jyu.fi/messages/received/");

	private final static UniqueInstanceGenerator messageInstanceGenerator = new UniqueInstanceGenerator("",
			MessageReceiverBehavior.messageNamespace);

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.matchConversationID = new ArrayList<String>(parameters.getParameterValues(param.matchConversationID));
		this.matchPerformative = new ArrayList<String>(parameters.getParameterValues(param.matchPerformative));
		this.matchOntology = new ArrayList<String>(parameters.getParameterValues(param.matchOntology));
		this.matchSender = new ArrayList<String>(parameters.getParameterValues(param.matchSender));
		this.matchContent = new ArrayList<String>(parameters.getParameterValues(param.matchContent));
		//TODO change implementation : change to lists is not really necessary
		this.mt = MessageTemplate.MatchAll();
		if (this.matchContent.size() != 0) {
			MessageTemplate mt2 = MessageTemplate.MatchContent(this.matchContent.get(0));
			for (int i = 1; i < this.matchContent.size(); i++) {
				mt2 = MessageTemplate.or(mt2, MessageTemplate.MatchContent(this.matchContent.get(i)));
			}
			this.mt = MessageTemplate.and(this.mt, mt2);
		}

		if (this.matchSender.size() != 0) {
			MessageTemplate mt2 = MessageTemplate.MatchSender(new AID(this.matchSender.get(0), AID.ISLOCALNAME));
			for (int i = 1; i < this.matchSender.size(); i++) {
				mt2 = MessageTemplate.or(mt2, MessageTemplate.MatchSender(new AID(this.matchSender.get(i), AID.ISLOCALNAME)));
			}
			this.mt = MessageTemplate.and(this.mt, mt2);
		}
		if (this.matchConversationID.size() != 0) {
			MessageTemplate mt2 = MessageTemplate.MatchConversationId(this.matchConversationID.get(0));
			for (int i = 1; i < this.matchConversationID.size(); i++) {
				mt2 = MessageTemplate.or(mt2, MessageTemplate.MatchConversationId(this.matchConversationID.get(i)));
			}
			this.mt = MessageTemplate.and(this.mt, mt2);
		}
		if (this.matchOntology.size() != 0) {
			MessageTemplate mt2 = MessageTemplate.MatchOntology(this.matchOntology.get(0));
			for (int i = 1; i < this.matchOntology.size(); i++) {
				mt2 = MessageTemplate.or(mt2, MessageTemplate.MatchOntology(this.matchOntology.get(i)));
			}
			this.mt = MessageTemplate.and(this.mt, mt2);
		}

		if (this.matchPerformative.size() != 0) {
			MessageTemplate mt2 = null;
			for (int i = 0; i < this.matchPerformative.size(); i++) {
				String mperf = this.matchPerformative.get(i);
				int p = -1;
				//				if (mperf.equals("request")) {
				//					p = ACLMessage.REQUEST;
				//				} else if (mperf.equals("inform")) {
				//					p = ACLMessage.INFORM;
				//				}
				p = ACLMessage.getInteger(mperf.toUpperCase());
				//if (p != -1) {
				if (mt2 == null) {
					mt2 = MessageTemplate.MatchPerformative(p);
				} else {
					mt2 = MessageTemplate.or(mt2, MessageTemplate.MatchPerformative(p));
				}
				//}
			}
			this.mt = MessageTemplate.and(this.mt, mt2);
		}

		//policy messages should not come here!
		MessageTemplate policyMessageTemplate = MessageTemplate.MatchOntology("http://ubiware.jyu.fi/2010/08/06/policies/response");
		this.mt = MessageTemplate.and(MessageTemplate.not(policyMessageTemplate), this.mt);

		this.print = parameters.getParameterValue(param.print);
		if (this.print == null) {
			this.print = "200";
		}
		if ((this.print != "true") && (this.print != "false")) {
			try {
				this.print_length = Integer.valueOf(this.print);
			} catch (NumberFormatException e) {
				this.print_length = 200;
			}
		}

		this.contentIsSAPL = parameters.getParameterValue(param.contentIsSAPL);
		if (this.contentIsSAPL == null) {
			this.contentIsSAPL = "true";
		}

		this.contentToFile = parameters.getParameterValue(param.contentToFile);

		this.addBeliefs = parameters.getParameterValue(param.addBeliefs);
		if (this.addBeliefs == null) {
			this.addBeliefs = "true";
		}

		this.waitOnlyFirst = parameters.getParameterValue(param.waitOnlyFirst);
		if (this.waitOnlyFirst == null) {
			this.waitOnlyFirst = "true";
		}

		this.maxWait = parameters.getParameterValue(param.maxWait);
		if (this.maxWait != null) {
			long toWait = new Long(this.maxWait).longValue();
			long now = System.currentTimeMillis();
			this.waitTill = now + toWait;
		}

		this.finished = false; //it is a cyclic behavior		
	}

	@Override
	public void doAction() {

		ACLMessage msg = this.myAgent.receive(this.mt);

		if (msg != null) {
			String content = msg.getContent();
			String received_ontology = msg.getOntology();

			if (!this.print.equals("false")) {
				String printContent;
				if (received_ontology.startsWith(".")) {
					int l = content.length();
					if (received_ontology.equals(".class") || received_ontology.equals(".zip")) {
						l = l / 2;
					}
					printContent = received_ontology + " (" + l + " bytes)";
				} else {
					printContent = '\"' + content + '\"';
				}
				if (!this.print.equals("true") && (printContent.length() > this.print_length)) {
					printContent = printContent.substring(0, this.print_length) + "...(" + content.length() + " chars)";
				}
				this.print(printContent + " from " + msg.getSender().getLocalName() + " received");
			}

			if (this.contentToFile != null) {
				try {
					File datafile = new File(this.contentToFile);
					String name = datafile.getName();
					int ind = name.indexOf('.');
					if (ind == -1) {
						datafile = new File(this.contentToFile + received_ontology);
					}

					File p = datafile.getParentFile();
					if ((p != null) && !p.exists()) {
						boolean directoryMakingSuccessFul = p.mkdirs();
						if (!directoryMakingSuccessFul) {
							System.err.println("Failed creating nessecary directories : " + p.getCanonicalPath());
							return;
						}
					}

					if (!(received_ontology.equals(".class") || received_ontology.equals(".zip"))) {
						FileWriter fwrite = null;
						BufferedWriter ff = null;
						try {
							fwrite = new FileWriter(datafile, false);
							ff = new BufferedWriter(fwrite);
							ff.write(content, 0, content.length());
							ff.close();
						} finally {
							if (ff != null) {
								ff.close();
							} else {
								//try to close FileWriter
								if (fwrite != null) {
									fwrite.close();
								}
							}
						}

					} else {
						FileOutputStream fos = null;
						BufferedOutputStream ff = null;
						try {
							fos = new FileOutputStream(datafile, false);
							ff = new BufferedOutputStream(fos);
							int bufferLength = content.length() / 2;
							byte[] buffer = new byte[bufferLength];
							for (int i = 0; i < bufferLength; i++) {
								int digit = Integer.parseInt(content.substring(i * 2, i * 2 + 2), 16);
								buffer[i] = (byte) digit;
							}
							ff.write(buffer);
						} finally {
							if (ff != null) {
								ff.close();
							} else {
								//try to at least close the underlying FileOutputStream
								if (fos != null) {
									fos.close();
								}
							}

						}

					}

				} catch (IOException e) {
					e.printStackTrace();
					return;
				}
			}

			//long curr = System.currentTimeMillis();
			//String messageId = "message" + curr;
			String messageId = MessageReceiverBehavior.messageInstanceGenerator.getNewID();
			int p = msg.getPerformative();
			String perf = "other";
			if (p == ACLMessage.REQUEST) {
				perf = "request";
			} else if (p == ACLMessage.INFORM) {
				perf = "inform";
			} else if (p == ACLMessage.QUERY_IF) {
				perf = "query-if";
			}
			if (this.addBeliefs.equals("true")) {
				StringBuilder messageReceivedBelief = new StringBuilder();
				messageReceivedBelief.append(messageId);
				messageReceivedBelief.append(' ');
				messageReceivedBelief.append('<');
				messageReceivedBelief.append(SaplN3Constants.PARAM_NS);
				messageReceivedBelief.append("received> {");
				//begin conversationID
				messageReceivedBelief.append('<');
				messageReceivedBelief.append(SaplN3Constants.PARAM_NS);
				messageReceivedBelief.append("conversationID> ");
				messageReceivedBelief.append(SaplN3Constants.is);
				messageReceivedBelief.append(' ');
				messageReceivedBelief.append('\"');
				messageReceivedBelief.append(msg.getConversationId());
				messageReceivedBelief.append('\"');
				//end conversationID
				messageReceivedBelief.append(' ');
				messageReceivedBelief.append('.');
				messageReceivedBelief.append(' ');
				//begin performative
				messageReceivedBelief.append('<');
				messageReceivedBelief.append(SaplN3Constants.PARAM_NS);
				messageReceivedBelief.append("performative> ");
				messageReceivedBelief.append(SaplN3Constants.is);
				messageReceivedBelief.append(' ');
				messageReceivedBelief.append('\"');
				messageReceivedBelief.append(perf);
				messageReceivedBelief.append('\"');
				//end performative
				messageReceivedBelief.append(' ');
				messageReceivedBelief.append('.');
				messageReceivedBelief.append(' ');
				//begin sender
				messageReceivedBelief.append('<');
				messageReceivedBelief.append(SaplN3Constants.PARAM_NS);
				messageReceivedBelief.append("sender> ");
				messageReceivedBelief.append(SaplN3Constants.is);
				messageReceivedBelief.append(' ');
				messageReceivedBelief.append('\"');
				messageReceivedBelief.append(msg.getSender().getLocalName());//TODO : should this remain the local name?
				messageReceivedBelief.append('\"');
				//end sender
				messageReceivedBelief.append(' ');
				messageReceivedBelief.append('.');
				messageReceivedBelief.append(' ');
				//begin ontology
				messageReceivedBelief.append('<');
				messageReceivedBelief.append(SaplN3Constants.PARAM_NS);
				messageReceivedBelief.append("ontology> ");
				messageReceivedBelief.append(SaplN3Constants.is);
				messageReceivedBelief.append(' ');
				messageReceivedBelief.append('\"');
				messageReceivedBelief.append(msg.getOntology());
				messageReceivedBelief.append('\"');
				//end ontology
				messageReceivedBelief.append(' ');
				messageReceivedBelief.append('.');
				messageReceivedBelief.append(' ');
				//begin content
				messageReceivedBelief.append('<');
				messageReceivedBelief.append(SaplN3Constants.PARAM_NS);
				messageReceivedBelief.append("content> ");
				messageReceivedBelief.append(SaplN3Constants.is);

				if (!this.contentIsSAPL.equals("true")) {
					messageReceivedBelief.append(' ');
					messageReceivedBelief.append('\"');
					messageReceivedBelief.append(RABUtils.escapeString(content));
					messageReceivedBelief.append('\"');
				} else {
					messageReceivedBelief.append(' ');
					messageReceivedBelief.append('{');
					messageReceivedBelief.append(content);
					messageReceivedBelief.append('}');
				}
				messageReceivedBelief.append('}');
				this.addBeliefsN3(messageReceivedBelief.toString());
				this.wakeAgent();
				// no need to wait any more, unique IDs are guaranteed by MessageInstanceGenerator.
				//				while (System.currentTimeMillis() == curr) {
				//					this.myAgent.doWait(2);
				//				}
			} else if (this.addBeliefs.startsWith(UbiwareAgent.CONTEXT_PREFIX)) {
				HashMap<String, String> vars = new HashMap<String, String>();
				vars.put("messageID", messageId);
				vars.put("conversationID", msg.getConversationId());
				vars.put("performative", perf);
				vars.put("sender", msg.getSender().getLocalName());
				vars.put("ontology", msg.getOntology());

				String temp_context = null;
				if (!this.contentIsSAPL.equals("true")) {
					vars.put("content", content);
				} else {
					temp_context = this.myAgent.addN3ToTempContext(content);
					if (temp_context != null) {
						vars.put("content", temp_context);
					} else {
						return;
					}
				}

				this.addBeliefs(this.addBeliefs, vars);
				this.wakeAgent();
			}

			if (this.waitOnlyFirst.equals("true")) {
				this.finished = true;
			}

		} else {
			if (this.maxWait != null) {
				long now = System.currentTimeMillis();
				if (now >= this.waitTill) {
					this.setFailed();
					this.finished = true;
				} else {
					this.block(this.waitTill - now);
				}
			} else {
				this.block();
			}
		}
	}
}
