package ubiware.shared;

import jade.lang.acl.ACLMessage;

import java.util.Set;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.Binding;
import ubiware.core.BindingsSet;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ProduceN3Options;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.ontology.param;
import ubiware.util.StringEscape;
import ubiware.util.saplbuilder.Resource;

public class QueryMessageSenderBehavior extends ReusableAtomicBehavior {

	private String query;
	private String context;
	private Resource receiver;
	private int perform;
	private String conversationID;
	private String ontology;
	private String sendPattern;
	private String preamble;

	@Override
	protected void doAction() {

		//now we replace in the query the variables with their values:
		StringBuilder messageContent = new StringBuilder();
		//bindings
		BindingsSet set = this.hasBeliefsN3(this.query, this.context);
		boolean firstStatement = true;
		if (set != null) {

			for (Binding binding : set) {
				Set<String> variables = binding.getVariables();
				if (!firstStatement) {
					messageContent.append(" . ");
				}
				String currentSubstitutionString = this.sendPattern;
				for (String variable : variables) {
					String variableValue = binding.get(variable);
					if (!(variableValue.startsWith("<") && variableValue.endsWith(">"))) {
						//it is a literal
						variableValue = '"' + StringEscape.escape(variableValue) + '"';
					}
					currentSubstitutionString = currentSubstitutionString.replace("\"?" + variable + "\"", variableValue);
				}
				messageContent.append(currentSubstitutionString);
				firstStatement = false;
			}
		}
		if (this.preamble != null) {
			//there is a preamble,  this must be added. It does not matter whether this happens in the beginning or the end!
			if (!firstStatement) {
				messageContent.append(" . ");
			}
			messageContent.append(this.preamble);
		}
		
		super.sendMessage(this.perform, this.receiver, messageContent.toString(), this.ontology, this.conversationID);

		
		//System.out.println(messageContent);
//		ACLMessage message = new ACLMessage(this.perform);
//		message.addReceiver(new AID(this.receiver, AID.ISLOCALNAME));
//		message.setContent(messageContent.toString());
//		message.setOntology(this.ontology);
//		message.setConversationId(this.conversationID);
//		this.myAgent.send(message);
	}

	private static ProduceN3Options options = new ProduceN3Options(false, true, false, false);

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		String queryContext = parameters.getObligatedContainerID(param.query);
		this.query = this.produceN3(queryContext, QueryMessageSenderBehavior.options);
		this.context = parameters.getObligatedContainerID(param.context);
		this.receiver = parameters.getObligatedResourceParameter(param.receiver);
		String sendPatternID = parameters.getOptionalContainerID(param.sendPattern, queryContext);
		this.sendPattern = this.produceN3(sendPatternID, QueryMessageSenderBehavior.options);
		this.conversationID = parameters.getObligatedNonEmptyStringParameter(param.conversationID);
		String performative = parameters.getObligatedNonEmptyStringParameter(param.performative);

		if (performative.equals("inform")) {
			this.perform = ACLMessage.INFORM;
		} else if (performative.equals("query-if")) {
			this.perform = ACLMessage.QUERY_IF;
		} else if (performative.equals("request")) {
			this.perform = ACLMessage.REQUEST;
		} else {
			System.err.println("The performative is not any of the understood performatives.");
			this.setFailed();
		}
		this.ontology = parameters.getOptionalStringParameter(param.ontology, "unspecified");
		String preambleID = parameters.getOptionalContainerID(param.preamble, null);
		if (preambleID != null) {
			this.preamble = this.produceN3(preambleID, QueryMessageSenderBehavior.options);
		}
	}

}
