package ubiware.shared;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.ontology.param;

public class SQLReaderBehavior extends ReusableAtomicBehavior {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private String db_user;
	private String db_url;
	private String db_drv;
	private String db_password;
	private String db_query;

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.db_drv = parameters.getObligatedNonEmptyStringParameter(param.driver);
		this.db_url = parameters.getObligatedNonEmptyStringParameter(param.url);
		this.db_user = parameters.getParameterValue(param.username);
		this.db_password = parameters.getParameterValue(param.password);
		this.db_query = parameters.getObligatedNonEmptyStringParameter(param.query);
	}

	@Override
	public void doAction() {

		long s = System.currentTimeMillis();
		ResultSet rset = null;
		Statement stmt = null;
		Connection conn = null;
		try {
			Class.forName(this.db_drv);
			if ((this.db_user != null) && (this.db_password != null)) {
				conn = DriverManager.getConnection(this.db_url, this.db_user, this.db_password);
			} else {
				conn = DriverManager.getConnection(this.db_url);
			}
			stmt = conn.createStatement();
			this.print("Connected [" + (System.currentTimeMillis() - s) + "]");
			s = System.currentTimeMillis();
			rset = stmt.executeQuery(this.db_query);
			this.print("Query executed [" + (System.currentTimeMillis() - s) + "]");
			s = System.currentTimeMillis();
			ResultSetMetaData meta = rset.getMetaData();
			int RC = meta.getColumnCount();
			String labels[] = new String[RC];

			for (short iCol = 0; iCol < RC; iCol++) {
				labels[iCol] = meta.getColumnName(iCol + 1);
			}

			this.print("Start creating N3 string for beliefs.");
			s = System.currentTimeMillis();
			StringBuilder beliefs = new StringBuilder();
			beliefs.append("<" + this.db_url + ">" + " :table {");
			boolean foundRow = false;
			int iRow = 0;
			while (rset.next()) {
				iRow++;
				String rid = String.valueOf(iRow);
				beliefs.append(((foundRow) ? ". " : "") + rid + " :row {");
				foundRow = true;
				boolean foundColumn = false;

				for (int iCol = 0; iCol < RC; iCol++) {
					String value = rset.getString(iCol + 1);
					if ((value == null) || value.equals("null") || value.equals("")) {
						value = "N/A";
					}
					beliefs.append(((foundColumn) ? ". " : "") + labels[iCol] + " :column \"" + value + "\"");
					foundColumn = true;
				}
				beliefs.append("}");
			}
			beliefs.append("}");

			this.print("N3 string is ready. Adding beliefs. [" + (System.currentTimeMillis() - s) + "]");
			s = System.currentTimeMillis();
			this.addBeliefsN3(beliefs.toString(), false);

			this.wakeAgent();
		} catch (Exception ex) {
			ex.printStackTrace();
			this.setFailed();
			return;
		} finally {
			if (rset != null) {
				try {
					rset.close();

				} catch (SQLException e) {
					// not much we can do here.
				}
			}
			if (stmt != null) {
				try {
					stmt.close();
				} catch (SQLException e) {
					// not much we can do here.
				}
			}
			if (conn != null) {
				try {
					conn.close();
				} catch (SQLException e) {
					// not much we can do here.
				}
			}
		}
		this.print("Success. [" + (System.currentTimeMillis() - s) + "]");
	}
}
