package ubiware.shared;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.RABUtils;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.ontology.param;

/**
 * from Saplguide.pdf:<br>
 * <h3>Full name: ubiware.shared.TextWriterBehavior</h3>
 * <p>
 * Action: Writes a text string to a text file.
 * </p>
 * <p>
 * Inputs:
 * <table border="1">
 * <tr>
 * <th>Name</th>
 * <th>Meaning</th>
 * <th>Mandatory</th>
 * <th>Default</th>
 * </tr>
 * <tr>
 * <td>p:input</td>
 * <td>The text to be written.</td>
 * <td>Yes</td>
 * <td></td>
 * </tr>
 * <tr>
 * <td>p:saveTo</td>
 * <td>The name of the file.</td>
 * <td>Yes</td>
 * <td></td>
 * </tr>
 * </table>
 * </p>
 * <p>
 * Outputs: Creates/overwrites a specified file.
 * </p>
 * 
 * 24.3.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public class TextWriterBehavior extends ReusableAtomicBehavior {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private String input;
	private String saveTo;

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.input = parameters.getObligatedStringParameter(param.input);
		this.saveTo = parameters.getObligatedStringParameter(param.saveTo);
	}

	@Override
	protected void doAction() {
		boolean ok = RABUtils.putResult(this.myAgent, this.input, this.saveTo);
		if (!ok) {
			this.setFailed();
		}
	}
}
