package ubiware.shared;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.ontology.param;

public class UnzipperBehavior extends ReusableAtomicBehavior {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private File file;
	private boolean delete;

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.file = parameters.getObligatedFileParameter(param.input);
		this.delete = parameters.getOptionalBooleanParameter(param.delete, false);
	}

	@Override
	public void doAction() {

		String path = this.file.getParent();
		ZipFile zipFile = null;
		try {
			zipFile = new ZipFile(this.file);
			Enumeration<? extends ZipEntry> entries = zipFile.entries();
			while (entries.hasMoreElements()) {
				ZipEntry entry = entries.nextElement();
				if (!entry.isDirectory()) {
					File f = new File(entry.getName());
					File p = f.getParentFile();
					if ((p != null) && !p.exists()) {
						boolean directoryMakingSuccessFul = p.mkdirs();
						if (!directoryMakingSuccessFul) {
							System.err.println("Failed creating nessecary directories : " + p.getCanonicalPath());
							this.setFailed();
							return;
						}
					}
					InputStream in = null;
					BufferedInputStream bin = null;
					OutputStream out = null;
					BufferedOutputStream bos = null;
					try {
						in = zipFile.getInputStream(entry);
						bin = new BufferedInputStream(in);
						out = new FileOutputStream(path + "/" + entry.getName());
						bos = new BufferedOutputStream(out);
						byte[] buffer = new byte[1024];
						int len;
						while ((len = bin.read(buffer)) >= 0) {
							bos.write(buffer, 0, len);
						}
					} finally {
						if (bos != null) {
							bos.close();
						} else if (out != null) {
							//close at least the underlying stream
							out.close();
						}
						if (bin != null) {
							bin.close();
						} else if (in != null) {
							in.close();
						}
					}
				}
			}

			this.print(this.file.getPath() + " unzipped");

			if (this.delete) {
				FileOutputStream fo = null;
				try {
					fo = new FileOutputStream(this.file);
					fo.write(13);
					fo.write(10);
					fo.close();
				} finally {
					if (fo != null) {
						try {
							fo.close();
						} catch (IOException e) {
							//ok, not much we can do
						}
					}
				}
				boolean successFullDeletion = this.file.delete();
				if (!successFullDeletion) {
					System.err.println("Deleting the zip file after extraction failed. Continuing since extraction succeeded.");
				}

			}

		} catch (IOException ex) {
			ex.printStackTrace();
			this.setFailed();
		} finally {
			try {
				zipFile.close();
			} catch (IOException ex) {
				ex.printStackTrace();
				this.setFailed();
			}
		}
	}
}
