package ubiware.shared;

import java.io.File;
import java.util.Stack;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.RABUtils;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.ontology.param;

public class XmlReaderBehavior extends ReusableAtomicBehavior {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	StringBuilder beliefs;
	StringBuilder literal;
	String lastLiteral = "";

	Stack<Integer> stack;
	private File file;

	private class Handler extends DefaultHandler {

		private String getAndInc() {
			Integer iii = XmlReaderBehavior.this.stack.pop();
			iii++;
			XmlReaderBehavior.this.stack.push(iii);
			String zeros = "0000";
			String id = iii.toString();
			id = zeros.substring(0, zeros.length() - id.length()) + id;
			return id;
		}

		private void checkLiteral() {
			if (XmlReaderBehavior.this.literal.length() > 0) {
				String str = XmlReaderBehavior.this.literal.toString().trim();
				if (str.length() > 0) {
					if ((XmlReaderBehavior.this.beliefs.length() > 0)
							&& (XmlReaderBehavior.this.beliefs.charAt(XmlReaderBehavior.this.beliefs.length() - 1) != '{')) {
						XmlReaderBehavior.this.beliefs.append(". ");
					}

					String iii = this.getAndInc();
					XmlReaderBehavior.this.beliefs.append(iii + " :leaf \"" + RABUtils.escapeString(str) + "\"");
					XmlReaderBehavior.this.lastLiteral = XmlReaderBehavior.this.literal.toString();
				}
				XmlReaderBehavior.this.literal = new StringBuilder();
			}
		}

		@Override
		public void startElement(String namespaceURI, String localName, String qName, Attributes atts) {
			this.checkLiteral();
			if (XmlReaderBehavior.this.beliefs.charAt(XmlReaderBehavior.this.beliefs.length() - 1) != '{') {
				XmlReaderBehavior.this.beliefs.append(". ");
			}

			String iii = this.getAndInc();
			XmlReaderBehavior.this.beliefs.append("{" + iii + " :tag " + ((namespaceURI.equals("")) ? "" : "<") + namespaceURI + localName
					+ ((namespaceURI.equals("")) ? "" : ">") + "} :branch {");

			XmlReaderBehavior.this.stack.push(0);

			for (int i = 0; i < atts.getLength(); i++) {
				iii = this.getAndInc();
				XmlReaderBehavior.this.beliefs.append(((i > 0) ? ". " : ""));
				String ns = atts.getURI(i);
				XmlReaderBehavior.this.beliefs.append("{" + iii + " :attribute " + ((ns.equals("")) ? "" : "<") + ns + atts.getLocalName(i)
						+ ((ns.equals("")) ? "" : ">") + "} :leaf " + atts.getValue(i));
			}
		}

		@Override
		public void endElement(String uri, String localName, String qName) {
			this.checkLiteral();
			if (XmlReaderBehavior.this.beliefs.charAt(XmlReaderBehavior.this.beliefs.length() - 1) == '{') {
				XmlReaderBehavior.this.beliefs.delete(XmlReaderBehavior.this.beliefs.lastIndexOf(":branch"), XmlReaderBehavior.this.beliefs
						.length());
				XmlReaderBehavior.this.beliefs.append(":leaf :empty");
			} else {
				XmlReaderBehavior.this.beliefs.append("}");
			}
			XmlReaderBehavior.this.stack.pop();
		}

		@Override
		public void characters(char[] ch, int start, int length) {
			XmlReaderBehavior.this.literal.append(ch, start, length);
		}
	}

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.file = parameters.getObligatedFileParameter(param.input);
	}

	@Override
	public void doAction() {
		int len = (int) this.file.length();

		this.beliefs = new StringBuilder(len);
		this.beliefs.append(this.file.getName() + " :tree {");
		this.literal = new StringBuilder();

		this.stack = new Stack<Integer>();
		this.stack.push(0);

		Handler handler = new Handler();

		try {
			SAXParserFactory factory = SAXParserFactory.newInstance();
			factory.setNamespaceAware(true);
			SAXParser parser = factory.newSAXParser();
			parser.parse(this.file, handler);
		} catch (Exception ex) {
			this.print(this.getClass().getName() + " error: Operation failed: " + ex.getClass().getName() + " " + ex.getMessage());
			this.setFailed();
			return;
		}

		this.beliefs.append("}");
		this.addBeliefsN3(this.beliefs.toString());
		this.wakeAgent();
	}
}
