package ubiware.shared.agentServer;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.eclipse.jetty.continuation.Continuation;
import org.eclipse.jetty.continuation.ContinuationSupport;

import ubiware.core.SaplN3Constants;
import ubiware.core.UbiwareAgent;
import ubiware.core.commands.AddBeliefsN3Command;
import ubiware.ontology.sapl;
import ubiware.util.IDGenerator;
import ubiware.util.saplbuilder.Literal;
import ubiware.util.saplbuilder.Resource;
import ubiware.util.saplbuilder.SaplDocument;

/**
 * The servlet implementing the actual behavior of the server When a request arrives, the request is wrapped together with a continuation
 * object in an {@link AgentServerBehaviorServerEvent} Then beliefs are added to the agent about the event happening. For aformat of the
 * beliefs, see {@link AgentServerBehavior}
 * 
 * 18.3.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public class AgentServerBehaviorServlet extends HttpServlet {

	private static final long serialVersionUID = 1L;
	private final UbiwareAgent myAgent;
	private final IDGenerator requestIDGenerator;
	private final int timeout;

	/**
	 * create a server attached to a certain agent
	 * 
	 * @param myAgent
	 *            the agent this Servlet is working with
	 * @param timeout
	 *            the timeout for answering to requests.
	 */
	public AgentServerBehaviorServlet(UbiwareAgent myAgent, int timeout) {
		this.myAgent = myAgent;
		this.timeout = timeout;
		this.requestIDGenerator = new IDGenerator("_serverevent_" + this.myAgent.getLocalName());
	}

	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
		this.doPost(req, resp);
	}

	@Override
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		//we just add the request and response to the agent's blackboard and add the belief of this event happening.
		//System.out.println("entering DOPOST");
		//create continuation object
		final Continuation continuation = ContinuationSupport.getContinuation(request);
		continuation.suspend(); // always suspend before registration
		continuation.setTimeout(this.timeout);
		//add the event to the blackboard
		Resource blackboardID = this.myAgent.addOnBlackboard(new AgentServerBehaviorServerEvent(request, response, continuation));
		//FIXME the type returned from the addOnBB call should return a resource!!!
		//Resource bbid = Resource.createUncheckedResource(blackboardID);
		Resource requestID = this.requestIDGenerator.getNewResource();
		//generate the belief which has to be posted to the agent.
		// builder = new SaplContainerBuilder();
		SaplDocument doc = new SaplDocument();
		doc.addStatement(requestID, sapl.hasBlackboardID, blackboardID);
		doc.addStatement(requestID, sapl.requestsPath, new Literal(request.getPathInfo().substring(1)));
		
//		builder.addPrefix("sapl", SaplN3Constants.SAPL_NS_URI);
		//FIXME the predicate should get standardized
//		builder.add(requestID, "sapl:hasBlackboardID", blackboardID);
//		builder.add(requestID, "sapl:requestsPath", "\"" + request.getPathInfo().substring(1) + "\""); //strip off first '/' as defined in AgentServerBehavior.
		//String oldStylePath = request.getPathInfo() + "?" + request.getQueryString();
		//builder.add(requestID, "sapl:hasOldStylePath", oldStylePath);
		//String N3 = builder.toString();
		//System.out.println(N3);
		//We have to add the belief as a command since we're not in a RAB
		this.myAgent.addCommand(new AddBeliefsN3Command(doc));
//		this.myAgent.addCommand(new AddBeliefsN3Command(N3));
	}
}
