package ubiware.shared.gui.buttonGUI;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.WindowConstants;

import ubiware.core.UbiwareAgent;
import ubiware.core.commands.AddBeliefsN3Command;
import ubiware.ontology.gui;
import ubiware.util.saplbuilder.Literal;
import ubiware.util.saplbuilder.Resource;
import ubiware.util.saplbuilder.SaplDocument;

/**
 * A GUI component displaying a list of buttons. When a button is clicked, the agent is informed about this event. This class is package
 * private since it needs to be constructed from NewGUIBehavior
 * 
 * @author Michael Cochez for Ubiware project
 * @since 4 Aug 2010
 */
final class ButtonGUI extends JFrame {

	private static final long serialVersionUID = 1L;
	private final UbiwareAgent myAgent;
	/**
	 * The id of this GUI
	 */
	private final Resource guiID;
	//	public final List<JButton> buttons;
	public final Map<String, JButton> textToButton;

	/**
	 * Construct a ButtonGUI.
	 * 
	 * @param text
	 *            The label which should appear together with the buttons.
	 * @param buttons
	 *            A list of button names. This list is not allowed to contain duplicates.
	 * @param myAgent
	 *            The agent this GUI belongs to, used to inform it about events.
	 * @param guiID
	 *            the ID this GUI is registerred with in the {@link GUIRepository}
	 * @param allowClose
	 *            Is this GUI allowed to be closed by the user.
	 */
	ButtonGUI(String text, Collection<String> buttons, UbiwareAgent myAgent, Resource guiID, boolean allowClose) {
		this.myAgent = myAgent;
		this.guiID = guiID;
		Container pane = this.getContentPane();
		this.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS));
		pane.add(Box.createRigidArea(new Dimension(0, 10)));
		pane.add(new JLabel(text));
		//		this.buttons = new ArrayList<JButton>(buttonsNames.size());
		this.textToButton = new HashMap<String, JButton>();
		//int buttonNumber = 0;
		ActionListener agentInformer = new ButtonGUIActionListener();
		for (String name : buttons) {
			//buttonNumber++;
			JButton button = new JButton(name);
			button.addActionListener(agentInformer);
			pane.add(button);
			pane.add(Box.createRigidArea(new Dimension(0, 10)));
			//			this.buttons.add(button);
			this.textToButton.put(name, button);
		}
		if (!allowClose) {
			this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		} else {
			//install a disposal on close.
			this.addWindowListener(new WindowAdapter() {
				@Override
				public void windowClosing(WindowEvent e) {
					try {
						GUIRepository.dispose(ButtonGUI.this.myAgent.getAID(), ButtonGUI.this.guiID, ButtonGUI.this);
					} catch (NoSuchGUIException e1) {
						throw new Error("The gui was created but not registerred.", e1);
					}
				}
			});
		}
		//in either case, the agent gets informaed about a closing event.
		this.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				
//				StringBuilder n3stringBuilder = new StringBuilder();
//				n3stringBuilder.append('{');
//				n3stringBuilder.append(' ');
//				n3stringBuilder.append(":User :clickedFrameButton ");
//				n3stringBuilder.append('\"');
//				n3stringBuilder.append("close");
//				n3stringBuilder.append('\"');
//				n3stringBuilder.append(' ');
//				n3stringBuilder.append('}');
//				n3stringBuilder.append(' ');
//				n3stringBuilder.append(SaplN3Constants.eventFromGUI);
//				n3stringBuilder.append(' ');
//				n3stringBuilder.append(ButtonGUI.this.guiID);
				//				System.out.println(n3stringBuilder.toString());
				ubiware.util.saplbuilder.Container event = new ubiware.util.saplbuilder.Container();
				event.addStatement(gui.User, gui.clickedFrameButton, gui.close);
				SaplDocument doc = new SaplDocument(1);
				doc.addStatement(event, gui.eventFromGUI, ButtonGUI.this.guiID);
				
				
				ButtonGUI.this.myAgent.addCommand(new AddBeliefsN3Command(doc));
			}
		});

		this.pack();
		//this.setSize((int) this.getSize().getWidth() + 20, (int) this.getSize().getHeight());

	}

	/**
	 * The ActionListener responsible for sending button click events to the agent.
	 * 
	 * @author Michael Cochez for Ubiware project
	 * @since 4 Aug 2010
	 */
	private final class ButtonGUIActionListener implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			//get the text from the button
			String buttonText = e.getActionCommand();
//			//build the belief which has to be added to the agent.
//			StringBuilder n3stringBuilder = new StringBuilder();
//			n3stringBuilder.append('{');
//			n3stringBuilder.append(' ');
//			n3stringBuilder.append(":User :clickedButton ");
//			n3stringBuilder.append('\"');
//			n3stringBuilder.append(buttonText);
//			n3stringBuilder.append('\"');
//			n3stringBuilder.append(' ');
//			n3stringBuilder.append('}');
//			n3stringBuilder.append(' ');
//			n3stringBuilder.append(SaplN3Constants.eventFromGUI);
//			n3stringBuilder.append(' ');
//			n3stringBuilder.append(ButtonGUI.this.guiID);
			
			
			ubiware.util.saplbuilder.Container event = new ubiware.util.saplbuilder.Container();
			event.addStatement(gui.User, gui.clickedButton, new Literal(buttonText));
			SaplDocument doc = new SaplDocument(1);
			doc.addStatement(event, gui.eventFromGUI, ButtonGUI.this.guiID);
			
			
			ButtonGUI.this.myAgent.addCommand(new AddBeliefsN3Command(doc));
		}
	}

}
