package ubiware.shared.gui.buttonGUI;

import jade.core.AID;

import java.net.URI;
import java.util.HashMap;
import java.util.Map;

import com.google.common.collect.Maps;

import ubiware.util.UniqueInstanceGenerator;
import ubiware.util.saplbuilder.Resource;

class GUIRepository {
	public static final UniqueInstanceGenerator GUIIDGenerator = new UniqueInstanceGenerator("GUI", GUIRepository.getGUINamespace());
	private static Map<AID, Map<Resource, ButtonGUI>> agentGUIs = Maps.newHashMap();
	private static Map<AID, ButtonGUI> agentdefaultGUIs = new HashMap<AID, ButtonGUI>();

	public static void registerGUI(AID myAgentID, Resource guiID, ButtonGUI theGUI) {
		if ((myAgentID == null) || (theGUI == null)) {
			throw new NullPointerException("myAgentID and theGUI must not be null");
		}
		if (!GUIRepository.agentdefaultGUIs.containsKey(myAgentID)) {
			//no default GUI yet.
			GUIRepository.agentdefaultGUIs.put(myAgentID, theGUI);
		}
		Map<Resource, ButtonGUI> theGUIMap = GUIRepository.agentGUIs.get(myAgentID);
		if (theGUIMap == null) {
			theGUIMap = new HashMap<Resource, ButtonGUI>(2);
			GUIRepository.agentGUIs.put(myAgentID, theGUIMap);
		}
		theGUIMap.put(guiID, theGUI);
	}

	private static URI getGUINamespace() {
		try {
			return new URI("http://ubiware.jyu.fi/gui/buttonGUI/");
		} catch (Exception e) {
			throw new Error("The URI is hardcoded and mustn't be wrong.", e);
		}
	}

	public static ButtonGUI getGUI(AID aid, Resource gUIID) throws NoSuchGUIException {

		if (gUIID == null) {
			//return the default
			ButtonGUI gui = GUIRepository.agentdefaultGUIs.get(aid);
			if (gui == null) {
				//TODO other exception
				throw new NoSuchGUIException("This agent does not have a registerred default GUI");
			}
			return gui;

		}
		//return the one for that ID
		Map<Resource, ButtonGUI> guiMap = GUIRepository.agentGUIs.get(aid);
		if (guiMap == null) {
			throw new NoSuchGUIException("This agent does not have registerred gui's");
		}
		ButtonGUI gui = guiMap.get(gUIID);
		if (gui == null) {
			throw new NoSuchGUIException("This agent does not have a GUI registerred with this ID '" + gUIID + "'");
		}
		return gui;
	}

	public static void dispose(AID aid, Resource guiID, ButtonGUI gui) throws NoSuchGUIException {
		ButtonGUI defaultGUI = GUIRepository.agentdefaultGUIs.get(aid);
		if ((defaultGUI != null) && defaultGUI.equals(gui)) {
			GUIRepository.agentdefaultGUIs.remove(aid);
		}
		Map<Resource, ButtonGUI> aidAgentGUIs = GUIRepository.agentGUIs.get(aid);
		if (aidAgentGUIs == null) {
			throw new NoSuchGUIException("This agent does not have registerred gui's");
		}
		ButtonGUI registerredGUI = aidAgentGUIs.get(guiID);
		if ((registerredGUI == null) || !registerredGUI.equals(gui)) {
			throw new NoSuchGUIException("This agent does not have a GUI registerred with this ID '" + guiID + "'");
		}
		aidAgentGUIs.remove(guiID);
	}
}
