package ubiware.shared.persistency;

import java.io.IOException;
import java.net.URL;
import java.util.Properties;

//package visibility
class DatabaseProperties {
	private static final long serialVersionUID = 1L;
	private static Properties properties = DatabaseProperties.loadProperties();
	public static final String JDBCConnectionURL = DatabaseProperties.properties
			.getProperty("ubiware.shared.persistency.JDBCConnectionURL");
	public static final String JDBCClass = DatabaseProperties.properties.getProperty("ubiware.shared.persistency.JDBCClass");
	public static final String databaseFolder = DatabaseProperties.properties.getProperty("ubiware.shared.persistency.databaseFolder");

	
	//check whether the properties are found as they should.
	static {
		if ((DatabaseProperties.JDBCClass == null) || (DatabaseProperties.JDBCConnectionURL == null)) {
			throw new Error("Database properties could not be read from file");
		}

		try {
			Class.forName(DatabaseProperties.JDBCClass);
		} catch (ClassNotFoundException e) {
			throw new Error("The library needed for the JDBC connection could not be found.", e);
		}
		DatabaseProperties.properties = null; //not needed any longer!
	}

	private static Properties loadProperties() {
		Properties props = new Properties();
		URL url = ClassLoader.getSystemResource("ubiware/shared/persistency/database.properties");
		if (url == null) {
			throw new Error("The database.properties properties file could no be loaded and hence this class cannot work.");
		}
		try {
			props.load(url.openStream());
		} catch (IOException e) {
			throw new Error("The properties file could not be read as expected", e);
		}
		return props;
	}

	private DatabaseProperties() {
		// instantiation does not make any sense
	}

	//quick test
	public static void main(String[] args) {
		System.out.println(DatabaseProperties.JDBCClass);
		System.out.println(DatabaseProperties.JDBCConnectionURL);
		System.out.println(DatabaseProperties.databaseFolder);
	}
}
