package ubiware.shared.persistency;

import java.sql.SQLException;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.ontology.param;

public class LoadPersistentContainer extends ReusableAtomicBehavior {

	private static final long serialVersionUID = 1L;
	private String persistentContainer;
	private String persistentContainerIdentification;

	@Override
	protected void doAction() {
		try {
			this.actualAction();
		} catch (SQLException e) {
			e.printStackTrace();
			this.setFailed();
		}
	}

	private void actualAction() throws SQLException {
		PersistentTableConnection connection = new PersistentTableConnection();
		String loadedsapl = connection.get(this.persistentContainerIdentification);
		connection.close();
		if (loadedsapl == null) {
			//there is no data in the table
			return;
		}
		String sapl = this.persistentContainer + " <http://www.ubiware.jyu.fi/persistency#hasContent> { " + loadedsapl + " }";
		//System.out.println("going to add sapl : " + sapl);
		if (!this.addBeliefsN3(sapl)) {
			System.err.println("Adding persistent beliefs to the agent failed! " + sapl);
			//it failed to add the beliefs
			this.setFailed();
		}
	}

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.persistentContainer = parameters.getObligatedNonEmptyStringParameter(param.persistentContainer);
		this.persistentContainerIdentification = parameters.getObligatedNonEmptyStringParameter(param.persistentContainerID);
	}

	//quick test
	//	public static void main(String[] args) throws SQLException {
	//		LoadPersistentContainer lpc = new LoadPersistentContainer();
	//		lpc.persistentContainerIdentification = "ABS";
	//		lpc.actualAction();
	//	}
}
