package ubiware.shared.persistency;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

class PersistentTableConnection {
	private Connection conn;
	
	static {
		try {
			CreatePersistentStore.ensureStorageExistence();
		} catch (SQLException e) {
			throw new Error(e);
		}
	}
	
	public PersistentTableConnection() throws SQLException {
		this.conn = DriverManager.getConnection(DatabaseProperties.JDBCConnectionURL);
		
	}

	public void put(String ID, String value) throws SQLException {
		/*
		 * We are using specific SQLite feature here: we thus don't need to delete before inserting
		 */
		synchronized (PersistentTableConnection.class) {
			PreparedStatement prep = this.conn.prepareStatement("insert or replace into persistentstore values (?, ?);");
			prep.setString(1, ID);
			prep.setString(2, value);
			prep.executeUpdate();
			prep.close();
		}
	}

	public String get(String Id) throws SQLException {
		synchronized (PersistentTableConnection.class) {
			PreparedStatement statement = this.conn.prepareStatement("select sapl from persistentstore where id = ?");
			statement.setString(1, Id);
			ResultSet rs = statement.executeQuery();
			rs.setFetchSize(1);
			String result = null;
			if (rs.next()) {
				result = rs.getString("sapl");
			} else {
				//System.out.println("No sapl found => nothing added");
				//there is no content! this can happen on initial load. This is not erroneous .
				//result stays null

			}
			rs.close();
			statement.close();
			return result;
		}

	}

	public void close() throws SQLException {
		this.conn.close();
	}
}
