package ubiware.shared.persistency;

import java.sql.SQLException;

import ubiware.core.BehaviorStartParameters;
import ubiware.core.BindingsSet;
import ubiware.core.IllegalParameterConfigurationException;
import ubiware.core.ProduceN3Options;
import ubiware.core.ReusableAtomicBehavior;
import ubiware.ontology.param;

public class StorePersistentContainer extends ReusableAtomicBehavior {

	private static final long serialVersionUID = 1L;
	private String persistentContainerContent;
	private String persistentContainerIdentification;
	private String persistentContainer;

	@Override
	protected void doAction() {
		try {
			this.doActualAction();
		} catch (SQLException e) {
			e.printStackTrace();
			this.setFailed();
		}
	}

	private void doActualAction() throws SQLException {
		PersistentTableConnection connection = new PersistentTableConnection();
		connection.put(this.persistentContainerIdentification, this.persistentContainerContent);
		connection.close();
	}

	private static final ProduceN3Options options = new ProduceN3Options(false, true, false, false);

	@Override
	protected void initializeRAB(BehaviorStartParameters parameters) throws IllegalParameterConfigurationException {
		this.persistentContainer = parameters.getObligatedNonEmptyStringParameter(param.persistentContainer);
		this.persistentContainerIdentification = parameters.getObligatedNonEmptyStringParameter(param.persistentContainerID);
		//this garbage collection step prevents empty results to cpm from the query.
		//TODO this is something wrong in the agent behavior.
		this.myAgent.collectGarbage();
		BindingsSet bindings = this.hasBeliefsN3(this.persistentContainer + " <http://www.ubiware.jyu.fi/persistency#hasContent> ?content");
		if (bindings == null) {
			this.persistentContainerContent = "";
		} else if (bindings.getSize() == 1) {
			//we are sure that there is at least one result => this code is safe
			String contentContextID = bindings.iterator().next().get("content");
			if (contentContextID == null) {
				throw new AssertionError("The contextID cannot be null");
			}
			this.persistentContainerContent = this.produceN3(contentContextID, StorePersistentContainer.options);
			if (this.persistentContainerContent == null) {
				System.err.println("The content of the persistent container could not be generated");
				this.setFailed();
			}
		} else {
			System.err.println("An error occured, there must be no or exactly one contnet for a persistent container");
			this.setFailed();
		}
	}
}
