package ubiware.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStream;

/**
 * Utilities which make working with byte and character streams more convenient.
 * 
 * 18.3.2010
 * 
 * @author Michael Cochez for UBIWARE project
 */
public class StreamingUtils {
	/**
	 * Send a file to an outputstream
	 * 
	 * @param file
	 *            The {@link File} to be send to the {@link OutputStream}
	 * @param os
	 *            The {@link OutputStream}
	 * @throws FileNotFoundException
	 *             If the {@link File} could not be found
	 * @throws IOException
	 *             If an {@link IOException} occurred.
	 */
	public static void sendFileToOutputStream(File file, OutputStream os) throws FileNotFoundException, IOException {

		FileInputStream fis = new FileInputStream(file);
		try {
			int read = 0;
			byte[] bytes = new byte[1024];
			//While there are still bytes in the file, read them and write them to our OutputStream
			while ((read = fis.read(bytes)) != -1) {
				os.write(bytes, 0, read);
			}
		} finally {
			//Clean resources
			fis.close();
			os.flush();
			os.close();
		}
	}

	public static String getInputFromFile(File file) throws IOException {
		int len = (int) file.length();
		char mess[] = new char[len];
		FileReader iread = null;
		BufferedReader ff = null;
		try {
			iread = new FileReader(file);
			ff = new BufferedReader(iread);
			int read = ff.read(mess, 0, len);
			String data = new String(mess, 0, read);
			return data;
		} finally {
			if (ff != null) {
				ff.close();
			} else if (iread != null) {
				iread.close();
			}
		}
	}
}
