package ubiware.util.saplbuilder;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

import ubiware.util.StringEscape;

/**
 * Representation of a literal.
 * 
 * @author Michael Cochez for Ubiware project
 * @since 17 Aug 2010
 */
public class Literal implements Subject, Object {
	private static final String DATE_FORMAT = "dd-MM-yyyy HH-mm-ss z";
	private String value;
	/**
	 * DataFormat used to format Dates to String representation.
	 */
	private static final DateFormat dateFormatter = new SimpleDateFormat("\"" + Literal.DATE_FORMAT
			+ "\"'^^<http://www.w3.org/2001/XMLSchema#string>'");
	static {
		Literal.dateFormatter.setTimeZone(TimeZone.getTimeZone("GMT"));
	}

	/**
	 * Constructs a literal from the given string representation. Keep in mind that the value must not be escaped.
	 * 
	 * @param value
	 *            the value of this Literal.
	 * @throws NullPointerException
	 *             If the specified value is null.
	 */
	public Literal(String value) {
		if (value == null) {
			throw new NullPointerException("The value for a literal must not be null");
		}
		this.value = "\"" + StringEscape.escape(value) + "\"";
	}

	/**
	 * Constructs a literal from a date.
	 * 
	 * @param date
	 *            The date to create a literal from.
	 */
	public Literal(Date date) {
		//DateFormat formatter = DateFormat.getDateTimeInstance(DateFormat.LONG, DateFormat.LONG, Locale.UK);//new SimpleDateFormat("dd-MM-yyyy HH-mm-ss");
		//formatter.setTimeZone(TimeZone.getTimeZone("GMT"));
		String dateString;
		synchronized (Literal.dateFormatter) {
			dateString = Literal.dateFormatter.format(date);
		}
		System.out.println(dateString);
		this.value = dateString;
	}

	public Literal(boolean value) {
		if (value) {
			this.value = '"' + "true" + '"';
		} else {
			this.value = '"' + "false" + '"';
		}
	}

	/**
	 * Generates the s-apl representation of the literal. Properly escaped.
	 */
	@Override
	public void generateSapl(StringBuilder builder) {
		builder.append(this.value);
	}

	/**
	 * Returns the literal value.
	 */
	@Override
	public String toString() {
		return this.value;
	}

}
