package ubiware.util.saplbuilder;

/**
 * A SaplDocument is an consistent representation of a s-apl document
 * 
 * @author Michael Cochez for Ubiware project
 * @since 17 Aug 2010
 */
public class SaplDocument extends StatementContainer {
	/**
	 * Creates a SaplDocument with the initial capacity equal to 5.
	 */
	public SaplDocument() {
		super(5);
	}

	/**
	 * Creates a SaplDocument filled with statements from the provided statement container
	 */
	public SaplDocument(StatementContainer sc) {
		super(5);
		for (Statement statement : sc) {
			this.addStatement(statement);
		}
	}

	/**
	 * Creates a SaplDocument with the initial capacity as specified.
	 * 
	 * @param initialCapacity
	 *            the initial capacity of the document.
	 */
	public SaplDocument(int initialCapacity) {
		super(initialCapacity);
	}

	/**
	 * Generate the S-apl representation of this SaplDocument in a String.
	 * 
	 * @return A string containg S-APL code this Sapl document contains.
	 */
	public String generateSapl() {
		StringBuilder saplBuilder = new StringBuilder();
		this.generateSapl(saplBuilder);
		return saplBuilder.toString();
	}

	@Override
	public void generateSapl(StringBuilder builder) {
		super.generateSapl(builder);
	}
}
