package ubiware.util.saplbuilder;

/**
 * Representation of a statement consisting of a Subject, Predicate and Object. (also known as Triple)
 * 
 * @author Michael Cochez for Ubiware project
 * @since 17 Aug 2010
 */
public class Statement {
	final Subject subject;
	final Predicate predicate;
	final Object object;

	/**
	 * Create a Statement with the given Subject, Predicate and Object.
	 * 
	 * @param subject
	 * @param predicate
	 * @param object
	 * @throws NullPointerException
	 *             If any of the components is null
	 */
	public Statement(Subject subject, Predicate predicate, Object object) {
		if ((subject == null) || (predicate == null) || (object == null)) {
			throw new NullPointerException("none of the elemnts of a statement can be null");
		}
		this.subject = subject;
		this.predicate = predicate;
		this.object = object;
	}

	/**
	 * Generate the S-APL representation of this statement.
	 * 
	 * @param builder
	 */
	void generateSapl(StringBuilder builder) {
		this.subject.generateSapl(builder);
		builder.append(' ');
		this.predicate.generateSapl(builder);
		builder.append(' ');
		this.object.generateSapl(builder);

	}

}
