package ubiware.util.saplbuilder;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * A collection to which Statements can be added.
 * 
 * @author Michael Cochez for Ubiware project
 * @since 17 Aug 2010
 */
public abstract class StatementContainer implements Iterable<Statement> {
	final List<Statement> statements;

	/**
	 * Create a StatementContainer with the specified initial capacity.
	 * 
	 * @param initialCapacity
	 */
	public StatementContainer(int initialCapacity) {
		this.statements = new ArrayList<Statement>(initialCapacity);
	}

	/**
	 * Same as {@link #addStatement(Statement)} with a newly created {@link Statement}
	 * 
	 * @param subject
	 * @param predicate
	 * @param object
	 */
	public void addStatement(Subject subject, Predicate predicate, Object object) {
		this.addStatement(new Statement(subject, predicate, object));
	}

	/**
	 * Add a statement to the collection.
	 * 
	 * @param statement
	 */
	public void addStatement(Statement statement) {
		this.statements.add(statement);
	}

		/**
		 * Add all statements in from the statement container.
		 * 
		 * @param statement
		 *            Statement container whose content will be added to this
		 *            container
		 */
	public void addStatements(StatementContainer sc) {
		for (Statement statement : sc) {
			this.addStatement(statement);
		}
	}

	/**
	 * generate a s-apl representation for the statements in this collection.
	 * 
	 * @param builder
	 */
	void generateSapl(StringBuilder builder) {
		boolean generateDot = false;
		for (Statement statement : this.statements) {
			if (generateDot) {
				builder.append(' ');
				builder.append('.');
				builder.append(' ');
			}
			statement.generateSapl(builder);
			generateDot = true;
		}
	}

	@Override
	public Iterator<Statement> iterator() {
		return this.statements.iterator();
	}

}
