package ubiware.util.saplbuilder;

/**
 * Representation of a variable. A variable can be used as a Subject, Predicate and Object.
 * 
 * @author Michael Cochez for Ubiware project
 * @since 7 May 2011
 */
public class Variable implements Subject, Predicate, Object {

	private final String variableName;

	/**
	 * Create a variable. Only the name of the variable must be specified. Thus, do not include the '?' sign as first character. Variable
	 * must start with an ASCII letter or '_' and contain only ASCII letters, '_' and numbers
	 * 
	 * @param variableName
	 */
	public Variable(String variableName) {
		//		FIXME : include check for validity of variable name
		this.variableName = variableName;
	}

	@Override
	public void generateSapl(StringBuilder builder) {
		builder.append('?');
		builder.append(this.variableName);
	}

	/**
	 * Gives a String representation of this variable.
	 */
	@Override
	public String toString() {
		return '?' + this.variableName;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (this.variableName.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(java.lang.Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (this.getClass() != obj.getClass()) {
			return false;
		}
		Variable other = (Variable) obj;
		return this.variableName.equals(other.variableName);
	}

}
